@@ -17,3 +17,6 @@
 #undef HAVE_XINERAMA
 #undef XPCOM_SHARED_LIB
 #undef GCC_VERSION
+#undef ENABLE_NAUTILUS_VIEW
+#undef STRING_HEADER
+#undef GNOME_PIXMAPS_DIR
