@@ -17,7 +17,9 @@
  */
 
 #include "galeon.h"
-#include "misc.h"
+#include "misc_string.h"
+#include "misc_gui.h"
+#include "misc_general.h"
 #include "embed.h"
 #include "window.h"
 #include "bookmarks.h"
@@ -26,14 +28,17 @@
 #include "prefs.h"
 #include "context.h"
 #include "dialog.h"
-#include "filepicker.h"
+#include "gfilepicker.h"
 #include "autocompletion.h"
+#include "bookmarks_io.h"
+#include "glade.h"
 
 #include <string.h>
 #include <time.h>
 #include <libgnome/gnome-config.h>
 #include <libgnome/gnome-i18n.h>
 #include <libgnomeui/gnome-popup-menu.h>
+#include <libgnomeui/gnome-dialog-util.h>
 #include <libgnomeui/gnome-preferences.h>
 #include <gtk/gtktogglebutton.h>
 #include <gtk/gtkselection.h>
@@ -71,10 +76,11 @@ static void bookmarks_editor_set_pixmap (BookmarksEditorControls *controls,
 					 BookmarkItem *bookmark, 
 					 gboolean active);
 static void bookmarks_editor_set_times (BookmarksEditorControls *controls,
-					BookmarkItem *bookmark,
-					BookmarkItem *bookmarkreal);
+					BookmarkItem *bookmark);
 static void bookmarks_selection_prepare (GList *l,
 					 BookmarksEditorControls *controls);
+static void bookmarks_editor_copy_or_cut (BookmarksEditorControls *controls, 
+					  gboolean cut);
 static gint bookmarks_row_compare (GList *a, GList *b);
 void
 bookmarks_editor_key_press_event_cb (GtkWidget *widget,GdkEventKey *event,
@@ -105,6 +111,7 @@ bookmarks_find_update_prevnext_sensitivity (BookmarksFindDialog *dialog);
 #endif
 
 extern gboolean block_search_position;
+extern gboolean block_docking;
 						
 void
 bookmarks_editor_ctree_tree_select_row_cb (GtkCTree *ctree, GList *node,
@@ -163,8 +170,7 @@ bookmarks_editor_ctree_tree_select_row_cb (GtkCTree *ctree, GList *node,
 		bookmarks_editor_set_pixmap               (controls, b, FALSE);
 		break;
 	}
-	bookmarks_editor_set_times (controls, b, 
-				    bookmarks_find_real_bookmark (b));
+	bookmarks_editor_set_times (controls, b);
 	controls->lock_edit_controls = FALSE;
 	
 	/* update search stuff */
@@ -187,7 +193,7 @@ bookmarks_editor_ctree_tree_unselect_row_cb (GtkCTree *ctree, GList *node,
 	bookmarks_editor_set_create_context_menu  (controls, NULL, FALSE);
 	bookmarks_editor_set_notes                (controls, NULL, FALSE);
 	bookmarks_editor_set_pixmap               (controls, NULL, FALSE);
-	bookmarks_editor_set_times                (controls, NULL, NULL);
+	bookmarks_editor_set_times                (controls, NULL);
 
 	controls->lock_edit_controls = FALSE;
 }
@@ -197,39 +203,43 @@ bookmarks_editor_edited_cb (GtkEditable *editable,
 			    BookmarksEditorControls *controls)
 {
 	if (!controls->lock_edit_controls && controls->selection) {
-		BookmarkItem *b_real;
-		BookmarkItem *b_alias;
+		BookmarkItem *b = controls->selection;
 		gchar *name = NULL;
 		gchar *url = NULL;
+		gchar *smarturl = NULL;
 		gchar *nick = NULL;
 		gchar *pixmap_file = NULL;
 		gboolean create_toolbar = FALSE;
 		gboolean create_context_menu = FALSE;
 		gchar *notes = NULL;
 		gchar *utf8_str;
-		/* note that b_alias and b_real may be the same bookmark */
-		b_alias = controls->selection;
-		b_real = bookmarks_find_real_bookmark (controls->selection);
+
+		/* read and translate data from the widgets */
 		if (controls->name_entry)
 		{
 			gchar *tmp1, *tmp2;
 			tmp1 = gtk_editable_get_chars 
 				(GTK_EDITABLE (controls->name_entry), 0, -1);
-			tmp2 = escape_uline_accel (tmp1);
+			tmp2 = misc_string_escape_uline_accel (tmp1);
 			utf8_str = mozilla_locale_to_utf8 (tmp2);
 			g_free (tmp2);
 			g_free (tmp1);
-			name = utf8_str;
+			name = misc_string_strip_newline (utf8_str);
+			g_free (utf8_str);
 		}
 		if (controls->url_entry)
 			url = gtk_editable_get_chars 
 				(GTK_EDITABLE (controls->url_entry), 0, -1);
+		if (controls->smarturl_entry)
+			smarturl = gtk_editable_get_chars
+				(GTK_EDITABLE (controls->smarturl_entry),
+				 0, -1);
 		if (controls->nick_entry)
 		{
 			gchar *tmp1, *tmp2;
 			tmp1 = gtk_editable_get_chars 
 				(GTK_EDITABLE (controls->nick_entry), 0, -1);
-			tmp2 = escape_uline_accel (tmp1);
+			tmp2 = misc_string_escape_uline_accel (tmp1);
 			utf8_str = mozilla_locale_to_utf8 (tmp2);
 			g_free (tmp2);
 			g_free (tmp1);
@@ -252,99 +262,66 @@ bookmarks_editor_edited_cb (GtkEditable *editable,
 			gchar *tmp1, *tmp2;
 			tmp1 = gtk_editable_get_chars 
 				(GTK_EDITABLE (controls->notes_text), 0, -1);
-			tmp2 = escape_uline_accel (tmp1);
+			tmp2 = misc_string_escape_uline_accel (tmp1);
 			utf8_str = mozilla_locale_to_utf8 (tmp2);
 			g_free (tmp2);
 			g_free (tmp1);
 			notes = utf8_str;
 		}
-		switch (b_real->type) {
+
+		/* modify the bookmark */
+		
+		bookmarks_toolbars_check_update (b);
+
+		switch (b->type) {
 		case BM_SITE:
-			if (controls->name_entry) {
-				if (b_real->name) g_free (b_real->name);
-				b_real->name = name;
-			}
-			if (controls->url_entry) {
-				if (b_real->url) g_free (b_real->url);
-				b_real->url = url;
-				auto_completion_add_url (b_real->url);
-			}
-			if (controls->nick_entry) {
-				if (b_real->nick) g_free (b_real->nick);
-				b_real->nick = nick;
-			}
-			if (controls->create_context_menu_toggle) {
-				b_alias->create_context_menu 
-					= create_context_menu;
-			}
-			/* FIXME: remove duplication */
-			if (controls->pixmap_file_entry) {
-				if (strcmp (pixmap_file, b_real->pixmap_file)
-				    != 0)
-				{
-					if (b_real->pixmap_data)
-						g_free (b_real->pixmap_data);
-					b_real->pixmap_data = 
-						pixmap_data_new_from_file 
-								(pixmap_file,
-								 FALSE);
-				}
-				if (b_real->pixmap_file) 
-					g_free (b_real->pixmap_file);
-				b_real->pixmap_file = pixmap_file;
-			}
-			if (controls->notes_text) {
-				if (b_real->notes) g_free (b_real->notes);
-				b_real->notes = notes;
-			}
+			if (controls->name_entry) 
+				bookmark_set_name (b, name);
+			if (controls->url_entry)
+				bookmark_set_url (b, url);
+			if (controls->smarturl_entry) 
+				bookmark_set_smarturl (b, smarturl);
+			if (controls->nick_entry) 
+				bookmark_set_nick (b, nick);
+			if (controls->create_context_menu_toggle)
+				bookmark_set_create_context_menu 
+					(b, create_context_menu);
+			if (controls->pixmap_file_entry) 
+				bookmark_set_pixmap (b, pixmap_file);
+			if (controls->notes_text) 
+				bookmark_set_notes (b, notes);
 			break;
 		case BM_FOLDER:
 		case BM_AUTOBOOKMARKS:
-			if (controls->name_entry) {
-				if (b_real->name) g_free (b_real->name);
-				b_real->name = name;
-			}
-			if (url) g_free (url);
-			if (nick) g_free (nick);
-			if (controls->create_toolbar_toggle) {
-				bookmarks_toolbars_check_update (b_alias);
-				b_alias->create_toolbar = create_toolbar;
-			}
-			if (controls->create_context_menu_toggle) {
-				b_alias->create_context_menu = 
-					create_context_menu;
-			}
-			/* FIXME: remove duplication */
-			if (controls->pixmap_file_entry) {
-				if (strcmp (pixmap_file, b_real->pixmap_file) 
-				    != 0)
-				{
-					if (b_real->pixmap_data)
-						g_free (b_real->pixmap_data);
-					b_real->pixmap_data = 
-						pixmap_data_new_from_file 
-								  (pixmap_file,
-								   FALSE);
-				}
-				if (b_real->pixmap_file) 
-					g_free (b_real->pixmap_file);
-				b_real->pixmap_file = pixmap_file;
-			}
-			if (controls->notes_text) {
-				if (b_real->notes) g_free (b_real->notes);
-				b_real->notes = notes;
-			}
+			if (controls->name_entry) 
+				bookmark_set_name (b, name);
+			if (controls->create_toolbar_toggle)
+				bookmark_set_create_toolbar
+					(b, create_toolbar);
+			if (controls->create_context_menu_toggle)
+				bookmark_set_create_context_menu 
+					(b, create_context_menu);
+			if (controls->pixmap_file_entry) 
+				bookmark_set_pixmap (b, pixmap_file);
+			if (controls->notes_text) 
+				bookmark_set_notes (b, notes);
 			break;
 		default:
 			break;
 		}
-		
-		b_real->time_modified = b_alias->time_modified = time (NULL);
-		bookmarks_editor_set_times (controls, b_alias, b_real);
-		bookmarks_editor_update_tree_items (b_real);
-		bookmarks_update_alias (b_real);
+		bookmark_set_time_modified (b, time (NULL));
+		bookmarks_editor_set_times (controls, b);
 		bookmarks_dirty = TRUE;
-		bookmarks_toolbars_check_update (b_real);
+
+		bookmarks_toolbars_check_update (b);
+
+		/* free stuff */
+		g_free (name);
+		g_free (url);
+		g_free (smarturl);
+		g_free (nick);
+		g_free (pixmap_file);
+		g_free (notes);
 	}
 }
 
@@ -391,18 +368,64 @@ bookmarks_editor_delete_event_cb (GtkWidget *widget, GdkEvent *event,
 	return TRUE;
 }
 
+void
+remove_confirmation_check_toggled_cb (GtkWidget *toggle, 
+				      BookmarksEditorControls *controls) 
+{
+	gboolean toggled = gtk_toggle_button_get_active (GTK_TOGGLE_BUTTON 
+							(toggle));
+	if (toggled)
+	{
+		eel_gconf_set_integer (CONF_BOOKMARKS_REMOVE_CONFIRMATION_MIN,
+				       -1);
+	}
+	else
+	{
+		eel_gconf_set_integer (CONF_BOOKMARKS_REMOVE_CONFIRMATION_MIN,
+				       2);
+	}
+}
+
+void 
+remove_bookmarks_ok_clicked_cb (GtkButton *button,
+				BookmarksEditorControls *controls)
+{
+	BookmarkItem *fake_root = controls->root_bookmark;
+	GList *l = fake_root->list;
+	fake_root->list = NULL;
+
+	bookmarks_editor_freeze_all ();
+	bookmarks_remove_several (l);
+	bookmarks_editor_thaw_all ();
+
+	g_list_free (l);
+	bookmarks_editor_remove_bookmarks_window_destroy (controls);
+}
+
+void 
+remove_bookmarks_cancel_clicked_cb (GtkButton *button,
+				    BookmarksEditorControls *controls)
+{
+	bookmarks_editor_remove_bookmarks_window_destroy (controls);
+}
+
+gboolean
+remove_bookmarks_delete_event_cb (GtkWidget *widget, GdkEvent *event,
+				  BookmarksEditorControls *controls)
+{
+	bookmarks_editor_remove_bookmarks_window_destroy (controls);
+	return TRUE;
+}
+
 void 
 bookmarks_editor_remove_clicked_cb (GtkButton *button,
 				    BookmarksEditorControls *controls)
 {
-	GList *selection_list = NULL, *l;
+	GList *selection_list = NULL;
 	GtkCList *clist = GTK_CLIST(controls->ctree);
-	GtkCTree *ctree = GTK_CTREE(controls->ctree);
 	BookmarkItem *next_to_select = NULL;
-	gboolean autobookmarks_deleted = FALSE;
-	gboolean need_to_ask = FALSE;
 
-	selection_list = bookmarks_get_selections_sorted(clist);
+	selection_list = bookmarks_get_selections_sorted (clist);
 
 	if (controls->selection && !g_list_next(clist->selection) &&
 	    controls->selection->parent) { 
@@ -416,61 +439,13 @@ bookmarks_editor_remove_clicked_cb (GtkButton *button,
 	
 	bookmarks_selection_prepare (selection_list, controls);
 	
-	/* check if we are trying to remove any non empty folder 
-	   and bug the user to make sure he does want it */
-	for (l = selection_list; l != NULL; l = g_list_next (l)) {
-		BookmarkItem *b = l->data;
-		need_to_ask = need_to_ask || ((b->type == BM_FOLDER)
-					      && (b->list != NULL)
-					      && (b->alias_of == NULL));
-	}
-
-	if (need_to_ask) {
-		GtkWidget *parent = (controls->dialog &&
-				     GTK_IS_WINDOW (controls->dialog)) ? 
-			controls->dialog : controls->window->WMain;
-		if (!dialog_ask_question 
-		    (_("You are going to remove (at least) a non empty \n"
-		       "bookmark folder. All its bookmarks will be deleted.\n"
-		       "Do you want to continue?"), parent)) {
-			g_list_free (selection_list);
-			return;
-		}
-	}
-
-	gtk_clist_freeze (clist);
-	for (l = selection_list; l != NULL; l = g_list_next (l)) {
-		BookmarkItem *b = l->data;
-
-		if (b->type == BM_AUTOBOOKMARKS)
-			autobookmarks_deleted = TRUE;
-
-		/* TODO: when deleting a bookmark which has aliases, we should
-		   warn the user, or make one of the aliases the new real
-		   bookmark */
-		if (b->alias && !b->alias_of) {
-			BookmarkItem *alias;
-			for (alias = b->alias; alias; alias = alias->alias) {
-				if (alias == next_to_select) 
-					next_to_select = NULL;
-			}
-		}
-		
-		bookmarks_toolbars_check_update (b);
-		bookmarks_remove_recursively (b);
-		bookmarks_dirty = TRUE;
-        }
-	gtk_clist_thaw (clist);
-
-	if (autobookmarks_deleted)
-	{
-		autobookmarks_root = NULL;
-		eel_gconf_set_boolean (CONF_BOOKMARKS_AB_ENABLED, FALSE);
-	}
+	bookmarks_editor_remove_bookmarks_window (selection_list);
 
+	/* Not sure if this is useful now
 	if (next_to_select)
 		gtk_ctree_select (ctree, bookmarks_get_tree_item 
 				  (next_to_select, controls));
+	*/
 
 	g_list_free (selection_list);
 }
@@ -488,7 +463,7 @@ bookmarks_editor_move_up_clicked_cb (GtkButton *button,
 	selections_sorted = bookmarks_get_selections_sorted(clist);
 	selection = selections_sorted;
 
-	gtk_clist_freeze(clist);
+	bookmarks_editor_freeze_all ();
 	while (selection)
 	{
 		b = gtk_ctree_node_get_row_data
@@ -506,7 +481,7 @@ bookmarks_editor_move_up_clicked_cb (GtkButton *button,
 		bookmarks_dirty = TRUE;
 		selection = selection->next;
 	}
-	gtk_clist_thaw (clist);
+	bookmarks_editor_thaw_all ();
 	
 	g_list_free (selections_sorted);
 }
@@ -524,7 +499,7 @@ bookmarks_editor_move_down_clicked_cb (GtkButton *button,
 	selections_sorted = bookmarks_get_selections_sorted (clist);
 	selection = selections_sorted = g_list_reverse (selections_sorted);
 
-	gtk_clist_freeze (clist);
+	bookmarks_editor_freeze_all ();
 	while (selection)
 	{
 		b = gtk_ctree_node_get_row_data
@@ -542,7 +517,7 @@ bookmarks_editor_move_down_clicked_cb (GtkButton *button,
 		bookmarks_dirty = TRUE;
 		selection = selection->next;
 	}
-	gtk_clist_thaw (clist);
+	bookmarks_editor_thaw_all ();
 	
 	g_list_free (selections_sorted);
 }
@@ -592,7 +567,8 @@ void fetch_icon_recursively (BookmarkItem *b,
 		break;
 		
 	case BM_SITE:
-		favicon_get_siteicon (NULL, b, controls);
+		favicon_download (NULL, b, FAVICON_DOWNLOAD_HOST_ICON |
+				           FAVICON_DOWNLOAD_OVERWRITE);
 		break;
 
 	case BM_SEPARATOR:
@@ -604,35 +580,37 @@ void
 bookmarks_editor_edit_toggled_cb (GtkWidget *toggle, 
 				  BookmarksEditorControls *controls) 
 {
-	if (controls->edit_frame != NULL) {
-		gboolean visible = TRUE;
-		if (GTK_IS_TOGGLE_BUTTON (toggle))
+	gboolean visible = TRUE;
+	
+	if (block_docking) return;
+	
+	g_return_if_fail (controls->edit_frame != NULL);
+	
+	if (GTK_IS_TOGGLE_BUTTON (toggle))
+	{
+		visible = gtk_toggle_button_get_active 
+				(GTK_TOGGLE_BUTTON (toggle));
+		if (controls->edit_menuitem != NULL)
 		{
-			visible = gtk_toggle_button_get_active 
-					(GTK_TOGGLE_BUTTON (toggle));
-			if (controls->edit_menuitem != NULL)
-			{
-				gtk_check_menu_item_set_active 
-					(GTK_CHECK_MENU_ITEM (
-						controls->edit_menuitem),
-					 visible);
-			}
+			gtk_check_menu_item_set_active 
+				(GTK_CHECK_MENU_ITEM (controls->edit_menuitem),
+				 visible);
 		}
-		else if (GTK_IS_CHECK_MENU_ITEM (toggle))
-		{
-			visible = GTK_CHECK_MENU_ITEM (toggle)->active;
-			if (controls->edit_button != NULL)
-			{
-				gtk_toggle_button_set_active 
-				     (GTK_TOGGLE_BUTTON (controls->edit_button),
-				      visible);
-			}
-		}
-		if (visible)
-			gtk_widget_show (controls->edit_frame);
-		else
-			gtk_widget_hide (controls->edit_frame);
 	}
+	else if (GTK_IS_CHECK_MENU_ITEM (toggle))
+	{
+		visible = GTK_CHECK_MENU_ITEM (toggle)->active;
+		if (controls->edit_button != NULL)
+		{
+			gtk_toggle_button_set_active 
+				(GTK_TOGGLE_BUTTON (controls->edit_button),
+				 visible);
+		}
+	}
+	if (visible)
+		gtk_widget_show (controls->edit_frame);
+	else
+		gtk_widget_hide (controls->edit_frame);
 }
 
 void
@@ -729,6 +707,37 @@ bookmarks_editor_new_folder_clicked_cb (GtkButton *button,
 	}
 }
 
+void 
+bookmarks_editor_new_autobm_clicked_cb (GtkButton *button,
+					BookmarksEditorControls *controls) 
+{
+	BookmarkItem *new = bookmarks_new_bookmark 
+		(BM_AUTOBOOKMARKS, TRUE, _("AutoBookmarks"), NULL,
+		 NULL, NULL, NULL);
+	BookmarkItem *near = controls->selection;
+	GtkCListDragPos insert_pos = GTK_CLIST_DRAG_INTO;
+
+	new->time_added = time (NULL);
+
+	if (!near) near = controls->root_bookmark;
+	bookmarks_insert_bookmark (new, near, insert_pos);
+	bookmarks_editor_place_tree_items (new);
+	bookmarks_editor_select_bookmark (controls, new);
+
+	/* Don't grab the focus if we're in the temp bookmarks window and the
+	   edit frame isn't visible */
+	if (controls->edit_frame)
+	{
+		if (!GTK_WIDGET_VISIBLE (controls->edit_frame))
+			return;
+	}
+	if (controls->name_entry)
+	{
+		gtk_widget_grab_focus (controls->name_entry);
+	}
+	autobookmarks_generate ();
+}
+
 void 
 bookmarks_editor_new_separator_clicked_cb (GtkButton *button,
 					   BookmarksEditorControls *controls) 
@@ -763,7 +772,8 @@ bookmarks_editor_handle_link_clicked (BookmarksEditorControls *controls,
 		embed = controls->window->active_embed;
 
 	bookmarks_set_visited (controls->selection);
-	if (strstr(controls->selection->url, "%s"))
+	if (controls->selection->smarturl &&
+	    strstr (controls->selection->smarturl, "%s"))
 	{
 		bookmarks_smart_bm_dialog (embed, &newembed,
 					   controls->selection, state);
@@ -813,7 +823,7 @@ bookmarks_editor_copy_clicked_cb (GtkButton *button,
 			bookmarks_create_copy_menu (controls->selection);
 		gnome_popup_menu_do_popup_modal
 			(GTK_WIDGET (destination_menu), 
-			 menu_position_under_widget, 
+			 misc_gui_menu_position_under_widget, 
 			 GTK_WIDGET (button), NULL, NULL);
 		gtk_object_unref (GTK_OBJECT (destination_menu));
 		bookmarks_dirty = TRUE;
@@ -915,6 +925,8 @@ bookmarks_editor_ctree_button_press_event_cb (GtkWidget *widget,
 		if (BOOKMARK_ITEM_IS_FOLDER (bi))
 		{
 			popup = gtk_menu_new ();
+			gtk_object_set_data (GTK_OBJECT(popup), "in_editor", 
+					     GINT_TO_POINTER (TRUE));
 			bookmarks_create_edit_items (GTK_MENU (popup),
 						     controls);
 			context_menu_add_seperator (GTK_MENU (popup));
@@ -924,6 +936,8 @@ bookmarks_editor_ctree_button_press_event_cb (GtkWidget *widget,
 		else if (bi->type == BM_SITE)
 		{
 			popup = gtk_menu_new ();
+			gtk_object_set_data (GTK_OBJECT(popup), "in_editor", 
+					     GINT_TO_POINTER (TRUE));
 			bookmarks_create_edit_items (GTK_MENU (popup),
 						     controls);
 			context_menu_add_seperator (GTK_MENU (popup));
@@ -933,8 +947,17 @@ bookmarks_editor_ctree_button_press_event_cb (GtkWidget *widget,
 		if (popup)
 		{
 			/* make sure that this entry is selected */
-			gtk_ctree_select (GTK_CTREE (controls->ctree),
-				  bookmarks_get_tree_item (bi, controls));
+			GtkCTreeNode *binode = bookmarks_get_tree_item 
+				(bi, controls);
+			if (!g_list_find 
+			    (GTK_CLIST (controls->ctree)->selection, binode))
+			{
+				gtk_clist_unselect_all 
+					(GTK_CLIST (controls->ctree));
+				gtk_ctree_select (GTK_CTREE (controls->ctree),
+						  binode);
+			}	
+
 			/* warning: ugly code */
 			gtk_object_set_data(GTK_OBJECT(popup), "GaleonWindow",
 					    window);
@@ -955,8 +978,8 @@ bookmarks_editor_ctree_button_press_event_cb (GtkWidget *widget,
 				  bookmarks_get_tree_item (
 					  controls->last_pressed, controls));
 		bookmarks_editor_handle_link_clicked (controls, 
-				mouse_state_to_link_state (event->button,
-							event->state));
+				misc_general_mouse_state_to_link_state (
+						event->button, event->state));
 	}
 
 	return FALSE;
@@ -969,7 +992,7 @@ bookmarks_editor_ctree_key_press_event_cb (GtkWidget *widget,
 {
 	if (event->keyval == GDK_Return)
 		bookmarks_editor_handle_link_clicked (controls,
-			key_state_to_link_state (event->state));
+			misc_general_key_state_to_link_state (event->state));
 	else if (event->state == 0)
 	{
 		if (event->keyval == GDK_Delete ||
@@ -1017,14 +1040,14 @@ bookmarks_editor_expand_all_cb (GtkButton *button,
 				BookmarksEditorControls *controls) 
 {
 	GList *l;
-	gtk_clist_freeze (GTK_CLIST (controls->ctree));
+	bookmarks_editor_freeze_all ();
 	for (l = controls->root_bookmark->list; l != NULL; l = l->next) {
 		BookmarkItem *b = l->data;
 		gtk_ctree_expand_recursive 
 			(GTK_CTREE (controls->ctree), 
 			 bookmarks_get_tree_item (b, controls));
 	}
-	gtk_clist_thaw (GTK_CLIST (controls->ctree));
+	bookmarks_editor_thaw_all ();
 }
 
 void 
@@ -1032,14 +1055,14 @@ bookmarks_editor_collapse_all_clicked_cb (GtkButton *button,
 					  BookmarksEditorControls *controls)
 {
 	GList *l;
-	gtk_clist_freeze (GTK_CLIST (controls->ctree));
+	bookmarks_editor_freeze_all ();
 	for (l = controls->root_bookmark->list; l != NULL; l = l->next) {
 		BookmarkItem *b = l->data;
 		gtk_ctree_collapse_recursive
 			(GTK_CTREE (controls->ctree), 
 			 bookmarks_get_tree_item (b, controls));
 	}
-	gtk_clist_thaw (GTK_CLIST (controls->ctree));
+	bookmarks_editor_thaw_all ();
 }
 
 /**
@@ -1070,7 +1093,7 @@ bookmarks_editor_set_name (BookmarksEditorControls *controls,
 	gchar *loc_str = NULL;
 
 	if (bookmark && bookmark->name) {
-		tmp = strip_uline_accel (bookmark->name);
+		tmp = misc_string_strip_uline_accel (bookmark->name);
 		loc_str = mozilla_utf8_to_locale (tmp);
 	}
 
@@ -1091,6 +1114,9 @@ bookmarks_editor_set_url (BookmarksEditorControls *controls,
 	bookmarks_editor_set_text (controls->url_entry, 
 				   bookmark ? bookmark->url : NULL, 
 				   active);
+	bookmarks_editor_set_text (controls->smarturl_entry,
+				   bookmark ? bookmark->smarturl : NULL,
+				   active);
 }
 
 /**
@@ -1198,20 +1224,25 @@ bookmarks_editor_set_pixmap (BookmarksEditorControls *controls,
 				   active);
 }
 
+/* why does it need to look for the real bookmark? FIXME, VERIFY */
 static void 
 bookmarks_editor_set_times (BookmarksEditorControls *controls,
-			    BookmarkItem *bookmark, BookmarkItem *bookmarkreal)
+			    BookmarkItem *bookmark)
 {
 	gchar *text;
+	BookmarkItem *bookmarkreal = NULL;
+	if (bookmark)
+		bookmarkreal = bookmarks_find_real_bookmark (bookmark);
 	if (controls->time_added_entry) {
-		text = bookmark ? time_to_string (bookmark->time_added) 
+		text = bookmark ? misc_string_time_to_string (
+						bookmark->time_added) 
 			: g_strdup ("");
 		gtk_entry_set_text (GTK_ENTRY (controls->time_added_entry),
 				    text);
 		g_free (text);
 	}
 	if (controls->time_modified_entry) {
-		text = bookmarkreal ? time_to_string
+		text = bookmarkreal ? misc_string_time_to_string
 			(bookmarkreal->time_modified) : g_strdup ("");
 		gtk_entry_set_text (GTK_ENTRY (controls->time_modified_entry),
 				    text);
@@ -1219,7 +1250,8 @@ bookmarks_editor_set_times (BookmarksEditorControls *controls,
 	}
 	if (controls->time_visited_entry) {
 		text = bookmarkreal 
-			? time_to_string (bookmarkreal->time_visited) 
+			? misc_string_time_to_string (
+						bookmarkreal->time_visited) 
 			: g_strdup ("");
 		gtk_entry_set_text (GTK_ENTRY (controls->time_visited_entry),
 				    text);
@@ -1252,13 +1284,19 @@ void
 bookmarks_editor_cut_cb (GtkButton *button,
 			 BookmarksEditorControls *controls) 
 {
-	bookmarks_editor_copy_cb (button, controls);
-	bookmarks_editor_remove_clicked_cb (button, controls);
+	bookmarks_editor_copy_or_cut (controls, TRUE);
 }
 
 void
 bookmarks_editor_copy_cb (GtkButton *button,
 			  BookmarksEditorControls *controls) 
+{
+	bookmarks_editor_copy_or_cut (controls, FALSE);
+}
+
+static void
+bookmarks_editor_copy_or_cut (BookmarksEditorControls *controls, 
+			      gboolean cut)
 {
 	if (controls->selection) {
 		BookmarkItem *dummy = bookmarks_new_bookmark 
@@ -1276,6 +1314,8 @@ bookmarks_editor_copy_cb (GtkButton *button,
 					 GDK_CURRENT_TIME);
 		dummy->list = NULL;
 		bookmarks_free_bookmark (dummy);
+		if (cut)
+			bookmarks_remove_several (selection_list);
 		g_list_free (selection_list);
 	}
 }
@@ -1338,8 +1378,7 @@ bookmarks_editor_selection_received_cb (GtkWidget *widget,
 		GList *li;
 		if (!near) near = controls->root_bookmark;
 		
-		gtk_clist_freeze (GTK_CLIST(controls->ctree));
-		gtk_clist_unselect_all (GTK_CLIST(controls->ctree));
+		bookmarks_editor_freeze_all ();
 		
 		/* b is the dummy bookmark, we have to add all it's children */
 		for (li = b->list; li; li = li->next)
@@ -1348,13 +1387,12 @@ bookmarks_editor_selection_received_cb (GtkWidget *widget,
 			new->time_modified = time (NULL);
 			bookmarks_insert_bookmark (new, near, insert_pos);
 			bookmarks_editor_place_tree_items (new);
-			bookmarks_editor_select_bookmark (controls, new);
 			near = new;
 			insert_pos = GTK_CLIST_DRAG_AFTER;
 		}
 		bookmarks_free_bookmark (b);
 		bookmarks_dirty = TRUE;
-		gtk_clist_thaw (GTK_CLIST(controls->ctree));
+		bookmarks_editor_thaw_all ();
 	}
 }
 
@@ -1387,10 +1425,10 @@ bookmarks_dock_detach_button_clicked_cb (GtkButton *button,
 	return_if_not_window (window);
 	bookmarks_editor_apply (controls);
 
-	gtk_clist_freeze (GTK_CLIST(controls->ctree));
+	bookmarks_editor_freeze_all ();
 	bookmarks_editor_remove_tree_item (controls, 
 			                   controls->root_bookmark);
-	gtk_clist_thaw (GTK_CLIST(controls->ctree));
+	bookmarks_editor_thaw_all ();
 
 	bookmarks_editors = g_list_remove (bookmarks_editors, controls);
 	eel_gconf_set_boolean (CONF_STATE_BOOKMARKS_DOCK_ENABLE, FALSE);
@@ -1404,14 +1442,23 @@ bookmarks_editor_show_attach_cb (GtkWidget *toggle,
 {
 	GaleonWindow *window;
 
+	if (block_docking) return;
+
 	window = controls->window;
 	return_if_not_window (window);
 
-	eel_gconf_set_boolean (CONF_STATE_BOOKMARKS_DOCK_ENABLE, TRUE);
-	if (controls->type != DOCKED_BOOKMARKS_EDITOR) {
+	if (controls->type == STANDARD_BOOKMARKS_EDITOR)
+	{
+		eel_gconf_set_boolean (CONF_STATE_BOOKMARKS_DOCK_ENABLE, TRUE);
 		bookmarks_editor_hide_dialog (controls);
 		bookmarks_editor_show_dock (window);
 	}
+	else
+	{
+		eel_gconf_set_boolean (CONF_STATE_BOOKMARKS_DOCK_ENABLE, FALSE);
+		bookmarks_editor_hide_dialog (controls);
+		bookmarks_editor_show_dialog (window);
+	}
 }
 
 void
@@ -1472,6 +1519,9 @@ bookmarks_editor_toolbar_appearance_menu_cb (GtkRadioMenuItem *menuitem,
 			(CONF_TOOLBAR_BOOKMARKS_EDITOR_STYLE,
 			 index);
 		break;
+	case REMOVE_BOOKMARKS_EDITOR:
+		/* no toolbar in that case */
+		break;
 	}
 	
 	gtk_widget_queue_resize (controls->dialog);
@@ -1490,13 +1540,6 @@ bookmarks_editor_toolbar_button_press_event_cb
 	{
 		GtkMenu *popup = GTK_MENU (gtk_menu_new ());
 	
-		if (gnome_preferences_get_menus_have_tearoff ())
-		{
-			GtkWidget *tearoff = gtk_tearoff_menu_item_new ();
-			gtk_menu_prepend (popup, tearoff);
-			gtk_widget_show (tearoff);
-		}
-
 		switch (controls->type)
 		{
 		case DOCKED_BOOKMARKS_EDITOR:
@@ -1507,6 +1550,9 @@ bookmarks_editor_toolbar_button_press_event_cb
 			style = eel_gconf_get_integer
 				 (CONF_TOOLBAR_BOOKMARKS_EDITOR_STYLE);
 			break;
+		case REMOVE_BOOKMARKS_EDITOR:
+			/* no toolbar in that case */
+			break;
 		}
 	
 		context_menu_add_radio_item 
@@ -1537,14 +1583,26 @@ bookmarks_editor_save_as_clicked_cb (GtkWidget *widget,
 				     BookmarksEditorControls *controls)
 {
 	gchar *file = NULL;
+	gint format_index;
+	FileFormat *file_formats = bookmarks_io_get_fileformats ();
 
 	if (show_file_picker (controls->dialog, _("Save as..."),
 			      g_get_home_dir(), NULL, modeSave,
-			      &file, NULL)) 
+			      &file, NULL, file_formats, &format_index)) 
 		if (dialog_check_overwrite_file (file, NULL)) {
-			bookmarks_save_as (file);
+			gboolean ok;
+			BookmarkTranslator *selected;
+			selected = g_list_nth 
+				(bookmarks_translators, format_index)->data;
+			ok = selected->save_to_file 
+				(bookmarks_root, default_bookmarks_root, file);
+			if (!ok)
+				gnome_error_dialog 
+					(_("Couldn't save your bookmarks to the "
+					   "specified file!"));
 		}
 	g_free (file);
+	g_free (file_formats);
 }
 
 void
@@ -1552,26 +1610,89 @@ bookmarks_editor_open_clicked_cb (GtkWidget *widget,
 				       BookmarksEditorControls *controls)
 {
 	gchar *file = NULL;
+	gint format_index;
+	FileFormat *file_formats = bookmarks_io_get_fileformats ();
 
 	if (show_file_picker (controls->dialog, _("Load from..."),
 			      g_get_home_dir(), NULL, modeOpen,
-			      &file, NULL)) {
+			      &file, NULL, file_formats, &format_index)) 
+	{
 		gchar *question = g_strdup_printf
 			(_("Your current set of bookmarks will be replaced "
 			   "with the contents of %s.\n"
 			   "Do you want to continue?"), file);
 		if (dialog_ask_question (question, controls->dialog)) {
-			bookmarks_set_root (bookmarks_load_from (file));
+			BookmarkItem *new_root;
+			BookmarkTranslator *selected;
+			selected = g_list_nth 
+				(bookmarks_translators, format_index)->data;
+			default_bookmarks_root = NULL;
+			new_root = selected->load_from_file 
+				(file, &default_bookmarks_root);
+			if (new_root)
+				bookmarks_set_root (new_root);
+			else 
+			{
+				gnome_error_dialog 
+					(_("Couldn't load any bookmark from that file.\n"
+					   "Make sure that it is in the correct file format."));
+			}
 		}
 		g_free (question);
 	}
 	g_free (file);
+	g_free (file_formats);
+
 	bookmarks_dirty = TRUE;
 	bookmarks_tb_dirty = TRUE;
 	/* refresh everywhere else */
 	bookmarks_editor_apply (controls);
 }
 
+void
+bookmarks_editor_import_clicked_cb (GtkWidget *widget, 
+				    BookmarksEditorControls *controls)
+{
+	gchar *file = NULL;
+	gint format_index;
+	FileFormat *file_formats = bookmarks_io_get_fileformats ();
+
+	if (show_file_picker (controls->dialog, _("Import from..."),
+			      g_get_home_dir(), NULL, modeOpen,
+			      &file, NULL, file_formats, &format_index)) 
+	{
+		gchar *question = g_strdup_printf
+			(_("Your current set of bookmarks will be merged "
+			   "with the contents of %s.\n"
+			   "Do you want to continue?"), file);
+		if (dialog_ask_question (question, controls->dialog)) {
+			BookmarkItem *new_root;
+			BookmarkTranslator *selected;
+			selected = g_list_nth 
+				(bookmarks_translators, format_index)->data;
+			new_root = selected->load_from_file (file, NULL);
+			if (new_root)
+				bookmarks_merge_trees (bookmarks_root, 
+						       new_root);
+			else
+			{
+				gnome_error_dialog 
+					(_("Couldn't load any bookmark from that file.\n"
+					   "Make sure that it is in the correct file format."));
+			}
+		}
+		g_free (question);
+	}
+	g_free (file);
+	g_free (file_formats);
+
+	bookmarks_dirty = TRUE;
+	bookmarks_tb_dirty = TRUE;
+	/* refresh everywhere else */
+	bookmarks_editors_refresh ();
+	bookmarks_editor_apply (controls);
+}
+
 void
 bookmarks_editor_sort_norec_clicked_cb (GtkWidget *widget, 
 					BookmarksEditorControls *controls)
@@ -1804,7 +1925,7 @@ bookmarks_editor_find_bookmarks_cb (GtkWidget *widget,
 	if (controls->type == STANDARD_BOOKMARKS_EDITOR)
 		dialog_set_parent (dialog->dialog, controls->dialog);
 	else
-		dialog_set_parent (dialog->dialog, controls->window->WMain);
+		dialog_set_parent (dialog->dialog, controls->window->wmain);
 
 	/* when docked, make docked search entry unsensitive */
 	/* if (controls->search_entry)
