@@ -17,11 +17,11 @@
  */
 
 #include "galeon.h"
-#include "bookmarks.h"
-#include "misc.h"
+#include "bookmarks_editor.h"
 #include "mozilla.h"
 
 #include <string.h>
+#include <libgnome/libgnome.h>
 #include <libgnome/gnome-i18n.h>
 #include <gtk/gtkselection.h>
 #include <gtk/gtkdnd.h>
