@@ -17,16 +17,19 @@
  */
 
 #include "galeon.h"
-#include "misc.h"
+#include "misc_gui.h"
+#include "misc_string.h"
 #include "embed.h"
 #include "bookmarks.h"
 #include "prefs.h"
 #include "context.h"
 #include "dialog.h"
+#include "favicon.h"
 
 #include <string.h>
 #include <libgnomeui/gnome-preferences.h>
 #include <libgnomeui/gnome-stock.h>
+#include <libgnomeui/gnome-dialog.h>
 #include <libgnome/gnome-config.h>
 #include <libgnome/gnome-i18n.h>
 #include <libgnomeui/gtkpixmapmenuitem.h>
@@ -73,6 +76,8 @@ bookmarks_menu_create (GaleonWindow *window)
 
 	return_if_not_window (window);
 
+	bookmarks_block_accel_update++;
+	
 	/* free dummywidgets */
 	if (window->dummy_widgets) 
 	{
@@ -89,11 +94,12 @@ bookmarks_menu_create (GaleonWindow *window)
 	menu = (GtkMenu *) window->bookmarks_menu;
 	add_menuitem = (GtkMenuItem *) window->add_bookmark;
 
-	window->bookmarks_tooltips = gtk_tooltips_new ();
 	bookmarks_create_file_submenu (add_menuitem, bookmarks_root);
 	bookmarks_menu_create_recursively (bookmarks_root, menu,
 		add_menuitem, GTK_TOOLTIPS(window->bookmarks_tooltips), 
 		TRUE, FALSE, FALSE);
+
+	bookmarks_block_accel_update--;
 }
 
 /**
@@ -112,6 +118,8 @@ bookmarks_menu_destroy (GaleonWindow *window)
 	GtkWidget *bookmarks_menu;
 	GtkWidget *separator;
 	GList *li, *l;
+
+	bookmarks_block_accel_update++;
 	
 	separator = window->bookmarks_separator;
 	bookmarks_menu = window->bookmarks_menu;
@@ -135,14 +143,9 @@ bookmarks_menu_destroy (GaleonWindow *window)
 				GTK_WIDGET (li->data));
 	}
 
-	/* destroy old tooltip */
-	if (window->bookmarks_tooltips)
-	{
-		gtk_object_destroy (GTK_OBJECT (window->bookmarks_tooltips));
-		window->bookmarks_tooltips = NULL;
-	}
-
 	g_list_free (l);
+
+	bookmarks_block_accel_update--;
 }
 
 /**
@@ -166,8 +169,8 @@ bookmarks_menu_create_recursively (BookmarkItem *bookmark, GtkMenu *menu,
 {
 	GList *l; 
 	GtkWidget *mi;
-	ExtraItemsLocation opts_location = eel_gconf_get_integer
-				(CONF_BOOKMARKS_XITEMS);
+	BookmarksExtraItemsLocation opts_location = eel_gconf_get_integer
+		(CONF_BOOKMARKS_XITEMS);
 
 	g_return_if_fail (bookmark->type == BM_FOLDER || 
 			  bookmark->type == BM_AUTOBOOKMARKS);
@@ -227,7 +230,9 @@ bookmarks_menu_create_recursively (BookmarkItem *bookmark, GtkMenu *menu,
 			gtk_menu_set_accel_group (GTK_MENU (submenu),
 						  menu->accel_group);
 			loc_str = mozilla_utf8_to_locale (b->name);
-			gtk_menu_set_title (GTK_MENU (submenu), loc_str);
+			gtk_menu_set_title (GTK_MENU (submenu), 
+					    loc_str[0] != '\0' ? loc_str
+					    		       : b->url);
 			g_free (loc_str);
 			mi = bookmarks_menu_create_item (b, menu, lock_accels);
 			gtk_menu_item_set_submenu (GTK_MENU_ITEM(mi), submenu);
@@ -273,7 +278,8 @@ bookmarks_menu_create_recursively (BookmarkItem *bookmark, GtkMenu *menu,
 			gtk_signal_connect_after
 				(GTK_OBJECT (mi), "activate", 
 				 GTK_SIGNAL_FUNC (bookmarks_activate_cb), b);
-			if (tooltips)
+			if (tooltips &&
+			    eel_gconf_get_integer (CONF_BOOKMARKS_TOOLTIPS) == 0)
 				gtk_tooltips_set_tip
 					(tooltips, GTK_WIDGET (mi),
 					 b->url, NULL);
@@ -368,9 +374,11 @@ bookmarks_menu_create_recursively_cb (GtkMenuItem *mi, gpointer data)
 {
 	BookmarkItem *b = gtk_object_get_data (GTK_OBJECT (mi), "bookmark");
 	GtkWidget *submenu = gtk_object_get_data (GTK_OBJECT (mi), "submenu");
-	GtkTooltips *tooltips = gtk_object_get_data (GTK_OBJECT (mi), "tooltips");
+	GtkTooltips *tooltips = gtk_object_get_data (GTK_OBJECT (mi), 
+						     "tooltips");
 	gboolean options = 
-		GPOINTER_TO_INT (gtk_object_get_data (GTK_OBJECT (mi), "options"));
+		GPOINTER_TO_INT (gtk_object_get_data (GTK_OBJECT (mi), 
+						      "options"));
 	gboolean disable_tearoffs = 
 		GPOINTER_TO_INT (gtk_object_get_data (GTK_OBJECT (mi), 
 						      "disable_tearoffs"));
@@ -398,8 +406,8 @@ static void
 bookmarks_create_file_menu_cb (GtkMenuItem *mi, BookmarkItem *b)
 {
 	gtk_signal_disconnect_by_func (GTK_OBJECT (mi),  
-			    bookmarks_create_file_menu_cb,
-			    b);
+				       bookmarks_create_file_menu_cb,
+				       b);
 	bookmarks_block_accel_update++;
 	bookmarks_create_file_menu (mi, b);
 	bookmarks_block_accel_update--;
@@ -409,9 +417,11 @@ static gboolean
 bookmark_has_subcategories (BookmarkItem *b) 
 {
 	GList *l;
-	for (l = b->list; l; l = l->next) {
-		BookmarkItem *bi = l->data;
-		if ((bi->type == BM_FOLDER) || (bi->type == BM_AUTOBOOKMARKS))
+	if (!BOOKMARK_ITEM_IS_FOLDER (b)) 
+		return FALSE;
+	for (l = b->list; l; l = l->next) 
+	{
+		if (BOOKMARK_ITEM_IS_FOLDER (l->data))
 			return TRUE;
 	}
 	return FALSE;
@@ -424,7 +434,7 @@ bookmark_has_subcategories (BookmarkItem *b)
  **/
 void
 bookmarks_menu_create_extra_items (GtkMenu *menu, BookmarkItem *bookmark,
-				   ExtraItemsLocation opts_location)
+				   BookmarksExtraItemsLocation opts_location)
 {
 	GtkMenu *extra_menu;
 	GtkWidget *extra_menu_item = NULL;
@@ -434,23 +444,25 @@ bookmarks_menu_create_extra_items (GtkMenu *menu, BookmarkItem *bookmark,
 	if (opts_location == EXTRA_ITEM_LOCATION_CONTEXT)
 		return;
 
-	if (opts_location == EXTRA_ITEM_LOCATION_TOP_SUBMENU || 
-			opts_location == EXTRA_ITEM_LOCATION_BOTTOM_SUBMENU)
+	if (opts_location == EXTRA_ITEM_LOCATION_TOP_SUBMENU 
+	    || opts_location == EXTRA_ITEM_LOCATION_BOTTOM_SUBMENU)
 	{
 		gboolean disable_tearoffs = 
-			GPOINTER_TO_INT (gtk_object_get_data (GTK_OBJECT (menu),
-						      "disable_tearoffs"));
+			GPOINTER_TO_INT (gtk_object_get_data
+					 (GTK_OBJECT (menu),
+					  "disable_tearoffs"));
 
 		extra_menu = GTK_MENU (gtk_menu_new ());
 		extra_menu_item = gtk_menu_item_new_with_label ("");
-		label_set_accel_text (_("_Folder actions"), 
+		misc_gui_label_set_accel_text (_("_Folder actions"), 
 			GTK_BIN (extra_menu_item)->child, GTK_WIDGET (menu),
 			extra_menu_item);
 		gtk_menu_item_set_submenu (GTK_MENU_ITEM (extra_menu_item),
 				GTK_WIDGET (extra_menu));
 
-		if (gnome_preferences_get_menus_have_tearoff () &&
-		    !disable_tearoffs){
+		if (gnome_preferences_get_menus_have_tearoff () 
+		    && !disable_tearoffs)
+		{
 			GtkWidget *tearoff = gtk_tearoff_menu_item_new ();
 			gtk_menu_prepend (extra_menu, tearoff);
 			gtk_widget_show (tearoff);
@@ -481,6 +493,7 @@ bookmarks_menu_create_extra_items (GtkMenu *menu, BookmarkItem *bookmark,
 		gtk_widget_show (GTK_WIDGET (extra_menu_item));
 	}
 }
+
 /**
  * Creates the items that are useful in both the menus and the bm editor 
  **/
@@ -540,6 +553,12 @@ bookmarks_menu_create_base_extra_items (GtkMenu *menu, BookmarkItem *bookmark)
 			bookmarks_regenerate_autobookmarks_cb, bookmark,
 			NULL, TRUE);
 	}
+	context_menu_add_item (menu, _("Save folder as..."),
+			bookmarks_export_activate_cb, bookmark,
+			GNOME_STOCK_MENU_SAVE_AS, TRUE);
+	context_menu_add_item (menu, _("Insert bookmarks here..."),
+			bookmarks_import_activate_cb, bookmark,
+			NULL, TRUE);
 }
 
 /**
@@ -573,6 +592,9 @@ bookmarks_item_create_base_extra_items (GtkMenu *menu, BookmarkItem *bookmark)
 	context_menu_add_item (menu, _("Copy link location"),
 			bookmarks_copy_link_activate_cb, bookmark,
 			GNOME_STOCK_MENU_COPY, TRUE);
+	context_menu_add_item (menu, _("Save bookmark as..."),
+			bookmarks_export_activate_cb, bookmark,
+			GNOME_STOCK_MENU_SAVE_AS, TRUE);
 }
 
 /**
@@ -589,7 +611,7 @@ bookmarks_create_file_submenu (GtkMenuItem *file_menuitem,
 	gchar *stripped;
 
 	loc_str = mozilla_utf8_to_locale (bookmark->name);
-	stripped = strip_uline_accel (loc_str);
+	stripped = misc_string_strip_uline_accel (loc_str);
 	g_free (loc_str);
 
 	bookmarks_block_accel_update++;
@@ -730,46 +752,53 @@ static GtkWidget *
 bookmarks_menu_create_item (BookmarkItem *bm, GtkMenu *menu,
 			    gboolean lock_accels)
 {
-	gchar *loc_str;
 	GtkWidget *pixmap;
 	const PixmapData *bm_icon;
 	
 	GtkWidget *m = gtk_pixmap_menu_item_new ();
-	GtkWidget *l;
 	GtkWidget *hb = gtk_hbox_new (FALSE, 0);
 
 	g_return_val_if_fail (bm->name != NULL, NULL);
 
 	bookmarks_block_accel_update++;
 
-	if (bm->pixmap_data == NULL || bm->pixmap_data->pixmap == NULL)
+	if (!bm->pixmap_data || !bm->pixmap_data->pixmap)
 	{
-		loc_str = mozilla_utf8_to_locale (bm->name);
-		l = gtk_label_new ("");
-		if (strlen (loc_str) > BOOKMARKS_MENU_MAX_LENGTH)
-		{
-			gchar *s, *t;
+		GtkWidget *label;
+		gchar *locale_str;
 
-			s = g_strndup(loc_str, BOOKMARKS_MENU_MAX_LENGTH);
-			t = g_strconcat (s, "...", NULL);
-			label_set_accel_text (t, l, GTK_WIDGET (menu), m);
-			g_free (t);
-			g_free (s);
-		} 
+		label = gtk_label_new ("");
+		locale_str = mozilla_utf8_to_locale (bm->name);
+
+		/* if the string is empty, use the URL instead */
+		if (!locale_str || locale_str[0] == '\0')
+		{
+			gchar *url_str = misc_string_shorten (bm->url,
+						BOOKMARKS_MENU_MAX_LENGTH);
+			misc_gui_label_set_accel_text (url_str, label,
+						       GTK_WIDGET (menu), m);
+			g_free (url_str);
+		}
+		/* otherwise, use the locale-translated name,
+		 * shortening it if necessary */
 		else
 		{
-			label_set_accel_text
-				(loc_str, l, GTK_WIDGET (menu), m);
-		}
-		gtk_box_pack_start (GTK_BOX (hb), l, FALSE, FALSE, 2);
-		g_free (loc_str);
+			gchar *name_str = misc_string_shorten (locale_str,
+						BOOKMARKS_MENU_MAX_LENGTH);
+			misc_gui_label_set_accel_text (name_str, label,
+						       GTK_WIDGET (menu), m);
+			g_free (name_str);
+		} 
+
+		gtk_box_pack_start (GTK_BOX (hb), label, FALSE, FALSE, 0);
+		if (locale_str) g_free (locale_str);
 	}
 	else
 	{
 		GtkWidget *image = gtk_pixmap_new (bm->pixmap_data->pixmap,
 						   bm->pixmap_data->mask);
 		gtk_box_pack_start (GTK_BOX (hb), GTK_WIDGET (image),
-				    FALSE, FALSE, 2);
+				    FALSE, FALSE, 0);
 	}
 	
 	if (bm->type == BM_FOLDER || bm->type == BM_AUTOBOOKMARKS)
@@ -794,7 +823,7 @@ bookmarks_menu_create_item (BookmarkItem *bm, GtkMenu *menu,
 			 * function */
 			accel_label = gtk_label_new ("");
 			gtk_box_pack_end (GTK_BOX (hb), accel_label, FALSE,
-					  FALSE, 2);
+					  FALSE, 0);
 
 			/* set object data */
 			gtk_object_set_data (GTK_OBJECT (m), "accel_label",
@@ -808,9 +837,7 @@ bookmarks_menu_create_item (BookmarkItem *bm, GtkMenu *menu,
 			gtk_signal_connect (GTK_OBJECT (m), 
 					    "remove_accelerator",
 					    bookmarks_remove_accel_cb, bm);
-			gtk_signal_connect (GTK_OBJECT (m), "select",
-					    bookmarks_ensure_accel_cb, bm);
-		
+
 			/* add accelerator */
 			gtk_widget_add_accelerator (m, "activate",
 						    menu->accel_group, 
@@ -818,7 +845,7 @@ bookmarks_menu_create_item (BookmarkItem *bm, GtkMenu *menu,
 						    bm->accel_mods, 
 						    GTK_ACCEL_VISIBLE);
 			
-			str = string_from_accel_data 
+			str = misc_gui_string_from_accel_data 
 				(m, bm->accel_key, bm->accel_mods);
 			gtk_label_set_text (GTK_LABEL (accel_label), str);
 			g_free (str);
@@ -828,7 +855,13 @@ bookmarks_menu_create_item (BookmarkItem *bm, GtkMenu *menu,
 			gtk_widget_lock_accelerators (m);
 		}
 
-		bm_icon = bookmarks_get_siteicon (bm->url);
+		/* statusbar signals */
+		gtk_signal_connect (GTK_OBJECT (m), "select",
+				    bookmarks_select_cb, bm);
+		gtk_signal_connect (GTK_OBJECT (m), "deselect",
+				    bookmarks_deselect_cb, bm);
+
+		bm_icon = favicon_get_pixmap (bm->url);
 		pixmap = gtk_pixmap_new (bm_icon->pixmap, bm_icon->mask);
 	}
 
@@ -855,17 +888,20 @@ bookmarks_smart_bm_dialog (GaleonEmbed *embed, GaleonEmbed **newembed,
 {	
 	gchar *loc_str;
 	GtkWidget *dialog;
-	gchar *s = bi->url, *strippedname;
+	gchar *s = bi->smarturl, *strippedname;
 	int num_parameters = 0;
 	int dlg_status = 0;
 
-	loc_str = bi->name;
-	strippedname = strip_uline_accel (loc_str);
+	g_return_if_fail (embed != NULL);
+
+	loc_str = mozilla_utf8_to_locale (bi->name);
+	strippedname = misc_string_strip_uline_accel (loc_str);
+	g_free (loc_str);
 	dialog = gnome_dialog_new (strippedname,
 				   GNOME_STOCK_BUTTON_OK,
 				   GNOME_STOCK_BUTTON_CANCEL,
 				   NULL);
-	gnome_dialog_set_default (GNOME_DIALOG(dialog), 0);
+	gnome_dialog_set_default (GNOME_DIALOG (dialog), 0);
 	
 	if (bi->pixmap_data == NULL)
 	{
@@ -887,7 +923,7 @@ bookmarks_smart_bm_dialog (GaleonEmbed *embed, GaleonEmbed **newembed,
 				"galeon_browser");
 	
 	/* count %s in the URL */
-	s = bi->url;
+	s = bi->smarturl;
 	while ((s = strstr(s, "%s"))) {
 		++num_parameters;
 		++s;
@@ -929,25 +965,37 @@ bookmarks_smart_bm_dialog_single (GtkWidget *dialog, GaleonEmbed *embed,
 			    
 	gnome_dialog_editable_enters (GNOME_DIALOG(dialog),
 				      GTK_EDITABLE(entry));
+
+	/* set parent and level in case of fullscreen */
+	if (embed->parent_window)
+		dialog_set_parent (dialog, embed->parent_window->wmain);
+
 	gtk_widget_show_all (dialog);
 
-	/* set parent and level in case of fulscreeen */
-	dialog_set_parent (dialog, embed->parent_window->WMain);
-
-	if ((dlg_status = dialog_run (GNOME_DIALOG (dialog), FALSE)) == 0) {
-		s = strstr (bi->url, "%s");
+	if ((dlg_status = misc_gui_gnome_dialog_run (GNOME_DIALOG (dialog),
+						     FALSE)) == 0)
+	{
+		s = strstr (bi->smarturl, "%s");
 		g_return_val_if_fail (s, 0);
 
-		tmp = g_strndup (bi->url, s - bi->url);
 		text = gtk_entry_get_text (GTK_ENTRY (entry));
-		translated_text = bookmarks_translate_string (text);
-		result = g_strconcat (tmp, translated_text, s+2, NULL);
 
-		embed_activate_link (embed, newembed, result, state);
+		if (text != NULL && strlen (text) > 0)
+		{
+			tmp = g_strndup (bi->smarturl, s - bi->smarturl);
+			translated_text = bookmarks_encode_smb_string (text);
+			result = g_strconcat (tmp, translated_text, s+2, NULL);
 
-		g_free (tmp);
-		g_free (translated_text);
-		g_free (result);
+			embed_activate_link (embed, newembed, result, state);
+
+			g_free (tmp);
+			g_free (translated_text);
+			g_free (result);
+		}
+		else
+		{
+			embed_activate_link (embed, newembed, bi->url, state);
+		}
 	}
 	return dlg_status;
 }
@@ -962,7 +1010,7 @@ bookmarks_smart_bm_dialog_multi (GtkWidget *dialog, GaleonEmbed *embed,
 				 GaleonEmbed **newembed, BookmarkItem *bi,
 				 LinkState state)
 {
-	gchar *start, *point, *tmp, *result;
+	gchar *start, *point, *tmp, *result, *label_str;
 	int i, dlg_status;
 	GtkWidget *seperator, *entry, *label;
 	GPtrArray *entries;
@@ -973,38 +1021,50 @@ bookmarks_smart_bm_dialog_multi (GtkWidget *dialog, GaleonEmbed *embed,
 
 	entries = g_ptr_array_new();
 
-	start = bi->url;
-	while(1)
+	start = bi->smarturl;
+	while (1)
 	{
-		point = strstr(start, "%s");
-		if (point == start)
+		point = strstr (start, "%s");
+
+		if (point)
 		{
-			/* user-input part of URL */
+			tmp = g_strndup (start, point - start);
+
+			/* find the beginning of the variable name */
+			label_str = strrchr (tmp, '&');
+			if (!label_str) label_str = strrchr (tmp, '?');
+			if (label_str) label_str++;
+
+			/* skip the equals sign */
+			if (label_str &&
+			    label_str[strlen (label_str) - 1] == '=')
+				label_str[strlen (label_str) - 1] = '\0';
+
+			/* create and insert the label */
+			if (label_str) label = gtk_label_new (label_str);
+			else label = gtk_label_new (tmp);
+			gtk_box_pack_start (GTK_BOX (
+						GNOME_DIALOG (dialog)->vbox), 
+				    	    label, TRUE, TRUE, 0);
+
+			/* free the temp string */
+			g_free (tmp);
+
+			/* create and insert the entry */
 			entry = gtk_entry_new ();
-			gtk_box_pack_start
-					(GTK_BOX (GNOME_DIALOG (dialog)->vbox),
-					 entry, TRUE, TRUE, 0);
-			g_ptr_array_add(entries, (gpointer) entry);
+			gtk_box_pack_start (GTK_BOX (
+						GNOME_DIALOG (dialog)->vbox),
+					    entry, TRUE, TRUE, 0);
+			g_ptr_array_add (entries, (gpointer) entry);
+
+			/* move on */
 			start = point + 2;
-			continue;
 		}
-		if (point == NULL)
-			point = start + strlen(start);
-		if (point == start)
-			break;
-		
-		/* static part of URL */
-		tmp = g_strndup(start, point-start);
-		label = gtk_label_new (tmp);
-		g_free(tmp);
-		gtk_box_pack_start (GTK_BOX (GNOME_DIALOG (dialog)->vbox), 
-				    label, TRUE, TRUE, 0);
-		g_ptr_array_add(entries, (gpointer) label);
-		start = point;
+		else break;
 	}
 
 	/* set focus on the first entry */
-	for (i = 0; i < entries->len; ++i) {
+	for (i = 0; i < (gint)entries->len; ++i) {
 		if (GTK_IS_ENTRY (entries->pdata[i]))
 		{
 			gtk_widget_grab_focus (GTK_WIDGET (entries->pdata[i]));
@@ -1023,43 +1083,70 @@ bookmarks_smart_bm_dialog_multi (GtkWidget *dialog, GaleonEmbed *embed,
 		}
 	}
 
+	/* set parent and level in case of fullscreen */
+	if (embed->parent_window)
+		dialog_set_parent (dialog, embed->parent_window->wmain);
+
 	gtk_widget_show_all (dialog);
 
-	/* set parent and level in case of fulscreeen */
-	dialog_set_parent (dialog, embed->parent_window->WMain);
-
-	if ((dlg_status = dialog_run (GNOME_DIALOG (dialog), FALSE)) == 0)
+	/* if user selected OK... */
+	if ((dlg_status = misc_gui_gnome_dialog_run (GNOME_DIALOG (dialog),
+						     FALSE)) == 0)
 	{
-		/* user selected OK, construct URL and go there */
-		result = g_strnfill(0,0);
-		for (i = 0; i < entries->len; ++i)
-		{
-			gchar *text = NULL, *translated_text = NULL;
+		/* make blank string */
+		result = g_strnfill (0,0);
 
-			if (GTK_IS_ENTRY (entries->pdata[i]))
+		/* start at the beginning of the smart url, and at the
+		 * first entry */
+		start = bi->smarturl;
+		i = 0;
+
+		while (1)
+		{
+			point = strstr (start, "%s");
+
+			/* if we found a %s... */
+			if (point)
 			{
-				text = gtk_entry_get_text 
-					(GTK_ENTRY (entries->pdata[i]));
-				translated_text = bookmarks_translate_string
-						   (text);
-				tmp = g_strconcat (result, translated_text,
-						   NULL);
+				gchar *url_text, *text, *translated_text;
+
+				/* get the url text and the translated
+				 * user-inputted text */
+				url_text = g_strndup (start, point - start);
+				text = gtk_entry_get_text (GTK_ENTRY (
+							entries->pdata[i]));
+				translated_text = bookmarks_encode_smb_string (
+									text);
+				/* stick it all together */
+				tmp = g_strconcat (result, url_text,
+						   translated_text, NULL);
+
+				/* free strings and move on */
+				g_free (url_text);
 				g_free (translated_text);
+				start = point + 2;
+				i++;
 			}
+			/* otherwise, copy over the last bit of the string */
 			else
 			{
-				gtk_label_get 
-					(GTK_LABEL (entries->pdata[i]), &text);
-				tmp = g_strconcat (result, text, NULL);
+				tmp = g_strconcat (result, start, NULL);
 			}
+
+			/* free the old string and use the new */
 			g_free (result);
 			result = tmp;
+
+			/* if we're done, break out of the loop */
+			if (!point) break;
 		}
 
+		/* open the url and free the string */
 		embed_activate_link (embed, newembed, result, state);
-
-		g_free(result);
+		g_free (result);
 	}
+
+	/* free the entry pointers and return */
 	g_ptr_array_free (entries, FALSE);
 	return dlg_status;
 }
@@ -1074,10 +1161,12 @@ bookmarks_init_menu_accels_recursively (BookmarkItem *b, GaleonWindow *window)
 {
 	GList *item;
 	GtkWidget *dummy;
- 
+
 	if (b->alias_of)
 		return; /* the real one should have been added already */
 	
+	bookmarks_block_accel_update++;
+
 	switch (b->type)
 	{
 	case BM_FOLDER:
@@ -1097,6 +1186,7 @@ bookmarks_init_menu_accels_recursively (BookmarkItem *b, GaleonWindow *window)
 			window->dummy_widgets =
 				g_list_append (window->dummy_widgets, dummy);
 			gtk_widget_ref (dummy);
+			gtk_object_sink (GTK_OBJECT (dummy));
 			gtk_object_set_data 
 				(GTK_OBJECT (dummy), "GaleonWindow", 
 				 window);
@@ -1109,20 +1199,18 @@ bookmarks_init_menu_accels_recursively (BookmarkItem *b, GaleonWindow *window)
 			
 			gtk_widget_add_accelerator 
 				(dummy, "activate",
-				 GNOME_APP (window->WMain)->accel_group,
+				 GNOME_APP (window->wmain)->accel_group,
 				 b->accel_key, b->accel_mods,
 				 GTK_ACCEL_VISIBLE);
 
-			gtk_signal_connect (GTK_OBJECT (dummy), 
-					    "remove_accelerator",
-					    bookmarks_remove_dummy_accel_cb,
-					    b);
 		}
 		break;
 
 	case BM_SEPARATOR:
 		break;
 	}
+
+	bookmarks_block_accel_update--;
 }
 
 /**
@@ -1203,8 +1291,9 @@ bookmarks_menu_sync_accel_labels_item (GtkMenuItem *item)
 			/* nothing to update */
 			return;
 		
-		str = string_from_accel_data (GTK_WIDGET (item), bm->accel_key, 
-					      bm->accel_mods);
+		str = misc_gui_string_from_accel_data (GTK_WIDGET (item),
+						       bm->accel_key, 
+						       bm->accel_mods);
 		gtk_label_set_text (GTK_LABEL (l), str);
 		g_free (str);
 	
