@@ -20,7 +20,7 @@
 extern "C" {
 #endif
 
-void downloader_save_url (const gchar *url);
+void downloader_external_save_url (const gchar *url);
 void downloader_save_link (GaleonEmbed *embed, const gchar *url);
 
 #ifdef __cplusplus
