@@ -22,12 +22,15 @@
    Authors: Ramiro Estrugo <ramiro@eazel.com>
 */
 
+#include <stdlib.h>
 #include <config.h>
 #include "eel-gconf-extensions.h"
 
 #include <gconf/gconf-client.h>
 #include <gconf/gconf.h>
 #include <libgnome/gnome-i18n.h>
+#include <libgnomeui/gnome-dialog.h>
+#include <libgnomeui/gnome-dialog-util.h>
 
 static GConfClient *global_gconf_client = NULL;
 
@@ -50,12 +53,31 @@ eel_gconf_client_get_global (void)
 	if (!gconf_is_initialized ()) {
 		char *argv[] = { "eel-preferences", NULL };
 		GError *error = NULL;
+		char *teststr;
 		
 		if (!gconf_init (1, argv, &error)) {
 			if (eel_gconf_handle_error (&error)) {
 				return NULL;
 			}
 		}
+		
+		/* check if gconf schemas are working */
+		teststr = gconf_client_get_string (eel_gconf_client_get_global(),
+					  "/apps/galeon/gconf_test", NULL);
+		if (!teststr)
+		{	
+			GtkWidget *dialog;
+			dialog = gnome_error_dialog (_("Cannot find a schema for galeon preferences. \n"
+						       "Check your gconf setup, look at galeon FAQ for \n"
+						       "more info"));
+			gnome_dialog_run_and_close (GNOME_DIALOG (dialog));
+			exit (0);
+		}
+		else
+		{
+			g_free (teststr);
+		}
+
 	}
 	
 	if (global_gconf_client == NULL) {
@@ -69,25 +91,10 @@ eel_gconf_client_get_global (void)
 gboolean
 eel_gconf_handle_error (GError **error)
 {
-	char *message;
-	static gboolean shown_dialog = FALSE;
-	
 	g_return_val_if_fail (error != NULL, FALSE);
 
 	if (*error != NULL) {
 		g_warning (_("GConf error:\n  %s"), (*error)->message);
-		if (! shown_dialog) {
-			shown_dialog = TRUE;
-
-			message = g_strdup_printf (_("GConf error:\n  %s\n"
-						     "All further errors shown "
-						     "only on terminal"),
-						   (*error)->message);
-			//eel_show_error_dialog (message, _("GConf Error"), NULL);
-			/* FIXME : display an error message */
-			g_warning ("GConf Error: %s", message);
-			g_free (message);
-		}
 		g_error_free (*error);
 		*error = NULL;
 
@@ -442,102 +449,23 @@ eel_gconf_get_value (const char *key)
 	return value;
 }
 
-/*
-static gboolean
-simple_value_is_equal (const GConfValue *a,
-		       const GConfValue *b)
+void
+eel_gconf_set_value (const char *key, GConfValue *value)
 {
-	g_return_val_if_fail (a != NULL, FALSE);
-	g_return_val_if_fail (b != NULL, FALSE);
+	GConfClient *client;
+	GError *error = NULL;
 
-	switch (a->type) {
-	case GCONF_VALUE_STRING:
-		return eel_str_is_equal (a->d.string_data, b->d.string_data);
-		break;
+	g_return_if_fail (key != NULL);
 
-	case GCONF_VALUE_INT:
-		return a->d.int_data == b->d.int_data;
-		break;
+	client = eel_gconf_client_get_global ();
+	g_return_if_fail (client != NULL);
 
-	case GCONF_VALUE_FLOAT:
-		return a->d.float_data == b->d.float_data;
-		break;
-
-	case GCONF_VALUE_BOOL:
-		return a->d.bool_data == b->d.bool_data;
-		break;
-	default:
-		g_assert_not_reached ();
-	}
+	gconf_client_set (client, key, value, &error);
 	
-	return FALSE;
+	if (eel_gconf_handle_error (&error)) {
+		return;
+	}
 }
-*/
-
-/*
-gboolean
-eel_gconf_value_is_equal (const GConfValue *a,
-			       const GConfValue *b)
-{
-	GSList *node_a;
-	GSList *node_b;
-
-	if (a == NULL && b == NULL) {
-		return TRUE;
-	}
-
-	if (a == NULL || b == NULL) {
-		return FALSE;
-	}
-
-	if (a->type != b->type) {
-		return FALSE;
-	}
-
-	switch (a->type) {
-	case GCONF_VALUE_STRING:
-	case GCONF_VALUE_INT:
-	case GCONF_VALUE_FLOAT:
-	case GCONF_VALUE_BOOL:
-		return simple_value_is_equal (a, b);
-		break;
-		
-	case GCONF_VALUE_LIST:
-		if (a->d.list_data.type != b->d.list_data.type) {
-			return FALSE;
-		}
-
-		if (a->d.list_data.list == NULL && b->d.list_data.list == NULL) {
-			return TRUE;
-		}
-
-		if (a->d.list_data.list != NULL || b->d.list_data.list != NULL) {
-			return FALSE;
-		}
-
-		if (g_slist_length (a->d.list_data.list) != g_slist_length (b->d.list_data.list)) {
-			return FALSE;
-		}
-		
-		for (node_a = a->d.list_data.list,node_b = b->d.list_data.list;
-		     node_a != NULL && node_b != NULL;
-		     node_a = node_a->next, node_b = node_b->next) {
-			g_assert (node_a->data != NULL);
-			g_assert (node_b->data != NULL);
-			if (!simple_value_is_equal (node_a->data, node_b->data)) {
-				return FALSE;
-			}
-		}
-		
-		return TRUE;
-	default:
-		g_assert (0);
-	}
-	
-	g_assert_not_reached ();
-	return FALSE;
-}
-*/
 
 void
 eel_gconf_value_free (GConfValue *value)
