@@ -19,7 +19,7 @@
 #include "galeon.h"
 #include "mozilla.h"
 #include "embed.h"
-#include "misc.h"
+#include "misc_string.h"
 #include "history.h"
 #include "eel-gconf-extensions.h"
 
@@ -32,6 +32,23 @@
 #include <gnome-xml/parser.h>
 #include <libgnomevfs/gnome-vfs-mime.h>
 
+/**
+ * embed_selection_get_cb: get selection on copy link location
+ */
+void
+embed_selection_get_cb (GtkWidget *widget, GtkSelectionData *selection_data,
+			   guint info, guint time_stamp, gpointer data)
+{
+	gchar *text;
+
+	text = gtk_object_get_data (GTK_OBJECT(widget), "selection");
+
+	g_return_if_fail (text != NULL);
+
+	gtk_selection_data_set (selection_data, GDK_SELECTION_TYPE_STRING,
+				8, text, strlen (text));
+}
+
 /** 
  * embed_drag_data_received_cb:
  */
@@ -79,7 +96,7 @@ embed_drag_data_received_cb (GtkWidget *widget, GdkDragContext *context,
 		{
 			/* find the uri to open */
 			drop_uri = (gchar *)(l->data);
-			parse_uri = misc_parse_uri (drop_uri);
+			parse_uri = misc_string_parse_uri (drop_uri);
 			real_uri = (parse_uri == NULL ? drop_uri : parse_uri);
 
 			/* open in first or next embed */
@@ -89,8 +106,8 @@ embed_drag_data_received_cb (GtkWidget *widget, GdkDragContext *context,
 			}
 			else
 			{
-				embed_create_from_url (embed, real_uri,
-						       FALSE, !tabbed);
+				embed_create_after_embed (embed, !tabbed,
+					real_uri, EMBED_CREATE_RAISE_WINDOW);
 			}
 			
 			/* free allocated string */
@@ -121,85 +138,3 @@ embed_tab_close_button_clicked_cb (GtkWidget *button, GaleonEmbed *embed)
 	return_if_not_embed (embed);
 	embed_close (embed);
 }
-
-/**
- * save_site_location: used when switching to a new tab, save the old
- * (possibly edited) site location in the old embed and hack up the
- * selection to preserve it (if any).
- */
-void
-save_site_location (GaleonEmbed *embed)
-{
-	GaleonWindow *window;
-	guchar *text, *stored;
-	int start, end, tmp;
-
-	/* get window */
-	window = embed->parent_window;
-	return_if_not_window (window);
-
-	/* if no location entry then don't change anything */
-	if (window->location_entry == NULL)
-	{
-		return;
-	}
-
-	/* free old value */
-	if (embed->site_location != NULL)
-	{
-		g_free (embed->site_location);
-	}
-	
-	/* get location text */
-	embed->site_location = gtk_editable_get_chars 
-		(GTK_EDITABLE (window->location_entry), 0, -1);
-			
-	/* did we have the selection? */
-	if (!(GTK_EDITABLE (window->location_entry)->has_selection))
-	{
-		/* no, give up */
-		return;
-	}
-
-	/* get selected text */
-	start = GTK_EDITABLE (window->location_entry)->selection_start_pos;
-	end = GTK_EDITABLE (window->location_entry)->selection_end_pos;
-
-	/* switch them when they are in the opposite order,
-	   it happens if you select from right to left */
-	if (start > end)
-	{
-		tmp = start;
-		start = end;
-		end = tmp;
-	}
-
-	text = gtk_editable_get_chars 
-		(GTK_EDITABLE (window->location_entry), 
-		 start, end);
-
-	/* free up stored per-window selection */
-	stored = gtk_object_get_data (GTK_OBJECT (window->WMain), "selection");
-	gtk_object_remove_data (GTK_OBJECT (window->WMain), "selection");
-	g_free (stored);
-
-        /* store in parent window */
-	gtk_object_set_data (GTK_OBJECT (window->WMain), "selection", text);
-	
-	/* window takes ownership of primary selection */
-	gtk_selection_owner_set (window->WMain, GDK_SELECTION_PRIMARY,
-				 GDK_CURRENT_TIME);
-}
-
-void
-embed_change_stylesheet_cb (GtkWidget *mi, AlternateStyleSheet *css)
-{
-	GaleonEmbed *embed =
-		(GaleonEmbed*) gtk_object_get_data (GTK_OBJECT (mi), "embed");
-	gint index =
-	       GPOINTER_TO_INT (gtk_object_get_data (GTK_OBJECT (mi), "index"));
-	return_if_not_embed (embed);
-
-	history_set_selected_authorcss (embed->site_location, index);
-	mozilla_set_alternate_stylesheet (embed, css);
-}
