@@ -17,5 +17,5 @@
  */
 
 gboolean find_next (GaleonEmbed *embed, const gchar *string);
-void find_show_dialog (GaleonWindow *window);
-void find_destroy_dialog (GaleonWindow *window);
+void find_show_dialog (GaleonEmbed *embed);
+void find_destroy_dialog (GaleonEmbed *embed);
