@@ -50,6 +50,14 @@ extern int mozilla_version;
  */
 #include "prefs_strings.h"
 
+/**
+ * OAF IIDs and objects
+ */
+#define GALEON_AUTOMATION_IID     "OAFIID:GNOME_Galeon_Automation"
+#ifdef ENABLE_NAUTILUS_VIEW
+#define GALEON_NAUTILUS_VIEW_IID  "OAFIID:GNOME_Galeon_NautilusViewFactory"
+#endif
+
 /* the virtual URL of "My Portal" feature */
 #define MYPORTAL_URL          "myportal:"
 
@@ -60,6 +68,9 @@ extern int mozilla_version;
 #define GALEON_BUGREPORT_URL  ("http://bugzilla.gnome.org/enter_bug.cgi" \
                                "?product=galeon&version=" VERSION)
 
+/* the Galeon FAQ page */
+#define GALEON_FAQ_URL (SHARE_DIR "/FAQ.html")
+
 /* bookmark file locations - relative to home dir */
 #define NETSCAPE_BOOKMARKS    "/.netscape/bookmarks.html"
 #define MOZILLA_PREFS_DIR     "/.mozilla/"
@@ -71,6 +82,12 @@ extern int mozilla_version;
 /* Has xpcom been initialised? */
 extern gboolean pushed_startup;
 
+/* Are we exiting? We use this to reject CORBA requests that otherwise
+   would crash galeon. It's not enough to unregister from OAF because
+   we may have received already some requests that have not been yet 
+   processed */
+extern gboolean galeon_exiting;
+
 /* types of drag and drop for links */
 extern const GtkTargetEntry url_drag_types[];
 extern const GtkTargetEntry drop_types[];
@@ -80,26 +97,7 @@ extern const gint drop_types_num_items;
 extern const gint embed_drop_types_num_items;
 
 /** the global list of MimeItems */
-extern GList *mime_types; /* FIXME: can this be made local?  MattA */
-			  /* No: philipl :-) */
-
-/** The root of the bookmarks tree */
-extern BookmarkItem *bookmarks_root;
-
-/** The root of the temporary bookmark tree */
-extern BookmarkItem *default_bookmarks_root;
-
-/** The AutoBookmarks folder */
-extern BookmarkItem *autobookmarks_root;
-
-/** Do we need to save the bookmarks? */
-extern gboolean bookmarks_dirty;
-
-/** Do we need to recreate the bookmarks toolbars? */
-extern gboolean bookmarks_tb_dirty;
-
-/** The bookmarks editors list */
-extern GList *bookmarks_editors;
+extern GList *mime_types;
 
 /** Global windows count, includes download progress dialogs **/
 extern gint window_count;
@@ -121,12 +119,20 @@ extern GList *sorted_charset_titles;
 /* are we in server mode? */
 extern gboolean galeon_server_mode;
 
+/* server mode timeout */
+extern gint galeon_server_timeout;
+
 /* are we in panel mode? */
 extern gboolean galeon_panel_mode;
 
 /* last save image directory */
 extern gchar *save_dir;
 
+/* when true, the session won't be automatically saved when locations
+ * change or embeds are closed.  useful for functions that close a lot of
+ * embeds at once. */
+extern gboolean dont_autosave_session;
+
 /**
  * The standard pixmaps for bookmarks / sites
  */
@@ -135,7 +141,8 @@ extern PixmapData *folder_open_pixmap_data;
 extern PixmapData *default_folder_pixmap_data;
 extern PixmapData *default_folder_open_pixmap_data;
 extern PixmapData *site_pixmap_data;
-extern PixmapData *smart_bm_tray_pixmap_data;
+extern PixmapData *smart_bm_fold_pixmap_data;
+extern PixmapData *smart_bm_unfold_pixmap_data;
 
 #define DEFAULT_CHROME GTK_MOZ_EMBED_FLAG_ALLCHROME
 
@@ -146,10 +153,11 @@ extern const gint bookmarks_dnd_targets_num_items;
  * of their own.
  */
 
+/* main.c */
+void galeon_exit (gboolean return_from_main_loop,
+		  gboolean unset_crashed_flag);
+
 /* netscape.c */
-gint netscape_import_bookmarks (GString *filename, gboolean use_locale);
-gint netscape_export_bookmarks (gchar *filename, BookmarkItem *root,
-				gboolean use_locale);
 gboolean netscape_import_prefs (void);
 
 /* mozilla_prefs.c */
@@ -158,10 +166,6 @@ void mozilla_prefs_import_language (void);
 /* autobookmarks.c */
 void autobookmarks_generate (void);
 
-/* cookies.c */
-GtkWidget *pdm_show_manager (GaleonWindow *window);
-GtkWidget *pdm_show_page (GaleonWindow *window, PDMPage page);
-
 /* newuser.c */
 gboolean newuser_check (void);
 void newuser_show_druid (void); 
@@ -169,6 +173,9 @@ void newuser_show_druid (void);
 /* panel.c */
 void panel_main (void);
 
+/* ProgressListener2.cpp */
+void downloader_dialog_init (void);
+
 #ifdef __cplusplus
 }
 #endif
