@@ -1,5 +1,5 @@
 /*
- *  Copyright (C) 2000 Marco Pesenti Gritti
+ *  Copyright (C) 2000, 2001, 2002 Marco Pesenti Gritti
  *
  *  This program is free software; you can redistribute it and/or modify
  *  it under the terms of the GNU General Public License as published by
@@ -22,14 +22,17 @@
 #include "window.h"
 #include "bookmarks.h"
 #include "autocompletion.h"
-#include "misc.h"
 #include "state.h"
 #include "prefs.h"
 #include "glade.h"
 #include "dialog.h"
 #include "eel-gconf-extensions.h"
 #include "mozilla.h"
+#include "favicon.h"
+#include "misc_gui.h"
+#include "misc_string.h"
 
+#include <unistd.h>
 #include <string.h>
 #include <gtk/gtkoptionmenu.h>
 #include <gtk/gtkselection.h>
@@ -38,11 +41,15 @@
 #include <libgnomeui/gtkpixmapmenuitem.h>
 #include <libgnomeui/gnome-app.h>
 #include <libgnomeui/gnome-preferences.h>
+#include <libgnomeui/gnome-entry.h>
 #include <libgnome/gnome-util.h>
 #include <libgnome/gnome-config.h>
 #include <libgnome/gnome-i18n.h>
 #include <libgnomevfs/gnome-vfs.h>
 #include <libgnomevfs/gnome-vfs-mime.h>
+#include <gnome-xml/xmlmemory.h>
+#include <gnome-xml/tree.h>
+#include <gnome-xml/parser.h>
 
 #include <time.h>
 
@@ -70,6 +77,8 @@ static GList *history_views = NULL;
 
 static GList *history_notifiers_id;
 
+static HistoryItem *last_visited_site = NULL;
+
 /* sorting functions for each history column */
 static const GCompareFunc history_compare[5] =
 {
@@ -91,20 +100,15 @@ enum
 	SAX_VALUE = 1
 };
 
-/** Where does the recent menu start? */
-#define GO_RECENT_POS 7
-
 /* local function prototypes */
 static void history_load (void);
-static GtkWidget *history_create_recent_menu_item (HistoryItem *hi, gint num,
-						   GtkWidget *menu);
-static void history_add_recent_menu (GaleonWindow *window, HistoryItem *hi);
 static void history_make_host_strings (HistoryHost *hh, gchar **text);
 static void history_make_item_strings (HistoryItem *hi, gchar **text);
 static HistoryItem *history_add_item (gchar *url, gchar *title_utf8, 
 				      GTime first, GTime last, gint visits);
 static void history_look_for_newer (gpointer key, gpointer value, 
 				    gpointer user_data);
+static void history_update_last_visited_site (HistoryItem *hi);
 static gint calculate_age (GTime last);
 static HistoryView *history_build_view (GaleonWindow *window,
 					const char *xml_root);
@@ -142,11 +146,6 @@ static void history_drag_data_get_cb (GtkWidget *widget,
 				      GtkSelectionData *selection_data, 
 				      guint info, guint time, 
 				      HistoryView *hv);
-void 
-window_menu_recent_activate_cb (GtkMenuItem *menuitem, gpointer user_data);
-void 
-window_menu_recent_release_cb (GtkMenuItem *menuitem, GdkEventButton *event,
-			       gpointer user_data);
 static void history_notifiers_init(HistoryView *hv);
 
 /**
@@ -170,7 +169,11 @@ history_exit (void)
 {
 	/* save the history */
 	history_save ();
-
+	
+	/* clear the autocompletion history 
+	 * clearing it before removing the history items gives a big speedup */
+	auto_completion_clear ();
+	
 	/* destroy the views */
 	g_list_foreach (history_views, (GFunc)(history_destroy_view_structure),
 			NULL);
@@ -192,6 +195,11 @@ history_exit (void)
 	g_hash_table_thaw (history);
 	g_hash_table_destroy (history);
 	history = NULL;
+	last_visited_site = NULL;
+
+	/* restore the autocompletion for bookmarks */
+	bookmarks_add_completion_recursively (bookmarks_root);
+
 }
 
 /**
@@ -200,31 +208,50 @@ history_exit (void)
 void
 history_clear (void)
 {
-	extern GCompletion *URLCompletion;
+	GList *l;
 
+	/* clear the autocompletion history 
+	 * clearing it before removing the history items gives a big speedup */
+	auto_completion_clear ();
+	
 	/* destroy hosts hash table */
 	g_hash_table_freeze (history_hosts);
 	g_hash_table_foreach_remove (history_hosts, (GHRFunc) 
 				     history_item_free_cb, NULL);
 	g_hash_table_thaw (history_hosts);
 
-	/* destroy history hash table */
+	/* Destroy history hash table */
 	g_hash_table_freeze (history);
 	g_hash_table_foreach_remove (history, (GHRFunc)history_item_free_cb, 
 				     NULL);
 	g_hash_table_thaw (history);
 
-	/* clear the autocompletion history (FIXME refactor) */
-	g_completion_clear_items (URLCompletion);
+	/* restore the autocompletion for bookmarks */
+	bookmarks_add_completion_recursively (bookmarks_root);
 
 	/* destroy all views */
 	g_list_foreach (history_views, (GFunc)history_refresh_view, NULL);
 
-	/* clear the go_menu */
-	history_clear_recent_menu ();
+	/* no last_visited_site */
+	last_visited_site = NULL;
 
 	/* clear the file */
 	history_save ();
+	
+	/* clear location entry dropdown */
+	for (l = all_windows; l; l = g_list_next (l))
+	{
+		GaleonWindow *w = (GaleonWindow *) l->data;
+
+		if (w->location_entry != NULL)
+		{
+			GnomeEntry *e = GNOME_ENTRY (w->location_gnomeentry);
+			gnome_entry_set_max_saved (e, 0);
+			gnome_entry_save_history (e);
+			gnome_entry_load_history (e);
+			gnome_entry_set_max_saved (e, 10);
+		}
+	}
 }
 
 /**
@@ -248,7 +275,7 @@ history_show_dialog (GaleonWindow *window)
 	history_notifiers_init (hv);
 
 	/* show the widget */
-	dialog_set_parent (hv->view, window->WMain);
+	dialog_set_parent (hv->view, window->wmain);
 }
 
 /**
@@ -309,6 +336,9 @@ history_show_dock (GaleonWindow *window)
 void
 history_refresh_view (HistoryView *hv)
 {
+	/* deselect all */
+	gtk_clist_unselect_all (GTK_CLIST (hv->ctree));
+	
 	/* stop showing updates */
 	gtk_clist_freeze (GTK_CLIST (hv->ctree));
 
@@ -418,8 +448,8 @@ history_visited (const gchar *url)
                 hi->visits++;
 		
 		/* update host data as well */
-		hi->host->last = now;
-		hi->host->visits++;
+		hi->host->item.last = now;
+		hi->host->item.visits++;
 		if (hi->visits > hi->host->dominant_item->visits)
 		{
 			hi->host->dominant_item = hi;
@@ -433,6 +463,9 @@ history_visited (const gchar *url)
 				       now, now, 1);
         }
 
+	/* update the last visited item */
+	history_update_last_visited_site (hi);
+
 	/* update all the views */
 	history_update_item_views (hi);
 
@@ -440,8 +473,89 @@ history_visited (const gchar *url)
 	history_dirty = TRUE;
 }
 
+/**
+ * history_remove_item: removes a history item.
+ */
+void
+history_remove_item (const gchar *url)
+{
+        HistoryItem *hi;
+	HistoryHost *hh;
+
+	/* check arguments */
+	g_return_if_fail (url != NULL);
+
+	/* lookup in history */
+        hi = g_hash_table_lookup (history, url);
+        if (hi != NULL)
+	{
+		GtkCTreeNode *node;
+		GList *l;
+
+		/* Remove the item */
+		g_hash_table_remove (history, url);
+
+		/* Refresh the history views */
+		for (l = history_views; l; l = g_list_next (l))
+		{
+			HistoryView *v = (HistoryView *) l->data;
+			node = g_hash_table_lookup (v->item_to_node, hi);
+			gtk_ctree_remove_node (GTK_CTREE (v->ctree), node);
+		}
+
+		/* Free the item. Must be done here as it is implictly
+		   referenced during the refresh */
+		history_item_free_cb ((gpointer)url, hi, NULL);
+
+		/* we have changed the history, so save at next checkpoint */
+		history_dirty = TRUE;
+
+		return;
+        }
+	
+	hh = g_hash_table_lookup (history_hosts, url);
+	if (hh != NULL)
+	{
+		GtkCTreeNode *node;
+		GList *l;
+
+		/* Remove the item */
+		g_hash_table_remove (history_hosts, url);
+
+		/* Refresh the history views */
+		for (l = history_views; l; l = g_list_next (l))
+		{
+			HistoryView *v = (HistoryView *) l->data;
+			node = g_hash_table_lookup (v->host_to_node, hh);
+			gtk_ctree_remove_node (GTK_CTREE (v->ctree), node);
+		}
+
+		/* Free the children */
+		for (l = hh->item.children; l; l = g_list_next (l))
+		{
+			HistoryItem *i = (HistoryItem *) l->data;
+			
+			/* Remove the item */
+			g_hash_table_remove (history, i->url);
+
+			/* Free the item */
+			history_item_free_cb ((gpointer)i->url, i, NULL);
+		}
+
+		/* Free the item. Must be done here as it is implictly
+		   referenced during the refresh */
+		history_item_free_cb ((gpointer)url, (HistoryItem*)hh, NULL);
+
+		/* we have changed the history, so save at next checkpoint */
+		history_dirty = TRUE;
+
+		return;
+	}
+}
+
 /**
  * history_visited: called to update the title of a page in the history
+ * Notice that the passed titles are not strduped
  */
 void
 history_set_page_title (const gchar *url, 
@@ -464,17 +578,6 @@ history_set_page_title (const gchar *url,
 	/* check supplied title */
 	g_return_if_fail (title_locale != NULL);
 	g_return_if_fail (title_utf8 != NULL);
-	if (strcmp (title_utf8, hi->title_utf8) == 0)
-	{
-		/* identical */
-		/* add to the Go menu of each window (FIXME?) */
-		g_list_foreach (all_windows, (GFunc)history_add_recent_menu, 
-				hi);
-		
-		g_free (title_locale);
-		g_free (title_utf8);
-		return;
-	}
       
 	/* set utf8 title */
 	g_free (hi->title_utf8);
@@ -500,9 +603,6 @@ history_set_page_title (const gchar *url,
 	/* update all the views */
 	history_update_item_views (hi);
 
-	/* add to the Go menu of each window (FIXME?) */
-	g_list_foreach (all_windows, (GFunc)history_add_recent_menu, hi);
-
 	/* we have changed the history, so save at next checkpoint */
 	history_dirty = TRUE;
 }
@@ -579,6 +679,9 @@ history_build_view (GaleonWindow *window, const char *xml_root)
 	/* add to the list of views */
 	history_views = g_list_append (history_views, hv);
 
+	/* nullify selection */
+	hv->selected_items = NULL;
+	
 	/* return view structure */
 	return hv;
 }
@@ -634,7 +737,7 @@ history_update_item_view (HistoryView *hv, HistoryItem *hi)
 		history_make_item_strings (hi, text);
 
 		/* make pixmap for item */
-		bm_icon = bookmarks_get_siteicon (hi->url);
+		bm_icon = favicon_get_pixmap (hi->url);
 		
 		/* update in tree */
 		gtk_ctree_node_set_pixtext (GTK_CTREE (hv->ctree),
@@ -708,20 +811,21 @@ history_add_host (const gchar *name)
 	host = g_new0 (HistoryHost, 1);
 
 	/* fill in structure */
-	host->title_locale       = g_strdup (name);
-	host->title_utf8         = g_strdup (name);
-	host->url                = g_strdup (name);
-	host->node               = NULL;
-	host->sibling            = NULL;
-	host->zoom               = 0;
-	host->selected_authorcss = -1;
-	host->first              = 0;
-	host->last               = 0;
-	host->visits             = 0;
-	host->dominant_item      = NULL;
+	host->item.title_locale       = g_strdup (name);
+	host->item.title_utf8         = g_strdup (name);
+	host->item.url                = g_strdup (name);
+	host->node                    = NULL;
+	host->sibling                 = NULL;
+ 	host->zoom                    = 0;
+	host->item.selected_authorcss = -1;
+	host->item.first              = 0;
+	host->item.last               = 0;
+	host->item.visits             = 0;
+	host->dominant_item           = NULL;
+	host->item.children           = NULL;
 
 	/* add to the hosts hashtable */
-	g_hash_table_insert (history_hosts, host->url, host);
+	g_hash_table_insert (history_hosts, host->item.url, host);
 
 	/* return completed structure */
 	return host;
@@ -797,7 +901,7 @@ history_filter_text (HistoryView *hv, gchar *url)
 		return TRUE;
 	}
 
-	return (g_strcasestr (url, hv->search) != NULL);
+	return (misc_string_strcasestr (url, hv->search) != NULL);
 }
 
 /*
@@ -823,11 +927,11 @@ history_filter_date (HistoryView *hv, GTime atime)
 		/* Today */
 	case 1:
 		break;
-		/* Yesterday */
+		/* Last two days */
 	case 2:
 		g_date_subtract_days (&current_date, 1);
 		break;
-		/* Two days ago */
+		/* Last three days */
 	case 3:
 		g_date_subtract_days (&current_date, 2);
 		break;
@@ -928,7 +1032,7 @@ history_show_dialog_add_item (gpointer key, gpointer value, gpointer user_data)
 	history_make_item_strings (hi, text);
 
 	/* fetch icon */
-	bm_icon = bookmarks_get_siteicon (hi->url);
+	bm_icon = favicon_get_pixmap (hi->url);
 	
 	/* add node to tree */
 	node = gtk_ctree_insert_node (GTK_CTREE (hv->ctree), parent_node,
@@ -963,11 +1067,11 @@ static void
 history_make_host_strings (HistoryHost *hh, gchar **text)
 {
 	/* print into strings */
-	text[0] = hh->url;
-	text[1] = hh->url;
-	store_time_in_string (hh->last, text[2]);
-	store_time_in_string (hh->first, text[3]);
-	sprintf (text[4], "%d", hh->visits);
+	text[0] = hh->item.url;
+	text[1] = hh->item.url;
+	misc_string_store_time_in_string (hh->item.last, text[2]);
+	misc_string_store_time_in_string (hh->item.first, text[3]);
+	sprintf (text[4], "%d", hh->item.visits);
 }
 
 /**
@@ -980,49 +1084,11 @@ history_make_item_strings (HistoryItem *hi, gchar **text)
 	text[0] = (hi->title_locale != NULL ? 
 		   hi->title_locale : hi->title_utf8);		
 	text[1] = hi->url;
-	store_time_in_string (hi->last, text[2]);
-	store_time_in_string (hi->first, text[3]);
+	misc_string_store_time_in_string (hi->last, text[2]);
+	misc_string_store_time_in_string (hi->first, text[3]);
 	sprintf (text[4], "%d", hi->visits);
 }
 
-/**
- * Clears the recent menu
- */
-void
-history_clear_recent_menu (void)
-{
-	GList *w;
-	gint go_recent_pos = GO_RECENT_POS;
-	
-	if (gnome_preferences_get_menus_have_tearoff ())
-	{
-		go_recent_pos++;
-	}
-
-	for (w = all_windows; w != NULL; w = g_list_next(w))
-	{
-		GaleonWindow *window = (GaleonWindow *)(w->data);
-		GtkWidget *menu;
-		GList *children;
-		GList *tmplist;
-
-		menu = window->go_menu;
-		children = gtk_container_children (GTK_CONTAINER(menu));
-		tmplist = children;
-		children = g_list_nth (children, go_recent_pos);
-		
-		while (children != NULL)
-		{ 
-			gtk_container_remove (GTK_CONTAINER (menu),
-					      GTK_WIDGET (children->data));
-			gtk_widget_destroy (GTK_WIDGET (children->data));
-			children = g_list_next (children);
-		}
-		
-		g_list_free (tmplist);
-	}
-}
-
 /**
  * Adds a new history item to the hashtable and to the history dialog
  */
@@ -1048,7 +1114,7 @@ history_add_item (gchar *url, gchar *title_utf8, GTime first,
 	hi->last               = last;
 	hi->visits             = visits;
 	hi->selected_authorcss = -1;
-
+	hi->children           = NULL;
 	
 	/* convert to locale if different */
 	title_locale = mozilla_utf8_to_locale (title_utf8);
@@ -1064,23 +1130,24 @@ history_add_item (gchar *url, gchar *title_utf8, GTime first,
 	}
 
 	/* update the host */
-	host->visits += visits;
+	host->item.visits += visits;
 	if (host->dominant_item == NULL)
 	{
 		/* first item of host */
-		host->first = first;
-		host->last = last;
+		host->item.first = first;
+		host->item.last = last;
 		host->dominant_item = hi;
 	}
 	else
 	{
-		host->first = MIN (host->first, first);
-		host->last = MAX (host->last, last);
+		host->item.first = MIN (host->item.first, first);
+		host->item.last = MAX (host->item.last, last);
 		if (visits > host->dominant_item->visits)
 		{
 			host->dominant_item = hi;
 		}
 	}
+	host->item.children = g_list_append (host->item.children, hi);
 
 	/* add to the table */
         g_hash_table_insert (history, hi->url, hi);
@@ -1088,22 +1155,39 @@ history_add_item (gchar *url, gchar *title_utf8, GTime first,
 	/* add a completion for it */
         auto_completion_add_url (hi->url);
 
+	/* update the last visited item */
+	history_update_last_visited_site (hi);
+
 	/* return the finished item */
 	return hi;
 }
 
+/**
+ * Returns the title of a site if it can be found in history
+ */
+const gchar *
+history_get_page_title (const gchar *url)
+{
+	HistoryItem *hi = g_hash_table_lookup (history, url);
+	if (hi)
+		return hi->title_locale ? hi->title_locale : hi->title_utf8;
+	else
+		return NULL;
+}
+
 /** 
  * Returns the url of the last visited page
  */ 
 const gchar *
 history_get_last_url (void)
 {
-        HistoryItem *last = NULL;
 
-        g_hash_table_foreach (history, history_look_for_newer, &last);
-        if (last != NULL)
+	if (last_visited_site == NULL)
+		g_hash_table_foreach (history, history_look_for_newer, NULL);
+
+        if (last_visited_site != NULL)
 	{
-                return last->url;
+                return last_visited_site->url;
         } 
 	else
 	{
@@ -1114,22 +1198,23 @@ history_get_last_url (void)
 
 /**
  * Helper function to locate the newer HistoryItem
- * FIXME: we can do this better by constantly tracking the most
- * recent history item
  */
-void 
+static void 
 history_look_for_newer (gpointer key, gpointer value, gpointer user_data)
 {
-        static GTime maxtime = 0;
         HistoryItem *hi = (HistoryItem *) value;
-        if (hi->last >= maxtime)
-	{
-                HistoryItem **last = (HistoryItem **) user_data;
-                *last = hi;
-                maxtime = hi->last;
-        }
+	history_update_last_visited_site (hi);
 }
 
+static void
+history_update_last_visited_site (HistoryItem *hi)
+{
+	if ((last_visited_site == NULL) 
+	    || (hi->last > last_visited_site->last))
+		last_visited_site = hi;
+}
+
+
 /** 
  * Returns the number of days since the history item was visited for the last time
  */
@@ -1141,119 +1226,6 @@ calculate_age (GTime last)
         return days;
 }
 
-/**
- * Adds an item to the recent menu of a given window or moves it to the start.
- */
-
-/* FIXME: this function looks awful */
-static void
-history_add_recent_menu (GaleonWindow *window, HistoryItem *hi)
-{
-	GtkWidget *menu;
-	GtkWidget *menuitem;
-	GList *l;
-	GList *li;
-	gint go_recent_pos = GO_RECENT_POS;
-	gint count = 1 - go_recent_pos;
-	gint max = 10;  /* should we add a preference for this? */
-	HistoryItem *mhi;
-
-	/* check if the menu exist */
-	if (GNOME_APP (window->WMain)->menubar == NULL)
-	{
-		return;
-	}
-	
- 	menu = window->go_menu;
-
-	if (gnome_preferences_get_menus_have_tearoff ())
-	{
-		go_recent_pos++;
-		count--;
-	}
-
-	l = gtk_container_children (GTK_CONTAINER(menu));
-
-        for (li = l; li != NULL; li = li->next)
-	{
-		if (count>0)
-		{
-			mhi = gtk_object_get_user_data (GTK_OBJECT (li->data));
-			g_assert (mhi != NULL);
-			gtk_widget_destroy (GTK_WIDGET (li->data));
-			if (mhi != hi)
-			{
-				if (count < max) 
-				{
-					menuitem =
-						history_create_recent_menu_item 
-							(mhi, count, menu);
-					gtk_menu_insert (GTK_MENU (menu),
-						menuitem, go_recent_pos+count);
-					gtk_widget_show (menuitem);
-				}
-				count++;
-			}
-		}
-		else 
-			count++;
-	}
-        g_list_free (l);
-        menuitem = history_create_recent_menu_item (hi, 0, menu);
-        gtk_menu_insert (GTK_MENU (menu), menuitem, go_recent_pos);
-        gtk_widget_show (menuitem);
-}
-
-GtkWidget *
-history_create_recent_menu_item (HistoryItem *hi, gint num, GtkWidget *menu)
-{
-	GtkWidget *hb = gtk_hbox_new (FALSE, 0);
-	const PixmapData *pixmap_data = bookmarks_get_siteicon (hi->url);
-	GtkWidget *pixmap = gtk_pixmap_new (pixmap_data->pixmap,
-		pixmap_data->mask);
-	GtkWidget *menuitem = gtk_pixmap_menu_item_new ();
-	gchar *title_locale, *s, *tmp = NULL;
-	GtkWidget *l;
-	gint length;
-
-	s = (hi->title_locale != NULL ? 
-	     hi->title_locale : hi->title_utf8);
-	length = strlen (s);
-	
-	/* shorten if needed */
-	if (length > 40)
-	{
-		tmp = g_strdup_printf ("%.29s...%s", s, s + length - 8);
-	}
-	
-	title_locale = escape_uline_accel (tmp ? tmp : s);
-	
-	l = new_num_accel_label (num, title_locale, FALSE, menu, menuitem);
-
-	if (tmp) g_free (tmp);
-	g_free (title_locale);
-	
-	gtk_box_pack_start (GTK_BOX (hb), l, FALSE, FALSE, 2);
-	gtk_container_add (GTK_CONTAINER(menuitem), hb);
-	gtk_pixmap_menu_item_set_pixmap (GTK_PIXMAP_MENU_ITEM(menuitem),
-		pixmap);
-	gtk_widget_lock_accelerators (menuitem);
-	
-	gtk_widget_show_all (hb);
-	if (gnome_preferences_get_menus_have_icons ())
-		gtk_widget_show (pixmap);
-	
-	gtk_object_set_user_data (GTK_OBJECT (menuitem), hi);
-	gtk_signal_connect (GTK_OBJECT(menuitem), "activate",
-			    GTK_SIGNAL_FUNC (window_menu_recent_activate_cb),
-			    NULL);
-	gtk_signal_connect (GTK_OBJECT(menuitem), "button-release-event",
-			    GTK_SIGNAL_FUNC (window_menu_recent_release_cb),
-			    NULL);
-
-	return menuitem;
-}
-
 /**
  * history_set_zoom: set the zoom level of the server referenced by the
  * given url
@@ -1317,7 +1289,7 @@ history_set_selected_authorcss (const gchar *url, gint n)
 	/* store selected css in host */
 	host = history_get_host (url);
 	if (host != NULL)
-		host->selected_authorcss = n;
+		host->item.selected_authorcss = n;
 }
 
 /**
@@ -1339,7 +1311,7 @@ history_get_selected_authorcss (const gchar *url)
 	host = history_get_host (url);
 	
 	/* our result */
-	return (host == NULL ? -1 : host->selected_authorcss);
+	return (host == NULL ? -1 : host->item.selected_authorcss);
 }
 
 /**
@@ -1375,18 +1347,47 @@ static gboolean
 history_item_free_cb (gpointer key, gpointer value, gpointer user_data)
 {
 	HistoryItem *hi = (HistoryItem *)value;
+	HistoryHost *host = NULL;
 
 	/* NB the key is either the URL, in which case it is free'd here
 	 * or if this is actually a host then it's just a pointer, so it
 	 * doesn't need free'ing */
 
+	/* remove from lists */
+	if (history_hosts && history)
+	{
+		host = history_get_host (hi->url);
+	}
+
+	if (host)
+	{
+		if (g_list_find (host->item.children, hi))
+		{
+			host->item.children = g_list_remove
+				(host->item.children, hi);
+		}
+	}
+	
+	/* if this was the last visited item, make it NULL. 
+	 * It will be recalculated when needed */
+	if (hi == last_visited_site)
+	{
+		last_visited_site = NULL;
+	}
+
 	/* free the item */
-	g_free (hi->url);
+	auto_completion_remove_url (hi->url);
+	g_free (hi->url); 
 	if (hi->title_locale != NULL)
 	{
 		g_free (hi->title_locale);
 	}
 	g_free (hi->title_utf8);
+	if (hi->children != NULL)
+	{
+		g_list_free (hi->children);
+		hi->children = NULL;
+	}
 	g_free (hi);
 
 	return TRUE;
@@ -1465,7 +1466,7 @@ tree_sort (GtkCTree *ctree, GtkCTreeNode *node, gpointer data)
 	     work = GTK_CTREE_ROW(work)->sibling)
 	{
 		sorted = g_list_prepend (sorted, work);
-		gtk_ctree_unlink (ctree, work, FALSE);
+		misc_gui_gtk_ctree_unlink (ctree, work, FALSE);
 	}
 	
 	/* do the sort */
@@ -1488,7 +1489,8 @@ tree_sort (GtkCTree *ctree, GtkCTreeNode *node, gpointer data)
 		}
 		else
 		{
-			gtk_ctree_link (ctree, this, node, list_start, FALSE);
+			misc_gui_gtk_ctree_link (ctree, this, node,
+						 list_start, FALSE);
 			list_start = this;
 		}
 	} 
@@ -1609,7 +1611,7 @@ history_parse_host_element (const xmlChar **attrs)
 	{
 		if (xmlStrcmp (attr[SAX_KEY], "name") == 0)
 		{
-			name = xml_decode_entity (attr[SAX_VALUE]);
+			name = misc_string_xml_decode_entity (attr[SAX_VALUE]);
 		}
 		else if (xmlStrcmp (attr[SAX_KEY], "zoom") == 0)
 		{
@@ -1630,7 +1632,7 @@ history_parse_host_element (const xmlChar **attrs)
 	/* create a host structure and store */
 	host = history_add_host (name);
 	host->zoom = zoom;
-	host->selected_authorcss = css;
+	host->item.selected_authorcss = css;
 	g_free (name);
 }
 
@@ -1652,11 +1654,12 @@ history_parse_item_element (const xmlChar **attrs)
 	{
 		if (xmlStrcmp(attr[SAX_KEY], "title") == 0)
 		{
-			title_utf8 = xml_decode_entity (attr[SAX_VALUE]);
+			title_utf8 = misc_string_xml_decode_entity (
+							attr[SAX_VALUE]);
 		}
 		else if (xmlStrcmp(attr[SAX_KEY], "url") == 0)
 		{
-			url = xml_decode_entity (attr[SAX_VALUE]);
+			url = misc_string_xml_decode_entity (attr[SAX_VALUE]);
 		}
 		else if (xmlStrcmp(attr[SAX_KEY], "first_time") == 0)
 		{
@@ -1686,8 +1689,8 @@ history_parse_item_element (const xmlChar **attrs)
 	}
 	else
 	{
-		if (url) g_free (url);
-		if (title_utf8) g_free (title_utf8);			
+		g_free (url);
+		g_free (title_utf8);			
 	}
 }
 
@@ -1729,11 +1732,13 @@ history_save_host (gpointer key, gpointer value, gpointer user_data)
         HistoryHost *hh = (HistoryHost *) value;
 	FILE *fp = (FILE *) user_data;
 
-	if (hh->zoom != 0 || hh->selected_authorcss != -1)
+	if (hh->zoom == 0 && hh->item.selected_authorcss == -1)
 	{
-		fputs ("  <host name=\"", fp);
-		fputs (hh->url, fp);
+		return;
 	}
+
+		fputs ("  <host name=\"", fp);
+		fputs (hh->item.url, fp);
 	
 	if (hh->zoom != 0)
 	{
@@ -1741,16 +1746,13 @@ history_save_host (gpointer key, gpointer value, gpointer user_data)
 		fprintf (fp, "%d", hh->zoom);
 	}
 
-	if (hh->selected_authorcss != -1)
+	if (hh->item.selected_authorcss != -1)
 	{
 		fputs ("\" authorcss=\"", fp);
-		fprintf (fp, "%d", hh->selected_authorcss);
+		fprintf (fp, "%d", hh->item.selected_authorcss);
 	}
 
-	if (hh->zoom != 0 || hh->selected_authorcss != -1)
-	{
-		fputs ("\"/>\n", fp);
-	}
+	fputs ("\"/>\n", fp);
 }
 
 /**
@@ -1846,35 +1848,41 @@ history_drag_data_get_cb (GtkWidget *widget, GdkDragContext *context,
 	BookmarkItem *b;
 	char *url;
 	gchar *mem;
+	GList *l;
 
-	g_return_if_fail (hv->selected_item != NULL);
-	url = hv->selected_item->url;
+	g_return_if_fail (hv->selected_items != NULL);
 
-	if (!url) return;
-
-	switch (info)
+	for (l = hv->selected_items; l; l = g_list_next (l))
 	{
-	case DND_TARGET_GALEON_BOOKMARK:
-		b = bookmarks_new_bookmark (BM_SITE, TRUE, url,
-					    url,
-					    NULL, NULL, NULL);
-		mem = bookmarks_item_to_string (b);
-		gtk_selection_data_set 
-			(selection_data, selection_data->target,
-			 8, mem, strlen (mem));
-		g_free (mem);
-		break;
-	case DND_TARGET_STRING:
-	case DND_TARGET_NETSCAPE_URL:
-	case DND_TARGET_GALEON_URL:
-		gtk_selection_data_set 
-			(selection_data, selection_data->target,
-			 8, url, 
-			 strlen (url));
-		break;
-	default:
-		g_warning ("Unknown DND type");
-		break;
+		HistoryItem *i = (HistoryItem *) l->data;
+		url = i->url;
+
+		if (!url) continue;
+
+		switch (info)
+		{
+		case DND_TARGET_GALEON_BOOKMARK:
+			b = bookmarks_new_bookmark (BM_SITE, TRUE, url,
+						    url,
+						    NULL, NULL, NULL);
+			mem = bookmarks_item_to_string (b);
+			gtk_selection_data_set 
+				(selection_data, selection_data->target,
+				 8, mem, strlen (mem));
+			g_free (mem);
+			break;
+		case DND_TARGET_STRING:
+		case DND_TARGET_NETSCAPE_URL:
+		case DND_TARGET_GALEON_URL:
+			gtk_selection_data_set 
+				(selection_data, selection_data->target,
+				 8, url, 
+				 strlen (url));
+			break;
+		default:
+			g_warning ("Unknown DND type");
+			break;
+		}
 	}
 }
 
@@ -1893,14 +1901,14 @@ notifiers [] =
 	{ NULL, NULL}
 };
 
-static
-void history_notifiers_init(HistoryView *hv) 
+static void 
+history_notifiers_init (HistoryView *hv) 
 {
 	int i;
 
 	for (i = 0; notifiers[i].gconf_key != NULL; i++)
 	{
-		galeon_notification_add(
+		galeon_notification_add (
 			notifiers[i].gconf_key,
 			(GConfClientNotifyFunc)notifiers[i].func,
 			(gpointer)hv,
@@ -1908,8 +1916,9 @@ void history_notifiers_init(HistoryView *hv)
 	}
 }
 
-
-void history_notifiers_remove(void) 
+void 
+history_notifiers_remove (void) 
 {
-	galeon_notification_remove(&history_notifiers_id);
+	galeon_notification_remove (&history_notifiers_id);
 }
+
