@@ -16,26 +16,30 @@
  *  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
  */
 
+
 /* Galeon includes */
 #include "galeon.h"
-#include "Galeon.h"
+#include "GaleonAutomation.h"
 #include "prefs.h"
 #include "history.h"
 #include "embed.h"
 #include "session.h"
-#include "mozilla_prefs.h"
-#include "misc.h"
 #include "bookmarks.h"
 #include "window.h"
 #include "dialog.h"
+#include "favicon.h"
+#include "gestures.h"
+#include "js_console.h"
+#include "mime.h"
+#include "mozilla.h"
+#include "misc_gui.h"
 
 #include <gtkmozembed.h>
 #include <glade/glade.h>
-
-/* GNOME includes */
 #include <libgnome/gnome-util.h>
 #include <libgnome/gnome-config.h>
 #include <libgnome/gnome-i18n.h>
+#include <libgnomeui/gnome-client.h>
 #include <libgnomeui/gnome-init.h>
 #include <libgnomeui/gnome-geometry.h>
 #include <libgnomeui/gnome-dialog-util.h>
@@ -48,8 +52,12 @@
 #ifdef ENABLE_GNOME_FILE_SELECTOR
 #include <bonobo/bonobo-main.h>
 #endif /* !ENABLE_GNOME_FILE_SELECTOR */
-
-#include <errno.h>
+#ifdef ENABLE_NAUTILUS_VIEW
+/* FIXME: include only the needed bits */
+#include <bonobo.h>
+#include <libnautilus/libnautilus.h>
+#include "galeon-nautilus-view.h"
+#endif
 
 /* local function prototypes */
 static gint translate_url_arguments (poptContext context, gchar ***url);
@@ -59,16 +67,32 @@ static void new_shell (CORBA_ORB orb);
 static gboolean galeon_init (int argc, char *argv[]);
 static void create_default_browser (void);
 static inline ViewState get_viewstate (void);
-static gint read_mozilla_version (void);
-static gint check_mozilla_version (void);
+
+#ifdef ENABLE_NAUTILUS_VIEW
+static BonoboObject *galeon_nautilus_view_new (BonoboGenericFactory *factory, const char *id, gpointer data);
+static void galeon_init_factory (void);
+#endif
 
 /* import from Galeon-impl.c */
 Galeon_Browser impl_Galeon_Browser__create (PortableServer_POA poa, 
 					    CORBA_Environment *corba_env);
 
+/* local function prototypes */
+gboolean client_save_yourself_cb (GnomeClient *client, gint phase, 
+				  GnomeSaveStyle save_style, gint shutdown,
+				  GnomeInteractStyle interact_style, gint fast);
+void client_die_cb (GnomeClient *client, gpointer data);
+
 /* global variables */
 gboolean galeon_server_mode = FALSE;
+gint galeon_server_timeout = -1;
 gboolean galeon_panel_mode = FALSE;
+gboolean galeon_exiting = FALSE;
+
+Galeon_Browser galeon_automation_client = NULL;
+#ifdef ENABLE_NAUTILUS_VIEW
+BonoboGenericFactory *galeon_nautilusview_factory = NULL;
+#endif
 
 /* file local variables */
 CORBA_Environment corba_env;                 /* corba environment for errors */
@@ -77,6 +101,7 @@ static gboolean open_in_new_tab    = FALSE;  /* force open in a new tab?     */
 static gboolean noraise            = FALSE;  /* no raise                     */
 static gboolean open_in_new_window = FALSE;  /* force open in a new window?  */
 static gboolean open_fullscreen    = FALSE;  /* open galeon in full screen ? */
+static gchar *session_filename     = NULL;   /* the session filename         */
 static gchar *geometry_string      = NULL;   /* the geometry string          */
 static gchar *bookmark_url         = NULL;   /* the temp bookmark to add     */
 static gboolean close_option       = FALSE;  /* --close                      */
@@ -105,6 +130,9 @@ static struct poptOption popt_options[] =
 	{ "existing", 'x', POPT_ARG_NONE, &open_in_existing, 0,
 	  N_("Attempt to load URL in existing Galeon window"),
 	  NULL },
+	{ "load-session", 'l', POPT_ARG_STRING, &session_filename, 0,
+	  N_("Load the given session file"),
+	  N_("FILE") },
 #ifdef ENABLE_APPLET
 	{ "panel", 'p', POPT_ARG_NONE, &galeon_panel_mode, 0,
 	  N_("Open as a panel applet"),
@@ -114,6 +142,10 @@ static struct poptOption popt_options[] =
 	  N_("Don't open any windows; instead act as a server "
 	     "for quick startup of new Galeon instances"),
 	  NULL },		  
+	{ "server-timeout", 'e', POPT_ARG_INT, &galeon_server_timeout, 0,
+	  N_("Time of inactivity (in seconds) until the server "
+	     "shuts down itself. Defaults to one hour, use 0 to disable."),
+	  N_("TIME") },		  
 	{ "add-bookmark", 't', POPT_ARG_STRING, &bookmark_url,
 	  0, N_("Add a bookmark (don't open any window)"), 
 	  N_("URL")},
@@ -182,7 +214,8 @@ main (int argc, char *argv[])
 #endif
 	{
 		gnome_init_with_popt_table (PACKAGE, VERSION, argc, argv,
-					    popt_options, 0, &context);
+					    popt_options /* oaf_popt_options*/
+					    , 0, &context);
 	}
 
 	if (galeon_panel_mode && galeon_server_mode)
@@ -192,12 +225,22 @@ main (int argc, char *argv[])
 			     "Ignoring server mode option."));
 	}
 
+	/* set the server timeout to an hour by default if the option 
+	 * was not present */
+	if (galeon_server_timeout < 0)
+	{
+		galeon_server_timeout = 1 * 60 * 60;
+	}
+	
 	/* load arguments that aren't regular options (urls to load) */
 	n_urls = translate_url_arguments (context, &url);
 
 	/* initialise CORBA */
 	orb = corba_init (&argc, argv);
-	
+#ifdef ENABLE_NAUTILUS_VIEW
+	bonobo_activate ();
+#endif
+
 	/* check if galeon is already running and use that instead */
 	if (!galeon_panel_mode && !galeon_server_mode)
 	{
@@ -208,6 +251,9 @@ main (int argc, char *argv[])
 		} 
 	}
 
+	/* if no running shell exist, quit and close prefs
+	 * are invalid
+	 */
 	if (close_option || quit_option)
 	{
 		g_message (_("Nothing to close: "
@@ -221,21 +267,51 @@ main (int argc, char *argv[])
 	/* initialise required external and internal Galeon services */
 	no_default = galeon_init (argc, argv);
 
+	/* load the session if requested */
+	if (session_filename)
+		session_load_from (session_filename, TRUE);
+
 	/* add a temp bookmark and exit if needed */
 	if (bookmark_url != NULL)
 	{
 		bookmarks_load ();
 		/* this will save the bookmarks too -- Matthew */
-		add_bookmark_default (BM_SITE, NULL, bookmark_url, NULL);
+		bookmarks_add_bookmark (NULL, bookmark_url, NULL, NULL,
+					BOOKMARK_ADD_NEVER_PROMPT_TITLE);
 		return 0;
 	}
 	
-	/* if we have command-line supplied URLs */
-	tabbed_mode = eel_gconf_get_boolean (CONF_TABS_TABBED) ? 1 : 0;
+	/* check if urls should be opened in tabs, and if so, find an embed */
+	if (get_viewstate () != VIEWSTATE_WINDOW) tabbed_mode = TRUE;
+	else tabbed_mode = FALSE;
+	if (tabbed_mode)
+	{
+		if (all_windows)
+		{
+			GaleonWindow *window = all_windows->data;
+			if (window && window->magic == GALEON_WINDOW_MAGIC)
+			{
+				if (window->active_embed &&
+				    window->active_embed->magic ==
+				    	GALEON_EMBED_MAGIC)
+				{
+					last_embed = window->active_embed;
+				}
+			}
+		}
+	}
+
+	/* open all the command-line supplied urls */
 	for (i = 0; i < n_urls; i++)
 	{
-		last_embed = embed_create_from_url (last_embed, url[i],
-						    FALSE, !tabbed_mode);
+		last_embed = embed_create_after_embed (last_embed,
+			!tabbed_mode, url[i], EMBED_CREATE_RAISE_WINDOW |
+			EMBED_CREATE_FORCE_APPEND);
+		if (open_fullscreen && last_embed->parent_window)
+		{
+			window_set_fullscreen_mode (last_embed->parent_window, 
+						    TRUE);
+		}
 	}
 	g_strfreev (url);
 
@@ -245,6 +321,12 @@ main (int argc, char *argv[])
 		create_default_browser ();
 	}
 
+	/* start quit timeout if no window has been opened */
+	if (galeon_server_mode && (all_embeds == NULL))
+	{
+		session_server_start_timeout ();
+	}
+
 	/* start main loop */
 #ifdef ENABLE_APPLET
 	if (galeon_panel_mode)
@@ -256,9 +338,18 @@ main (int argc, char *argv[])
 #endif
 	{
 		/* enter the main GTK event processing loop */
+#ifdef ENABLE_NAUTILUS_VIEW	       
+		bonobo_main ();
+#else
 		gtk_main ();
+#endif
 	}
 
+	/* absolutely no more mozembeds - this happens after the
+           mainloop has returned to make sure that all timers have
+           expired and everything has been handled. */
+	gtk_moz_embed_pop_startup ();
+
 	eel_gconf_monitor_remove ("/apps/galeon");
 	eel_gconf_monitor_remove ("/apps/nautilus/preferences");
 
@@ -266,133 +357,6 @@ main (int argc, char *argv[])
 	return 0;
 }
 
-/**
- * read_mozilla_version: extract mozilla version from mozilla prefs file
- */
-static gint
-read_mozilla_version (void)
-{
-	gchar agentstr[160], *p, *endptr;
-	gint ver= 0, v;
-	gchar *preffile;
-	FILE *f;
-
-	preffile = g_strconcat (getenv ("MOZILLA_FIVE_HOME"),
-				"/defaults/pref/all.js", NULL);
-	f = fopen (preffile, "r");
-	
-	if (f == NULL)
-	{
-		g_warning ("error opening %s: %s", preffile, strerror (errno));
-		g_free (preffile);
-		return 0;
-	}
-
-	while ((p = fgets (agentstr, 160, f)) != NULL)
-	{
-		if ((p = strstr (agentstr, "useragent.misc")) != NULL &&
-			(p = strstr (p, "rv:")) != NULL)
-			break;
-	}
-	fclose(f);
-
-	if (p == NULL)
-	{
-		g_warning ("Mozilla version not found in %s", preffile);
-	}
-	else
-	{
-		p += 3;
-		v = strtol (p, &endptr, 10);
-		ver = v << 24;
-		if (*endptr == '.')
-		{
-			p = endptr + 1;
-			v = strtol (p, &endptr, 10);
-			ver += v << 16;
-			if (*endptr == '.')
-			{
-				p = endptr + 1;
-				v = strtol (p, &endptr, 10);
-				ver += v << 8;
-			}
-		}
-		if (*endptr == '+')
-		{
-			ver++;
-		}
-	}
-
-	g_free (preffile);
-	return ver;
-}
-
-/**
- * check_mozilla_version: verify that the runtime mozilla is the same version
- * as the compile time.  If not print a warning and display a confirmation
- * dialog.
- */
-static gint
-check_mozilla_version (void)
-{
-	mozilla_version = read_mozilla_version ();
-
-	if (MOZILLA_VERSION != mozilla_version)
-	{
-		gchar *questionStr, *rememberStr, *verMatchStr;
-		gboolean continuev=0, rememberv=0;
-		g_warning ("compiled MOZILLA_VERSION ("VERSIONFMT") !="
-			" detected mozilla_version ("VERSIONFMT")",
-			VERSIONFMTARGS (MOZILLA_VERSION),
-			VERSIONFMTARGS (mozilla_version));
-
-		verMatchStr = g_strdup_printf ("%xv%x", MOZILLA_VERSION,
-						mozilla_version);
-		rememberStr = eel_gconf_get_string 
-				(CONF_MOZ_MISMATCH);
-
-		if (rememberStr != NULL)
-		{
-			int ignore = strcmp (rememberStr, verMatchStr);
-			g_free (rememberStr);
-			if (ignore == 0)
-			{
-				g_free (verMatchStr);
-				return eel_gconf_get_integer
-				    (CONF_MOZ_MISMATCH_ACTION);
-			}
-		}
-
-		questionStr = g_strdup_printf (_(
-"The version of Mozilla Galeon was compiled against ("VERSIONFMT") "
-"does not match the detected version ("VERSIONFMT").\n\n"
-"You may experience segfaults and other oddities, please test "
-"with a matching version of Mozilla before reporting any bugs.\n\n"
-"Continue anyway?"),
-			VERSIONFMTARGS (MOZILLA_VERSION),
-			VERSIONFMTARGS (mozilla_version));
-
-
-		continuev = dialog_confirm_check (NULL, _("Version mismatch!"),
-					questionStr, _("Don't ask me again"),
-					&rememberv);
-	
-		g_free (questionStr);
-		if (rememberv)
-		{
-			eel_gconf_set_string
-				(CONF_MOZ_MISMATCH,
-				verMatchStr);
-			eel_gconf_set_integer
-				(CONF_MOZ_MISMATCH_ACTION,
-				 !continuev);
-		}
-		g_free (verMatchStr);
-		return !continuev;
-	}
-	return 0;
-}
-
 /**
  * translate_url_arguments: gather URL arguments and expand them fully
  * with realpath if they're filenames
@@ -466,6 +430,12 @@ corba_init (int *argc, char *argv[])
         if ( !bonobo_init (orb, NULL, NULL)) {
                 g_warning ("Cannot initialize Bonobo component framework !");
         }
+#else
+#ifdef ENABLE_NAUTILUS_VIEW
+        if ( !bonobo_init (orb, NULL, NULL)) {
+                g_warning ("Cannot initialize Bonobo component framework !");
+        }
+#endif
 #endif /* !ENABLE_GNOME_FILE_SELECTOR */
         
 	/* return handle */
@@ -485,6 +455,7 @@ new_view_on_running_shell (gint n_urls, gchar **url)
 	ViewState viewstate;
 	gboolean result = TRUE;
 	gint i;
+	gchar *default_url;
 	
 	/* compute target viewstate */
 	viewstate = get_viewstate ();
@@ -499,11 +470,19 @@ new_view_on_running_shell (gint n_urls, gchar **url)
 		return FALSE;
 	}
 
+	/* load the session if requested */
+	if (session_filename)
+	{
+		Galeon_Browser_loadSession (client, session_filename,
+					    &corba_env);
+		return TRUE;
+	}
+
 	/* if found and we're given a bookmark to add... */
 	if (bookmark_url != NULL)
 	{
-		Galeon_Browser_addTempBookmark (client, bookmark_url, 
-						bookmark_url, &corba_env);
+		Galeon_Browser_addTempBookmark (client, bookmark_url,
+						&corba_env);
 		return TRUE;
 	}
 
@@ -519,15 +498,23 @@ new_view_on_running_shell (gint n_urls, gchar **url)
 	/* provided with urls? */
 	if (n_urls == 0)
 	{
+		default_url = embed_get_default_url (NULL);
 		/* no, open a default window */
-		result = Galeon_Browser_loadurl (client, "", viewstate, 
+		result = Galeon_Browser_loadurl (client, default_url, viewstate,
+						 geometry_string ?
+						 geometry_string : "",
+						 open_fullscreen,
 						 &corba_env);
+		g_free (default_url);
 	}
 
 	/* open all of the urls */
 	for (i = 0; i < n_urls; i++)
 	{
 		result = Galeon_Browser_loadurl (client, url[i], viewstate, 
+						 geometry_string ?
+						 geometry_string : "",
+						 open_fullscreen,
 						 &corba_env);
 		if (!result)
 		{
@@ -560,16 +547,16 @@ static void
 new_shell (CORBA_ORB orb)
 {
 	PortableServer_POA poa;
-	Galeon_Browser client;
 	gchar *reg_id;
 
 	/* make a client */
 	poa = (PortableServer_POA) CORBA_ORB_resolve_initial_references 
 		(orb, "RootPOA", &corba_env);
-	client = impl_Galeon_Browser__create (poa, &corba_env);
+	galeon_automation_client = impl_Galeon_Browser__create (poa,
+								&corba_env);
 
 	/* check */
-	if (client == NULL)
+	if (galeon_automation_client == NULL)
 	{
 		g_warning (_("Galeon object already active, "
 			     "server registration failed\n"));
@@ -577,15 +564,13 @@ new_shell (CORBA_ORB orb)
 	}
 
 	/* register server with oaf */
-	reg_id = oaf_make_registration_id ("OAFIID:GNOME_Galeon_Automation",
-					   g_getenv ("DISPLAY"));
-	oaf_active_server_register (reg_id, client);
+	reg_id = GALEON_AUTOMATION_IID;
+	oaf_active_server_register (reg_id, galeon_automation_client);
 
 	PortableServer_POAManager_activate
 		(PortableServer_POA__get_the_POAManager (poa, &corba_env), 
 		 &corba_env);
 
-	g_free (reg_id);
 }
 
 /**
@@ -599,31 +584,14 @@ galeon_init (int argc, char *argv[])
 	gboolean session_recovery;
 	gboolean vfs_success;
 	gboolean new_user;
-	gchar *profile;
-	extern gboolean pushed_startup;
-	gboolean moz_version_checked = FALSE;
-/*	GdkAtom message_type;*/
-	gchar *teststr;
-	gchar *mozprefs;
-	gboolean set_mozilla_prefs = FALSE;
 
-	/* init mozilla home */
-	gtk_moz_embed_set_comp_path (g_getenv ("MOZILLA_FIVE_HOME"));
+	/* preload the prefs */
+	/* it also enables notifiers support */
+	eel_gconf_monitor_add ("/apps/galeon");
+	eel_gconf_monitor_add ("/apps/nautilus/preferences");
 
-	/* set a path for the profile */
-	profile = g_concat_dir_and_file (g_get_home_dir (), ".galeon/mozilla");
-
-	/* no mozilla prefs ? */
-	mozprefs = g_concat_dir_and_file (profile, "/galeon/prefs.js"); 
-	if (!g_file_exists(mozprefs))
-	{
-		set_mozilla_prefs = TRUE;
-	}
-	g_free (mozprefs);
-
-	/* initialize profile */
-	gtk_moz_embed_set_profile_path (profile, "galeon");
-	g_free (profile);
+	/* initialise GLADE */
+	glade_gnome_init ();
 
 	/* initialise embed */
 	if (!pushed_startup)
@@ -631,76 +599,31 @@ galeon_init (int argc, char *argv[])
 		embed_startup_init ();
 	}
 
-	/* set mozilla prefs if necessary */
-	if (set_mozilla_prefs)
-	{
-		mozilla_prefs_set ();
-	}
-
-	/* ensure mozilla matches (and initialise mozilla_version var) */
-	if (g_getenv ("MOZILLA_FIVE_HOME") != NULL)
-	{
-		if (check_mozilla_version ())
-		{
-			exit (1);
-		}
-		moz_version_checked = TRUE;
-	}
-
-	/* set the user agent, this is always neccesary because the galeon
-	   version may change */
-	mozilla_prefs_set_user_agent ();
-
 	/* initialise GNOME VFS */
 	vfs_success = gnome_vfs_init ();
 	g_assert (vfs_success);
 
-	/* initialise GLADE */
-	glade_gnome_init ();
-
-	/* preload the prefs */
-	/* it also enables notifiers support */
-	eel_gconf_monitor_add ("/apps/galeon");
-	eel_gconf_monitor_add ("/apps/nautilus/preferences");
-
-	/* check if gconf schemas are working */
-	teststr = gconf_client_get_string (eel_gconf_client_get_global(),
-					  "/apps/galeon/gconf_test", NULL);
-	if (!teststr)
-	{
-		GtkWidget *dialog;
-		dialog = gnome_error_dialog ("Cannot find a schema for galeon preferences. \nCheck your gconf setup, look at galeon FAQ for more info");
-		gnome_dialog_run_and_close (GNOME_DIALOG (dialog));
-		exit (0);
-	}
-	else
-	{
-		g_free (teststr);
-	}
-
 	/* check if this is the first time this user has run Galeon */
 	new_user = newuser_check ();
 
 	/* initialise Galeon itself */
-	initialise_colours ();
-	preferences_load ();
+	misc_gui_colors_init ();
+	prefs_load ();
 	history_init ();
 	bookmarks_init ();
-
-	/* if we couldn't check the first time, try now since initializing moz
-	 * will set the MOZILLA_FIVE_HOME in some configurations */
-	if (!moz_version_checked)
-	{
-		if (check_mozilla_version ())
-		{
-			exit (1);
-		}
-	}
+	downloader_dialog_init ();
+	favicon_key_files_init ();
+	gestures_init ();
+	js_console_init ();
+	session_history_load ();
+#if ENABLE_NAUTILUS_VIEW
+	galeon_init_factory ();
+#endif
 
 	/* resume a crashed session, if any */
 	session_recovery = session_autoresume ();
 
-	/* connect our session recovery to GNOME session */
+	/* attach session-manager callbacks */
 	session_client = gnome_master_client ();
 	gnome_client_set_restart_command 
 		(session_client, galeon_server_mode ? 2 : 1, restart_argv);
@@ -708,19 +631,8 @@ galeon_init (int argc, char *argv[])
 					GNOME_RESTART_IF_RUNNING);
 	gtk_signal_connect (GTK_OBJECT (session_client), "save_yourself",
 			    (GtkSignalFunc)client_save_yourself_cb, NULL);
-		
-	/* update tab styles and aa pixmaps when theme changed */
-	/* FIXME theme change is caught too many times, disabling for now
-	 *    -- Jorn 10/10/2001 */
-#if 0
-	message_type = gdk_atom_intern ("_GTK_READ_RCFILES", FALSE);
-	gdk_add_client_message_filter (message_type, (GdkFilterFunc)
-				       gtk_theme_changed_cb, NULL);
-#endif
-
-	/* FIXME: catch GDK errors which happen on theme changes
-	 * (until we work out what causes this) */
-	gdk_error_trap_push ();
+	gtk_signal_connect (GTK_OBJECT (session_client), "die",
+			    (GtkSignalFunc)client_die_cb, NULL);
 
 	/* if this flag doesn't get cleared, then Galeon crashed */
 	eel_gconf_set_integer (CONF_CRASH_CRASHED, TRUE);
@@ -740,7 +652,6 @@ create_default_browser (void)
 	GaleonWindow *window;
 	gint xpos, ypos;
 	gint width, height;
-	gchar *url;
 	
 	/* don't bother if there's already at least one */
 	if (g_list_length (all_embeds) != 0)
@@ -749,47 +660,39 @@ create_default_browser (void)
 	}
 
 	/* create an empty window */
-	window = window_create (GTK_MOZ_EMBED_FLAG_ALLCHROME);
+	window = window_create (DEFAULT_CHROME, NULL);
+	
+	/* collect geometry information */
+	if (gnome_parse_geometry (geometry_string, &xpos, &ypos,
+				       &width, &height))
+	{
+		/* setup size and show it */
+		gtk_window_set_default_size (GTK_WINDOW (window->wmain), 
+					     width, height);
+		gtk_widget_set_uposition (GTK_WIDGET (window->wmain),
+					  xpos, ypos);
+		window->set_size = TRUE;
+	}
+	
+	/* build the default embed inside it */
+	embed = embed_create_in_window (window, NULL, NULL,
+					EMBED_CREATE_RAISE_WINDOW |
+					EMBED_CREATE_LOAD_DEFAULT_URL);
 	
 	/* show as fullscreen? */
 	/* THE WINDOW NEEDS TO BE VISIBLE HERE */
 	if (open_fullscreen)
 	{
-		gtk_widget_show (GTK_WIDGET (window->WMain));
-		gtk_check_menu_item_set_active
-			(GTK_CHECK_MENU_ITEM (window->view_fullscreen),
-			 TRUE);
+		window_set_fullscreen_mode (window, TRUE);
 	}
-	/* collect geometry information */
-	else if (gnome_parse_geometry (geometry_string, &xpos, &ypos,
-				       &width, &height))
-	{
-		/* setup size and show it */
-		gtk_window_set_default_size (GTK_WINDOW (window->WMain), 
-					     width, height);
-		gtk_widget_set_uposition (GTK_WIDGET (window->WMain),
-					  xpos, ypos);
-		window->set_size = TRUE;
-	}
-
-	/* build the default embed inside it */
-	embed = embed_create_in_window (window);
-	embed_set_visibility (embed, TRUE);
+	
 	window_grab_location_focus (window);
-	url = embed_get_default_url (NULL);
-	embed_load_url (embed, url);
-	g_free (url);
 }
 
 /**
  * get_viewstate:
  *
  * Determine the viewstate. A viewstate defines how the URI should be opened.
- * 
- * VIEWSTATE_EXISTING:    Open up in existing window if there is one
- * VIEWSTATE_WINDOW:      Open up in window
- * VIEWSTATE_TAB:         Open up in tab
- * VIEWSTATE_NORAISE_TAB: Open up in tab without raising the window
  */
 static inline ViewState
 get_viewstate (void)
@@ -799,25 +702,126 @@ get_viewstate (void)
 
 	tabbed_mode = eel_gconf_get_boolean (CONF_TABS_TABBED);
 
-	result = tabbed_mode ? VIEWSTATE_TAB : VIEWSTATE_WINDOW;
-
-	if ((noraise && open_in_new_tab) ||
-	    (noraise && tabbed_mode))
-	{
-		result = VIEWSTATE_NORAISE_TAB;
-	}
-	else if (open_in_new_window)
-	{
-		result = VIEWSTATE_WINDOW;
-	}
-	else if (open_in_new_tab)
-	{
-		result = VIEWSTATE_TAB;
-	}
-	else if (open_in_existing)
-	{
+	/* figure out whether we should open in an existing browser, a
+	 * window, or a tab, and whether this was explicitly requested or
+	 * just defaulted to */
+	if (open_in_existing)
 		result = VIEWSTATE_EXISTING;
-	}
+	else if (open_in_new_tab)
+		result = VIEWSTATE_TAB_FORCE;
+	else if (open_in_new_window || !tabbed_mode)
+		result = VIEWSTATE_WINDOW;
+	else
+		result = VIEWSTATE_TAB_DEFAULT;
+
+	/* see if noraise was requested */
+	if (result == VIEWSTATE_TAB_FORCE && noraise)
+		result = VIEWSTATE_TAB_FORCE_NORAISE;
+	else if (result == VIEWSTATE_TAB_DEFAULT && noraise)
+		result = VIEWSTATE_TAB_DEFAULT_NORAISE;
 
 	return result;
 }
+
+#ifdef ENABLE_NAUTILUS_VIEW
+static void 
+galeon_init_factory (void)
+{
+	BonoboGenericFactory *galeon_nautilusview_factory;
+	galeon_nautilusview_factory = bonobo_generic_factory_new_multi
+	       	(GALEON_NAUTILUS_VIEW_IID,
+	         galeon_nautilus_view_new, NULL);
+	if (!BONOBO_IS_GENERIC_FACTORY (galeon_nautilusview_factory))
+		g_error ("Couldn't create the factory!");
+}
+
+static BonoboObject *
+galeon_nautilus_view_new (BonoboGenericFactory *factory, const char *id, 
+			  gpointer data)
+{
+	if (galeon_exiting) 
+		return NULL;
+	else
+		return galeon_nautilus_view_new_component ();
+}
+
+#endif
+
+void 
+galeon_exit (gboolean return_from_main_loop, gboolean unset_crashed_flag)
+{
+	extern GList *mozilla_notifiers;
+	
+	/**
+	 * First of all, unregister with OAF
+	 */
+	oaf_active_server_unregister (GALEON_AUTOMATION_IID, 
+				      galeon_automation_client);
+#ifdef ENABLE_NAUTILUS_VIEW
+/* TODO: I can't find how to get the CORBA object from a BonoboGenericFactory
+        oaf_active_server_unregister (GALEON_NAUTILUS_VIEW_IID,
+	                              [galeon_nautilusview_factory]);
+*/
+#endif
+	galeon_exiting = TRUE;
+
+	if (unset_crashed_flag)
+		eel_gconf_set_integer (CONF_CRASH_CRASHED, FALSE);
+
+	/* unset the this-session-only download dir pref */
+	eel_gconf_set_string (CONF_STATE_LAST_DOWNLOAD_DIR, "");
+
+	/* shut down galeon subsystems */
+	prefs_destroy (); /* this will sync prefs if dialog is open */
+	mime_db_shutdown ();
+	bookmarks_exit ();
+	history_exit ();
+	mozilla_save_prefs ();
+	favicon_key_files_close ();
+	js_console_exit ();
+	session_history_save ();
+
+	/* check... */
+	g_assert (g_list_length (all_embeds) == 0);
+	g_assert (g_list_length (all_windows) == 0);
+
+	/* shut down GNOME subsystems */
+	gnome_vfs_shutdown ();
+
+	/* remove notifiers for mozilla prefs */
+	galeon_notification_remove (&mozilla_notifiers);
+
+	/* return from the main loop if requested */
+	if (return_from_main_loop) 
+		gtk_main_quit();
+	else
+		/* otherwise, just exit */
+		gtk_exit (0);
+}
+
+/**
+ * client_save_yourself_cb: called during window manager session shutdown,
+ * we use our internal exit-with-session functionality to restore next
+ * time the user starts Galeon.
+ */
+gboolean
+client_save_yourself_cb (GnomeClient *client, gint phase, 
+			 GnomeSaveStyle save_style, gint shutdown,
+			 GnomeInteractStyle interact_style, gint fast)
+{
+	/* save session */
+	session_save ();
+
+	/* we're done */
+	return TRUE;
+}
+
+/**
+ * client_die_cb: called during session shutdown.  merely causes galeon to
+ * exit cleanly.
+ */
+void
+client_die_cb (GnomeClient *client, gpointer data)
+{
+	galeon_exit (TRUE, TRUE);
+}
