@@ -22,23 +22,27 @@
 #include "history.h"
 #include "embed.h"
 #include "menubar.h"
-#include "misc.h"
+#include "misc_general.h"
 #include "bookmarks.h"
+#include "bookmarks_editor.h"
 #include "print.h"
 #include "find.h"
 #include "session.h"
 #include "xlink.h"
 #include "dialog.h"
-#include "filepicker.h"
+#include "gfilepicker.h"
 #include "mozilla.h"
-#ifdef ENABLE_EXPERIMENTAL_FILTERS
-#include "filters.h"
-#endif
+#include "link_interfaces.h"
+#include "persistent_data_manager.h"
+#include "autoreload.h"
+#include "page_info.h"
+#include "js_console.h"
 
 #include <string.h>
 #include <gtkmozembed.h>
 #include <gtk/gtkspinbutton.h>
 #include <libgnomeui/gnome-about.h>
+#include <libgnomeui/gnome-dialog-util.h>
 #include <libgnome/gnome-util.h>
 #include <libgnome/gnome-help.h>
 #include <libgnome/gnome-config.h>
@@ -50,13 +54,15 @@ void
 window_menu_recent_release_cb (GtkMenuItem *menuitem, GdkEventButton *event,
 			       gpointer user_data);
 
+static GtkWidget *window_menu_edit_find_focus(GaleonWindow *window);
+
 /**
  * window_menu_prefs_activate_cb: show preferences dialog
  */
 void
 window_menu_prefs_activate_cb (GtkMenuItem *menuitem, GaleonWindow *window)
 {
-	preferences_open (window);
+	prefs_open (window);
 }
 
 /**
@@ -92,8 +98,56 @@ window_menu_bookmarks_edit_activate_cb (GtkMenuItem *menuitem,
 void
 window_menu_find_activate_cb (GtkMenuItem *menuitem, GaleonWindow *window)
 {
+	GaleonEmbed *embed;
+
 	return_if_not_window (window);
-	find_show_dialog (window);
+	embed = window->active_embed;
+	return_if_not_embed (embed);
+	
+	find_show_dialog (embed);
+}
+
+/**
+ * window_menu_find_next_activate_cb: show find dialog
+ */
+void
+window_menu_find_next_activate_cb (GtkMenuItem *menuitem, GaleonWindow *window)
+{
+	GaleonEmbed *embed;
+	gchar *val, *utf8;
+
+	return_if_not_window (window);
+	embed = window->active_embed;
+	return_if_not_embed (embed);
+
+	val = eel_gconf_get_string (CONF_FIND_WORD);
+	if (val)
+	{
+		utf8 = mozilla_utf8_to_locale (val);
+		find_next (embed, utf8);
+		g_free (utf8);
+		g_free (val);
+	}
+}
+
+/**
+ * window_menu_select_all_activate_cb: select all text on the current page
+ */
+void
+window_menu_select_all_activate_cb (GtkMenuItem *menuitem, GaleonWindow *window)
+{
+	GtkEditable *editable;
+	GtkWidget *widget;
+
+	if ((widget=window_menu_edit_find_focus(window)) == NULL)
+	{
+		mozilla_select_all (window->active_embed);
+		return;
+	}
+
+	editable = GTK_EDITABLE (widget);
+
+	gtk_editable_select_region (editable, 0, -1);
 }
 
 /*
@@ -113,13 +167,22 @@ window_menu_edit_find_focus(GaleonWindow *window)
 		}
 	}
 
+	if (window->find_entry != NULL)
+	{
+		if (GTK_WIDGET_HAS_FOCUS (window->find_entry))
+		{
+			return (window->find_entry);
+		}
+	}
+
 	/* Search through the toolbars, then down each toolbar
 	** looking for a focused widget
 	*/
 
 	for (widget=NULL, tb_entry = g_list_first(window->bookmarks_toolbars);
 		widget==NULL && tb_entry!=NULL;
-		tb_entry = g_list_next(tb_entry)) {
+		tb_entry = g_list_next(tb_entry))
+	{
 
 		GtkToolbar *toolbar=(GtkToolbar *) tb_entry->data;
 		GList *tbc_entry;
@@ -127,15 +190,22 @@ window_menu_edit_find_focus(GaleonWindow *window)
 		/* Step through the toolbar children */
 		for (tbc_entry = g_list_first(toolbar->children);
 			widget==NULL && tbc_entry!=NULL;
-			tbc_entry = g_list_next(tbc_entry)) {
+			tbc_entry = g_list_next(tbc_entry))
+		{
 
 			GtkToolbarChild *toolbarchild =
 				(GtkToolbarChild *) tbc_entry->data;
+			GtkWidget *entry;
 
-			if (toolbarchild->type == GTK_TOOLBAR_CHILD_WIDGET &&
-				GTK_WIDGET_HAS_FOCUS (toolbarchild->widget)) {
+			if (toolbarchild->type != GTK_TOOLBAR_CHILD_WIDGET)
+				continue;
+			
+			entry = gtk_object_get_data
+				(GTK_OBJECT (toolbarchild->widget), "entry");
 
-				widget=toolbarchild->widget;
+			if (entry && GTK_WIDGET_HAS_FOCUS (entry))
+			{
+				widget = entry;
 			}
 		}
 	}
@@ -261,10 +331,10 @@ void
 window_menu_recent_activate_cb (GtkMenuItem *menuitem, gpointer user_data)
 {
         GaleonWindow *window = window_from_widget (GTK_WIDGET (menuitem));
-        HistoryItem *hi = gtk_object_get_user_data (GTK_OBJECT (menuitem));
+        RecentMenuEntry *entry = gtk_object_get_user_data (GTK_OBJECT (menuitem));
 
 	return_if_not_window (window);
-	window_load_url(window, hi->url);
+	window_load_url(window, entry->url);
 }
 
 void 
@@ -272,15 +342,16 @@ window_menu_recent_release_cb (GtkMenuItem *menuitem, GdkEventButton *event,
 			       gpointer user_data)
 {
 	GaleonWindow *window = window_from_widget (GTK_WIDGET (menuitem));
-        HistoryItem *hi = gtk_object_get_user_data (GTK_OBJECT (menuitem));
+        RecentMenuEntry *entry = gtk_object_get_user_data (GTK_OBJECT (menuitem));
 	GtkMenu *menu = GTK_MENU (GTK_WIDGET (menuitem)->parent);
 	
 	return_if_not_window (window);
 
 	gtk_menu_shell_deactivate (GTK_MENU_SHELL (menu));
 	
-	embed_activate_link (window->active_embed, NULL, hi->url,
-		mouse_state_to_link_state (event->button, event->state));
+	embed_activate_link (window->active_embed, NULL, entry->url,
+			     misc_general_mouse_state_to_link_state (
+				     		event->button, event->state));
 }
 
 /**
@@ -312,6 +383,10 @@ void window_menu_history_activate_cb (GtkMenuItem *menuitem,
 		{
 			history_show_dock (window);
 		}
+		else
+		{
+			window_undock (window);
+		}
 	}
 	else
 	{
@@ -349,13 +424,13 @@ window_menu_help_about_activate_cb (GtkMenuItem *menuitem,
 		N_("Yanko Kaneti <yaneti@declera.com>"),
 		"Philip Langdale <philipl@mail.utexas.edu>",
 		"Matthew Mueller <donut@azstarnet.com>",
-		"Daniel Erat  <dan_erat@pobox.com>",
+		"Daniel Erat <dan_erat@pobox.com>",
 		"Jorn Baayen <jorn@nl.linux.org>",
 		" ", /* dont delete the space */
 		N_("Graphic arts: "),
 		/* if your encoding allows it, use eacute (U00E9)
 		 * for the two 'e's of Frederic */
-		N_("Toussaint Frederic (fred@linuxgraphic.org)"),
+		N_("Toussaint Frederic <fred@linuxgraphic.org>"),
 		NULL
 	};
 	
@@ -392,14 +467,14 @@ window_menu_help_about_activate_cb (GtkMenuItem *menuitem,
 		for (i = 0; i < num_authors ; i++)
 			translated_names[i] = _(authors[i]);
 		
-		file = user_file ("about_logo.png", FALSE);
+		file = misc_general_user_file ("about_logo.png", FALSE);
 		about = gnome_about_new 
 			(_(PACKAGE), VERSION,
 			/* Translators: Please change the (C) to a real
 			 * copyright character if your character set allows it
 			 * (Hint: iso-8859-1 is one of the character sets that
 			 * has this symbol). */
-			 _("Copyright (C) 2000-2001 Marco Pesenti Gritti"),
+			 _("Copyright (C) 2000-2002 Marco Pesenti Gritti"),
 				 (const gchar **)translated_names, 
 				 bottom_line, file);
 		g_free (bottom_line);
@@ -412,7 +487,7 @@ window_menu_help_about_activate_cb (GtkMenuItem *menuitem,
 		gtk_widget_show (about);
 	}
 
-	dialog_set_parent (about, window->WMain);
+	dialog_set_parent (about, window->wmain);
 }
 
 /**
@@ -422,98 +497,29 @@ void
 window_menu_help_manual_activate_cb (GtkMenuItem *menuitem, 
 				     GaleonWindow *window)
 {
-	return_if_not_window (window);
 	gnome_help_goto (NULL, "ghelp:galeon-manual");
 }
 
 /**
- * window_menu_help_homepage_activate_cb: go to the Galeon homepage
+ * window_menu_help_generic_release_cb: 
  */
 void
-window_menu_help_homepage_activate_cb (GtkWidget *menuitem, 
-				       GaleonWindow *window)
-{
-	return_if_not_window (window);
-	window_load_url (window, GALEON_HOMEPAGE_URL);
-}
-
-/**
- * window_menu_help_homepage_release_cb: go to the Galeon homepage
- */
-void
-window_menu_help_homepage_release_cb (GtkWidget *menuitem, 
-				      GdkEventButton *event,
-				      GaleonWindow *window)
+window_menu_help_generic_release_cb (GtkWidget *menuitem, 
+				 GdkEventButton *event,
+				 GaleonWindow *window)
 {
+	gchar *url =
+		(gchar*) gtk_object_get_data (GTK_OBJECT (menuitem), "url");
 	GaleonEmbed *new = NULL;
 	GtkMenu *menu = GTK_MENU (menuitem->parent);
 	
 	return_if_not_window (window);
 
-	gtk_menu_shell_deactivate (GTK_MENU_SHELL(menu));
+	gtk_menu_shell_deactivate (GTK_MENU_SHELL (menu));
 	
-	embed_activate_link (window->active_embed, &new, GALEON_HOMEPAGE_URL,
-		mouse_state_to_link_state (event->button, event->state));
-}
-
-/**
- * window_menu_help_bugreport_activate_cb: go to the Galeon bug report page
- */
-void
-window_menu_help_bugreport_activate_cb (GtkWidget *menuitem, 
-					GaleonWindow *window)
-{
-	return_if_not_window (window);
-	window_load_url (window, GALEON_BUGREPORT_URL);
-}
-
-/**
- * window_menu_help_bugreport_release_cb: go to the Galeon bug report page
- */
-void
-window_menu_help_bugreport_release_cb (GtkWidget *menuitem, 
-				       GdkEventButton *event,
-				       GaleonWindow *window)
-{
-	GaleonEmbed *new = NULL;
-	GtkMenu *menu = GTK_MENU (menuitem->parent);
-	
-	return_if_not_window (window);
-
-	gtk_menu_shell_deactivate (GTK_MENU_SHELL(menu));
-	
-	embed_activate_link (window->active_embed, &new, GALEON_BUGREPORT_URL,
-		mouse_state_to_link_state (event->button, event->state));
-}
-
-/**
- * window_menu_help_about_plugins_activate_cb: go to about:plugins
- */
-void
-window_menu_help_about_plugins_activate_cb (GtkWidget *menuitem, 	
-					    GaleonWindow *window)
-{
-	return_if_not_window (window);
-	window_load_url (window, "about:plugins");
-}
-
-/**
- * window_menu_help_about_plugins_release_cb: go to about:plugins
- */
-void
-window_menu_help_about_plugins_release_cb (GtkWidget *menuitem, 	
-					   GdkEventButton *event,
-					   GaleonWindow *window)
-{
-	GaleonEmbed *new = NULL;
-	GtkMenu *menu = GTK_MENU (menuitem->parent);
-	
-	return_if_not_window (window);
-
-	gtk_menu_shell_deactivate (GTK_MENU_SHELL(menu));
-	
-	embed_activate_link (window->active_embed, &new, "about:plugins",
-		mouse_state_to_link_state (event->button, event->state));
+	embed_activate_link (window->active_embed, &new, url,
+		misc_general_mouse_state_to_link_state (event->button,
+							event->state));
 }
 
 /** 
@@ -528,7 +534,10 @@ window_menu_new_window_activate_cb (GtkMenuItem *menuitem,
 	return_if_not_window (window);
 	return_if_not_embed (window->active_embed);
 
-	embed = embed_create_default (window->active_embed, TRUE);
+	embed = embed_create_after_embed (window->active_embed, TRUE, NULL,
+					  EMBED_CREATE_LOAD_DEFAULT_URL |
+					  EMBED_CREATE_FORCE_JUMP |
+					  EMBED_CREATE_RAISE_WINDOW);
 
 	return_if_not_window (embed->parent_window);
 	window_grab_location_focus (embed->parent_window);
@@ -545,7 +554,10 @@ window_menu_new_tab_activate_cb (GtkMenuItem *menuitem,
 	return_if_not_window (window);
 	return_if_not_embed (window->active_embed);
 	window_grab_location_focus (window);
-	embed = embed_create_default (window->active_embed, FALSE);
+	embed = embed_create_after_embed (window->active_embed, FALSE, NULL,
+					  EMBED_CREATE_LOAD_DEFAULT_URL |
+					  EMBED_CREATE_FORCE_JUMP |
+					  EMBED_CREATE_RAISE_WINDOW);
 }
 
 /** 
@@ -596,6 +608,7 @@ window_menu_exit_with_session_activate_cb (GtkMenuItem *menuitem,
 					   GaleonWindow *window)
 {
 	return_if_not_window (window);
+
 	session_quit (TRUE);
 }
 
@@ -605,10 +618,44 @@ window_menu_exit_with_session_activate_cb (GtkMenuItem *menuitem,
 void
 window_menu_print_activate_cb (GtkMenuItem *menuitem, GaleonWindow *window)
 {
+	GaleonEmbed *embed;
+	
 	return_if_not_window (window);
-	print_show_dialog (window);
+	embed = window->active_embed;
+	return_if_not_embed (embed);
+
+	print_show_dialog (embed);
 }
 
+/**
+ * window_menu_send_to_activate_cb: send link to
+ */
+void
+window_menu_send_to_activate_cb (GtkMenuItem *menuitem, GaleonWindow *window)
+{
+	GaleonEmbed *embed;
+	gchar *url, *site;
+
+	return_if_not_window (window);
+	embed = window->active_embed;
+	return_if_not_embed (embed);
+
+	if (embed->location == NULL) return;
+	
+	/* FIXME: i don't like this... it means that the URL could be
+	 * encoded incorrectly (e.g. ?s that are valid in the context of
+	 * the URL will be %HH-escaped).  i don't know what we can do
+	 * instead, though. - dan */
+	site = bookmarks_encode_smb_string (embed->location);
+	url = g_strconcat ("mailto:",
+			   /* xgettext:no-c-format */
+			   _("?Subject=Check%20this%20out%21"),
+			   "&Body=", site, NULL); 
+	gtk_moz_embed_load_url (GTK_MOZ_EMBED (embed->mozembed), url);
+
+	g_free (url);
+	g_free (site);
+}
 
 /** 
  * window_menu_close_activate_cb: close current active embed
@@ -629,7 +676,7 @@ window_menu_back_activate_cb (GtkMenuItem *menuitem, GaleonWindow *window)
 {
 	return_if_not_window (window);
 	return_if_not_embed (window->active_embed);
-	gtk_moz_embed_go_back (GTK_MOZ_EMBED(window->active_embed->mozEmbed));
+	gtk_moz_embed_go_back (GTK_MOZ_EMBED(window->active_embed->mozembed));
 }
 
 /** 
@@ -652,7 +699,7 @@ window_menu_forward_activate_cb (GtkMenuItem *menuitem, GaleonWindow *window)
 	return_if_not_window (window);
 	return_if_not_embed (window->active_embed);
 	gtk_moz_embed_go_forward 
-		(GTK_MOZ_EMBED (window->active_embed->mozEmbed));
+		(GTK_MOZ_EMBED (window->active_embed->mozembed));
 }
 
 /** 
@@ -664,7 +711,7 @@ window_menu_detach_tab_activate_cb (GtkMenuItem *menuitem,
 {
 	return_if_not_window (window);
 	return_if_not_embed (window->active_embed);
-	embed_move_tab (window->active_embed, NULL);
+	embed_move_to_new_window (window->active_embed);
 }
 
 /** 
@@ -742,7 +789,7 @@ window_menu_stop_activate_cb (GtkMenuItem *menuitem,
 {
 	return_if_not_window (window);
 	return_if_not_embed (window->active_embed);
-	gtk_moz_embed_stop_load(GTK_MOZ_EMBED(window->active_embed->mozEmbed));
+	gtk_moz_embed_stop_load(GTK_MOZ_EMBED(window->active_embed->mozembed));
 }
 
 /** 
@@ -773,11 +820,9 @@ window_menu_add_bookmark_activate_cb (GtkMenuItem *menuitem,
 				      GaleonWindow *window)
 {
 	return_if_not_window (window);
-	return_if_not_embed (window->active_embed);
-	g_return_if_fail (default_bookmarks_root != NULL);
 
-	bookmarks_file_bookmark_cb (GTK_WIDGET (menuitem),
-				    default_bookmarks_root);
+	/* use bookmarks file cb */
+	bookmarks_file_bookmark_cb (window->wmain, NULL);
 }
 
 /** 
@@ -973,7 +1018,7 @@ window_view_image_sites_activate_cb (GtkMenuItem *menuItem,
 				      GaleonWindow *window)
 {
 	return_if_not_window (window);
-	pdm_show_page (window, IMAGESITES);
+	pdm_show_page (window, PDM_IMAGESITES);
 }
 
 void
@@ -1017,7 +1062,7 @@ window_view_cookies_activate_cb (GtkMenuItem *menuitem,
 				 GaleonWindow *window)
 {
 	return_if_not_window (window);
-	pdm_show_page (window, COOKIES);
+	pdm_show_page (window, PDM_COOKIES);
 }
 
 void
@@ -1025,7 +1070,7 @@ window_view_cookie_sites_activate_cb (GtkMenuItem *menuitem,
 				      GaleonWindow *window)
 {
 	return_if_not_window (window);
-	pdm_show_page (window, COOKIESITES);
+	pdm_show_page (window, PDM_COOKIESITES);
 }
 
 void
@@ -1068,8 +1113,8 @@ window_menu_zoom_in_activate_cb (GtkCheckMenuItem *menuitem,
 		float zoom;
 		mozilla_get_zoom (embed, &zoom);
 		zoom = zoom*100+10;
-		embed_set_zoom (embed, zoom);
-		history_set_zoom (embed->site_location, zoom);
+		embed_set_zoom (embed, zoom, TRUE);
+		history_set_zoom (embed->location, zoom);
 	}
 }
 
@@ -1083,15 +1128,17 @@ window_menu_zoom_100_activate_cb (GtkCheckMenuItem *menuitem,
 	embed = window->active_embed;
 	return_if_not_embed (embed);
 
-	if (window->zoom_spin)
+	if (window->zoom_spin &&
+	    gtk_spin_button_get_value_as_int (
+		    GTK_SPIN_BUTTON (window->zoom_spin)) != 100)
 	{
 		gtk_spin_button_set_value 
 			(GTK_SPIN_BUTTON (window->zoom_spin), 100);
 	}
 	else
 	{
-		embed_set_zoom (embed, 100);
-		history_set_zoom (embed->site_location, 100);
+		embed_set_zoom (embed, 100, TRUE);
+		history_set_zoom (embed->location, 100);
 	}
 }
 
@@ -1115,8 +1162,8 @@ window_menu_zoom_out_activate_cb (GtkCheckMenuItem *menuitem,
 		float zoom;
 		mozilla_get_zoom (embed, &zoom);
 		zoom = zoom*100-10;
-		embed_set_zoom (embed, zoom);
-		history_set_zoom (embed->site_location, zoom);
+		embed_set_zoom (embed, zoom, TRUE);
+		history_set_zoom (embed->location, zoom);
 	}
 }
 
@@ -1174,14 +1221,14 @@ window_view_passwords_activate_cb (GtkMenuItem *menuItem,
 			           GaleonWindow *window)
 {
 	return_if_not_window (window);
-	pdm_show_page (window, PASSWORDS);
+	pdm_show_page (window, PDM_PASSWORDS);
 }
 void
 window_view_password_sites_activate_cb (GtkMenuItem *menuItem,
 				        GaleonWindow *window)
 {
 	return_if_not_window (window);
-	pdm_show_page (window, PASSWORDSITES);
+	pdm_show_page (window, PDM_PASSWORDSITES);
 }
 
 void
@@ -1199,18 +1246,18 @@ window_menu_save_session_as_activate_cb (GtkWidget *menuitem,
 					 GaleonWindow *window)
 {
 	gchar *file = NULL;
-	/* FIXME: add a pref for this? Marco could kill me... */
-	gchar *sessions_dir = g_concat_dir_and_file (g_get_home_dir (), 
-						     "/.galeon/sessions");
-	if (show_file_picker (window->WMain, _("Save session as..."),
+	gchar *sessions_dir;
+	
+	sessions_dir = g_concat_dir_and_file (g_get_home_dir (), 
+					      "/.galeon/sessions");
+	if (show_file_picker (window->wmain, _("Save session as..."),
 			      sessions_dir, NULL, modeSave,
-			      &file, NULL))
+			      &file, NULL, NULL, NULL))
 	{
-		if (dialog_check_overwrite_file (file, NULL)) 
-		{
-			session_save_to (file);
-			session_history_add (file);
-		}
+		/* we don't need to check overwrite here, since the file
+		 * picker already does it */
+		session_save_to (file);
+		session_history_add (file);
 		g_free (file);
 	}
 	g_free (sessions_dir);
@@ -1221,15 +1268,16 @@ window_menu_open_session_activate_cb (GtkWidget *menuitem,
 				      GaleonWindow *window)
 {
 	gchar *file = NULL;
-	/* FIXME: add a pref for this? Marco could kill me... */
 	gchar *sessions_dir = g_concat_dir_and_file (g_get_home_dir (), 
 						     "/.galeon/sessions");
-	if (show_file_picker (window->WMain, _("Open session..."),
+
+	if (show_file_picker (window->wmain, _("Open session..."),
 			      sessions_dir, NULL, modeOpen,
-			      &file, NULL)) 
+			      &file, NULL, NULL, NULL)) 
 	{
-		session_load_from (file);
-		session_history_add (file);
+		if (session_open_confirmation ())
+			if (session_load_from (file, FALSE))
+				session_history_add (file);
 	}
 	
 	g_free (file);
@@ -1247,16 +1295,28 @@ void
 window_menu_show_javascript_console_cb (GtkWidget *meniutem,
 					GaleonWindow *window)
 {
-	mozilla_show_javascript_console ();
+	GaleonEmbed *embed;
+
+	return_if_not_window (window);
+	embed = window->active_embed;
+	return_if_not_embed (embed);
+
+	js_console_show (embed);
 }
 
-#ifdef ENABLE_EXPERIMENTAL_FILTERS
 void
-window_menu_filters_activate_cb (GtkWidget *menuitem, GaleonWindow *window)
+window_menu_show_autoreload_cb (GtkWidget *meniutem,
+				GaleonWindow *window)
 {
-	filters_show_dialog (window);
+	autoreload_show_dialog (window->active_embed);
+}
+
+void
+window_menu_show_page_info_cb (GtkWidget *menuitem,
+			       GaleonWindow *window)
+{
+	page_info_show_dialog (window->active_embed, PAGE_INFO_GENERAL);
 }
-#endif
 
 void
 window_menu_no_proxies_activate_cb (GtkCheckMenuItem *menuitem,
@@ -1290,3 +1350,42 @@ window_menu_automatic_proxies_activate_cb (GtkCheckMenuItem *menuitem,
 		eel_gconf_set_integer (CONF_NETWORK_PROXY_MODE, 2);
 	}
 }
+
+void
+window_menu_proxy_settings_activate_cb (GtkCheckMenuItem *menuitem,
+					GaleonWindow *window)
+{
+	return_if_not_window (window);
+	prefs_show_page (window, PREFS_PAGE_ID_ADVANCED_NETWORK);
+}
+
+void
+window_menu_related_links_activate_cb (GtkMenuItem *menuitem, 
+				       GaleonWindow *window)
+{
+	GtkMenu *menu;
+	GList *child_list;
+	GList *li;
+	if (menuitem->submenu == NULL) 
+	{
+		menu = GTK_MENU (gtk_menu_new ());
+		gtk_menu_item_set_submenu (menuitem, GTK_WIDGET (menu));
+	}
+	else
+	{
+		menu = GTK_MENU (menuitem->submenu);
+	}
+
+	/* remove old menu items */
+	child_list = gtk_container_children (GTK_CONTAINER (menu));
+	for (li = child_list; li != NULL; li = li->next)
+	{
+		gtk_widget_destroy (GTK_WIDGET (li->data));
+	}
+	g_list_free (child_list);
+
+	/* create menu items */
+	link_interfaces_build_menu (window, menu, TRUE);
+	
+	gtk_menu_reposition (menu);
+}
