@@ -65,17 +65,16 @@ GnomeUIInfo file_menu_uiinfo[] =
 	  window_menu_save_as_activate_cb, NULL, NULL,
 	  GNOME_APP_PIXMAP_STOCK, GNOME_STOCK_MENU_SAVE_AS,
 	  GDK_S, GDK_CONTROL_MASK, NULL },
+	{ GNOME_APP_UI_ITEM, N_("Send link to..."),
+	  N_("Send the current URL by email"),
+	  window_menu_send_to_activate_cb, NULL, NULL,
+	  GNOME_APP_PIXMAP_STOCK, GNOME_STOCK_MENU_MAIL_NEW, 0, 0, NULL },
 	GNOMEUIINFO_SEPARATOR,
 	{ GNOME_APP_UI_ITEM, N_("_Print..."), N_("Print the current page"),
 	  window_menu_print_activate_cb, NULL, NULL,
 	  GNOME_APP_PIXMAP_STOCK, GNOME_STOCK_MENU_PRINT,
 	  GDK_P, GDK_CONTROL_MASK, NULL },
 	GNOMEUIINFO_SEPARATOR,
-	{ GNOME_APP_UI_TOGGLEITEM, N_("_Work offline"),
-	  N_("Toggle offline mode"),
-	  window_menu_offline_mode_activate_cb, NULL, NULL,
-	  GNOME_APP_PIXMAP_NONE, NULL, 0, 0, NULL },
-	GNOMEUIINFO_SEPARATOR,
 	{ GNOME_APP_UI_ITEM, N_("Sa_ve session as..."),
 	  N_("Save the current session to a file"),
 	  window_menu_save_session_as_activate_cb, NULL, NULL,
@@ -84,6 +83,10 @@ GnomeUIInfo file_menu_uiinfo[] =
 	  N_("Restore a previous session from a file"),
 	  window_menu_open_session_activate_cb, NULL, NULL,
 	  GNOME_APP_PIXMAP_NONE, NULL, 0, 0, NULL },
+	{ GNOME_APP_UI_ITEM, N_("Recent sessions"),
+	  N_("Restore a recently-opened or -saved session"),
+	  NULL, NULL, NULL,
+	  GNOME_APP_PIXMAP_NONE, NULL, 0, 0, NULL },
 	GNOMEUIINFO_SEPARATOR,
 	{ GNOME_APP_UI_ITEM, N_("_Close"),
 	  N_("Close the current tab or window"),
@@ -93,7 +96,8 @@ GnomeUIInfo file_menu_uiinfo[] =
 	{ GNOME_APP_UI_ITEM, N_("Exit with _session"),
 	  N_("Exit the program, saving the current session"),
 	  window_menu_exit_with_session_activate_cb, NULL, NULL,
-	  GNOME_APP_PIXMAP_NONE, NULL, 0, 0, NULL },
+	  GNOME_APP_PIXMAP_NONE, NULL, GDK_Q,
+	  GDK_CONTROL_MASK | GDK_SHIFT_MASK, NULL },
 	GNOMEUIINFO_MENU_EXIT_ITEM (window_menu_exit_activate_cb, NULL),
 	GNOMEUIINFO_END
 };
@@ -111,6 +115,20 @@ GnomeUIInfo edit_menu_uiinfo[] =
 	  window_menu_find_activate_cb, NULL, NULL,
 	  GNOME_APP_PIXMAP_STOCK, GNOME_STOCK_MENU_SEARCH,
 	  GDK_F, GDK_CONTROL_MASK, NULL },
+	{ GNOME_APP_UI_ITEM, N_("Find _next"),
+	  N_("Search for the next occurance of a word or phrase in the current page"),
+	  window_menu_find_next_activate_cb, NULL, NULL,
+	  GNOME_APP_PIXMAP_STOCK, GNOME_STOCK_MENU_SEARCH,
+	  GDK_G, GDK_CONTROL_MASK, NULL },
+	  
+	GNOMEUIINFO_SEPARATOR,
+
+	{ GNOME_APP_UI_ITEM, N_("_Select all"),
+	  N_("Select all text in the current page"),
+	  window_menu_select_all_activate_cb, NULL, NULL,
+	  GNOME_APP_PIXMAP_NONE, NULL,
+	  GDK_A, GDK_CONTROL_MASK, NULL },
+	
 	GNOMEUIINFO_END
 };
 
@@ -119,16 +137,16 @@ GnomeUIInfo zoom_menu_uiinfo[] =
 	{ GNOME_APP_UI_ITEM, N_("_In"), N_("Zoom in"),
 	  window_menu_zoom_in_activate_cb, NULL, NULL,
  	  GNOME_APP_PIXMAP_NONE, NULL,
-	  GDK_equal, GDK_CONTROL_MASK, NULL },
+	  GDK_8, GDK_CONTROL_MASK, NULL },
 	  /* xgettext:no-c-format */
 	{ GNOME_APP_UI_ITEM, N_("_100%"), N_("Reset zoom level to 100%"),
 	  window_menu_zoom_100_activate_cb, NULL, NULL,
  	  GNOME_APP_PIXMAP_NONE, NULL,
-	  GDK_0, GDK_CONTROL_MASK, NULL },
+	  GDK_9, GDK_CONTROL_MASK, NULL },
 	{ GNOME_APP_UI_ITEM, N_("_Out"), N_("Zoom out"),
 	  window_menu_zoom_out_activate_cb, NULL, NULL,
 	  GNOME_APP_PIXMAP_NONE, NULL,
-	  GDK_minus, GDK_CONTROL_MASK, NULL },
+	  GDK_0, GDK_CONTROL_MASK, NULL },
 	GNOMEUIINFO_END
 };
 
@@ -170,12 +188,17 @@ GnomeUIInfo view_menu_uiinfo[] =
 	  NULL, NULL, NULL,
 	  GNOME_APP_PIXMAP_NONE, NULL,
 	  0, 0, NULL },
-	{ GNOME_APP_UI_ITEM, N_("_Stylesheet"),
+	{ GNOME_APP_UI_ITEM, N_("St_ylesheet"),
 	  N_("Specify stylesheet settings for the current page"),
 	  NULL, NULL, NULL,
 	  GNOME_APP_PIXMAP_NONE, NULL,
 	  0, 0, NULL },
 	GNOMEUIINFO_SEPARATOR,
+	{ GNOME_APP_UI_ITEM, N_("Page i_nformation..."),
+	  N_("Show information about the current site"),
+	  window_menu_show_page_info_cb, NULL, NULL,
+	  GNOME_APP_PIXMAP_NONE, NULL,
+	  0, 0, NULL },
 	{ GNOME_APP_UI_ITEM, N_("_Page source"),
 	  N_("View the source code of the current page"),
 	  window_menu_view_source_activate_cb, NULL, NULL,
@@ -275,6 +298,19 @@ GnomeUIInfo animate_images_menu_uiinfo[] =
 
 GnomeUIInfo cookies_menu_uiinfo[] =
 {
+	{ GNOME_APP_UI_ITEM, N_("_Block cookies from this site"),
+	  N_("Block cookies from this site"),
+	  window_block_cookies_from_activate_cb, NULL, NULL,
+	  GNOME_APP_PIXMAP_NONE, NULL,
+	  0, 0, NULL },
+	{ GNOME_APP_UI_ITEM, N_("_Allow cookies from this site"),
+	  N_("Allow cookies from this site"),
+	  window_allow_cookies_from_activate_cb, NULL, NULL,
+	  GNOME_APP_PIXMAP_NONE, NULL,
+	  0, 0, NULL },
+
+	GNOMEUIINFO_SEPARATOR,
+
 	{ GNOME_APP_UI_ITEM, N_("View _cookies..."),
 	  N_("View stored cookies"),
 	  window_view_cookies_activate_cb, NULL, NULL,
@@ -285,37 +321,31 @@ GnomeUIInfo cookies_menu_uiinfo[] =
 	  window_view_cookie_sites_activate_cb, NULL, NULL,
 	  GNOME_APP_PIXMAP_NONE, NULL,
 	  0, 0, NULL },
-	GNOMEUIINFO_SEPARATOR,
-	{ GNOME_APP_UI_ITEM, N_("_Allow cookies from this site"),
-	  N_("Allow cookies from this site"),
-	  window_allow_cookies_from_activate_cb, NULL, NULL,
-	  GNOME_APP_PIXMAP_NONE, NULL,
-	  0, 0, NULL },
-	{ GNOME_APP_UI_ITEM, N_("_Block cookies from this site"),
-	  N_("Block cookies from this site"),
-	  window_block_cookies_from_activate_cb, NULL, NULL,
-	  GNOME_APP_PIXMAP_NONE, NULL,
-	  0, 0, NULL },
+
 	GNOMEUIINFO_END
 };
 
 GnomeUIInfo block_images_menu_uiinfo[] =
 {
-	{ GNOME_APP_UI_ITEM, N_("_Allow images from this site"),
-	  N_("Allow images from this site to be loaded"),
-	  window_allow_images_from_activate_cb, NULL, NULL,
-	  GNOME_APP_PIXMAP_NONE, NULL,
-	  0, 0, NULL },
 	{ GNOME_APP_UI_ITEM, N_("_Block images from this site"),
 	  N_("Block images from this site from being loaded"),
 	  window_block_images_from_activate_cb, NULL, NULL,
 	  GNOME_APP_PIXMAP_NONE, NULL,
 	  0, 0, NULL },
+	{ GNOME_APP_UI_ITEM, N_("_Allow images from this site"),
+	  N_("Allow images from this site to be loaded"),
+	  window_allow_images_from_activate_cb, NULL, NULL,
+	  GNOME_APP_PIXMAP_NONE, NULL,
+	  0, 0, NULL },
+
+	GNOMEUIINFO_SEPARATOR,
+
 	{ GNOME_APP_UI_ITEM, N_("_View image sites..."),
 	  N_("View sites for which images will not be loaded"),
 	  window_view_image_sites_activate_cb, NULL, NULL,
 	  GNOME_APP_PIXMAP_NONE, NULL,
 	  0, 0, NULL },
+
 	GNOMEUIINFO_END
 };
 
@@ -358,6 +388,12 @@ GnomeUIInfo proxies_menu_uiinfo[] =
 {
 	{ GNOME_APP_UI_RADIOITEMS, NULL, NULL, proxies_always_uiinfo,
 	  NULL, NULL, 0, NULL, 0, 0, NULL },
+	GNOMEUIINFO_SEPARATOR,
+	{ GNOME_APP_UI_ITEM, N_("_Settings..."),
+	  N_("Detailed proxy configuration"),
+	  window_menu_proxy_settings_activate_cb, NULL, NULL,
+	  GNOME_APP_PIXMAP_NONE, NULL,
+	  0, 0, NULL },
 	GNOMEUIINFO_END
 };
 
@@ -368,17 +404,17 @@ GnomeUIInfo settings_menu_uiinfo[] =
 	  load_images_menu_uiinfo, NULL, NULL,
 	  GNOME_APP_PIXMAP_NONE, NULL,
 	  0, 0, NULL },
-	{ GNOME_APP_UI_SUBTREE, N_("_Animate images"),
+	{ GNOME_APP_UI_SUBTREE, N_("A_nimate images"),
 	  N_("Control whether images are animated "),
 	  animate_images_menu_uiinfo, NULL, NULL,
 	  GNOME_APP_PIXMAP_NONE, NULL,
 	  0, 0, NULL },
-	GNOMEUIINFO_SEPARATOR,
-	{ GNOME_APP_UI_SUBTREE, N_("_Proxy"),
+	{ GNOME_APP_UI_SUBTREE, N_("Pro_xy"),
 	  N_("Control how proxies should be used"),
 	  proxies_menu_uiinfo, NULL, NULL,
 	  GNOME_APP_PIXMAP_NONE, NULL,
 	  0, 0, NULL },
+	GNOMEUIINFO_SEPARATOR,
 	{ GNOME_APP_UI_TOGGLEITEM, N_("Use own _fonts"),
 	  N_("Use default fonts instead of page-requested fonts"),
 	  window_menu_own_fonts_activate_cb, NULL, NULL,
@@ -404,6 +440,10 @@ GnomeUIInfo settings_menu_uiinfo[] =
 	  window_menu_allow_popups_activate_cb, NULL, NULL,
 	  GNOME_APP_PIXMAP_NONE, NULL,
           0, 0, NULL },
+	{ GNOME_APP_UI_TOGGLEITEM, N_("_Work offline"),
+	  N_("Load pages from the cache instead of over the network"),
+	  window_menu_offline_mode_activate_cb, NULL, NULL,
+	  GNOME_APP_PIXMAP_NONE, NULL, 0, 0, NULL },
 	GNOMEUIINFO_SEPARATOR,
 	GNOMEUIINFO_MENU_PREFERENCES_ITEM (window_menu_prefs_activate_cb, NULL),
 	GNOMEUIINFO_END
@@ -462,6 +502,12 @@ GnomeUIInfo go_menu_uiinfo[] =
 	  GNOME_APP_PIXMAP_STOCK, GNOME_STOCK_MENU_STOP,
 	  GDK_Escape, 0, NULL },
 	GNOMEUIINFO_SEPARATOR,
+	{ GNOME_APP_UI_ITEM, N_("Related _links"),
+	  N_("Open the related links menu"),
+	  window_menu_related_links_activate_cb, NULL, NULL,
+	  GNOME_APP_PIXMAP_NONE, NULL,
+	  0, 0, NULL},
+	GNOMEUIINFO_SEPARATOR,
 	GNOMEUIINFO_END
 };
 
@@ -473,18 +519,12 @@ GnomeUIInfo tools_menu_uiinfo[] =
 	  0, 0, NULL },
 	{ GNOME_APP_UI_SUBTREE, N_("_Images"), N_("Image blocking management"),
 	  block_images_menu_uiinfo, NULL, NULL,
-	  GNOME_APP_PIXMAP_NONE, NULL,
-	  0, 0, NULL },
+ 	  GNOME_APP_PIXMAP_NONE, NULL,
+ 	  0, 0, NULL },
 	{ GNOME_APP_UI_SUBTREE, N_("_Passwords"), N_("Password management"),
 	  passwords_menu_uiinfo, NULL, NULL,
 	  GNOME_APP_PIXMAP_NONE, NULL,
 	  0, 0, NULL },
-#ifdef ENABLE_EXPERIMENTAL_FILTERS
-	{ GNOME_APP_UI_ITEM, N_("_Filters..."), N_("Open the filter editor"),
-	  window_menu_filters_activate_cb, NULL, NULL,
-	  GNOME_APP_PIXMAP_NONE, NULL,
-	  0, 0, NULL },
-#endif
 	GNOMEUIINFO_SEPARATOR,
 	{ GNOME_APP_UI_ITEM, N_("_History..."),
 	  N_("Log of visited pages"),
@@ -502,6 +542,12 @@ GnomeUIInfo tools_menu_uiinfo[] =
 	  window_menu_show_javascript_console_cb, NULL, NULL,
 	  GNOME_APP_PIXMAP_NONE, NULL,
 	  0, 0, NULL },
+	GNOMEUIINFO_SEPARATOR,
+	{ GNOME_APP_UI_ITEM, N_("_Autoreload..."),
+	  N_("Automatically reload the current site every so often"),
+	  window_menu_show_autoreload_cb, NULL, NULL,
+	  GNOME_APP_PIXMAP_NONE, NULL,
+	  0, 0, NULL },
 	GNOMEUIINFO_SEPARATOR, 
 	{ GNOME_APP_UI_ITEM, N_("_Rerun first time druid..."),
 	  N_("Run the first time druid again"),
@@ -513,6 +559,11 @@ GnomeUIInfo tools_menu_uiinfo[] =
 
 GnomeUIInfo help_menu_uiinfo[] =
 {
+        { GNOME_APP_UI_ITEM, N_("Galeon _FAQ"),
+          N_("Open the Galeon FAQ (Frequently Asked Questions)"),
+          NULL, NULL, NULL,
+          GNOME_APP_PIXMAP_NONE, NULL,
+          0, 0, NULL },
 	{ GNOME_APP_UI_ITEM, N_("Galeon _manual"), 
 	  N_("Open the Galeon manual"),
 	  window_menu_help_manual_activate_cb, NULL, NULL,
@@ -520,17 +571,17 @@ GnomeUIInfo help_menu_uiinfo[] =
 	  0, 0, NULL },
 	{ GNOME_APP_UI_ITEM, N_("Galeon _home page"),
 	  N_("Go to the Galeon homepage"),
-	  window_menu_help_homepage_activate_cb, NULL, NULL,
+	  NULL, NULL, NULL,
 	  GNOME_APP_PIXMAP_STOCK, GNOME_STOCK_MENU_HOME,
 	  0, 0, NULL },
 	{ GNOME_APP_UI_ITEM, N_("_Report a bug"),
 	  N_("Report a bug in Galeon"),
-	  window_menu_help_bugreport_activate_cb, NULL, NULL,
+	  NULL, NULL, NULL,
 	  GNOME_APP_PIXMAP_NONE, NULL,
 	  0, 0, NULL },
 	{ GNOME_APP_UI_ITEM, N_("About _plugins"),
 	  N_("Show information about the currently installed plugins"),
-	  window_menu_help_about_plugins_activate_cb, NULL, NULL,
+	  NULL, NULL, NULL,
 	  GNOME_APP_PIXMAP_NONE, NULL,
 	  0, 0, NULL },
 	GNOMEUIINFO_MENU_ABOUT_ITEM (window_menu_help_about_activate_cb, NULL),
@@ -544,8 +595,8 @@ GnomeUIInfo menubar_uiinfo[] =
 	GNOMEUIINFO_SUBTREE (N_("_View"), view_menu_uiinfo),
 	GNOMEUIINFO_SUBTREE (N_("_Tab"), tab_menu_uiinfo),
 	GNOMEUIINFO_SUBTREE (N_("_Settings"), settings_menu_uiinfo),
-	GNOMEUIINFO_SUBTREE (N_("_Bookmarks"), bookmarks_menu_uiinfo),
 	GNOMEUIINFO_SUBTREE (N_("_Go"), go_menu_uiinfo),
+	GNOMEUIINFO_SUBTREE (N_("_Bookmarks"), bookmarks_menu_uiinfo),
 	GNOMEUIINFO_SUBTREE (N_("T_ools"), tools_menu_uiinfo),
 	GNOMEUIINFO_SUBTREE (N_("_Help"), help_menu_uiinfo),
 	GNOMEUIINFO_END
