@@ -33,21 +33,9 @@ void window_menu_help_about_activate_cb (GtkMenuItem *menuitem,
 					 GaleonWindow *window);
 void window_menu_help_manual_activate_cb (GtkMenuItem *menuitem,
 					  GaleonWindow *window);
-void window_menu_help_homepage_activate_cb (GtkWidget *menuitem,
-					    GaleonWindow *window);
-void window_menu_help_bugreport_activate_cb (GtkWidget *menuitem,
-					     GaleonWindow *window);
-void window_menu_help_about_plugins_activate_cb (GtkWidget *menuitem,
-					         GaleonWindow *window);
-void window_menu_help_homepage_release_cb (GtkWidget *menuitem,
-				           GdkEventButton *event,
-					   GaleonWindow *window);
-void window_menu_help_bugreport_release_cb (GtkWidget *menuitem,
-					    GdkEventButton *event,
-					    GaleonWindow *window);
-void window_menu_help_about_plugins_release_cb (GtkWidget *menuitem,
-						GdkEventButton *event,
-					        GaleonWindow *window);
+void window_menu_help_generic_release_cb (GtkWidget *menuitem,
+					  GdkEventButton *event,
+					  GaleonWindow *window);
 void window_menu_new_window_activate_cb (GtkMenuItem *menuitem,
                                          GaleonWindow *window);
 void window_menu_new_tab_activate_cb (GtkMenuItem *menuitem,
@@ -70,6 +58,8 @@ void window_menu_exit_with_session_activate_cb (GtkMenuItem *menuitem,
                                                 GaleonWindow *window);
 void window_menu_print_activate_cb (GtkMenuItem *menuitem,  
                                     GaleonWindow *window);
+void window_menu_send_to_activate_cb (GtkMenuItem *menuitem,
+				      GaleonWindow *window);
 void window_menu_close_activate_cb (GtkMenuItem *menuitem,
                                     GaleonWindow *window);  
 void window_menu_view_source_mode_activate_cb (GtkCheckMenuItem *menuitem,
@@ -134,6 +124,10 @@ void window_menu_home_page_activate_cb (GtkMenuItem *menuitem,
                                         GaleonWindow *window);
 void window_menu_find_activate_cb (GtkMenuItem *menuitem,
                                    GaleonWindow *window);
+void window_menu_find_next_activate_cb (GtkMenuItem *menuitem,
+                                        GaleonWindow *window);
+void window_menu_select_all_activate_cb (GtkMenuItem *menuitem,
+                                         GaleonWindow *window);
 gboolean window_menu_edit_expose_event_cb (GtkWidget *editmenu, 
 					   GdkEvent *event,
 					   GaleonWindow *window);
@@ -190,15 +184,26 @@ void window_menu_show_java_console_cb (GtkWidget *meniutem,
 				       GaleonWindow *window);
 void window_menu_show_javascript_console_cb (GtkWidget *meniutem,
 					     GaleonWindow *window);
-void window_menu_filters_activate_cb (GtkWidget *menuitem,
-				      GaleonWindow *window);
+void window_menu_show_autoreload_cb (GtkWidget *meniutem,
+				     GaleonWindow *window);
+void window_menu_show_page_info_cb (GtkWidget *menuitem,
+				    GaleonWindow *window);
 void window_menu_no_proxies_activate_cb (GtkCheckMenuItem *menuitem,
 					 GaleonWindow *window);
 void window_menu_manual_proxies_activate_cb (GtkCheckMenuItem *menuitem,
 					     GaleonWindow *window);
 void window_menu_automatic_proxies_activate_cb (GtkCheckMenuItem *menuitem,
 					        GaleonWindow *window);
-
+void window_menu_proxy_settings_activate_cb (GtkCheckMenuItem *menuitem,
+					     GaleonWindow *window);
+void window_menu_related_links_activate_cb (GtkMenuItem *menu, 
+					    GaleonWindow *window);
+void window_menu_recent_activate_cb (GtkMenuItem *menuitem, 
+				     gpointer user_data);
+void window_menu_recent_release_cb (GtkMenuItem *menuitem, 
+				    GdkEventButton *event,
+				    gpointer user_data);
+	                    		
 /* from window_callbacks.c */
 gboolean window_delete_cb (GtkWidget *widget, GdkEventAny *event,
                            GaleonWindow *window);
