@@ -143,20 +143,21 @@
  * really messy.
  */
 
+extern "C" {
+#include "libgnomevfs/gnome-vfs-mime-handlers.h"
+}
+
 #include "galeon.h"
 #include "mime.h"
 #include "window.h"
 #include "mozilla.h"
 #include "prefs.h"
-#include "misc.h"
 #include "glade.h"
 #include "downloader.h"
 #include "dialog.h"
-#include "ProgressCallbacks.h"
-
-extern "C" {
-#include "libgnomevfs/gnome-vfs-mime-handlers.h"
-}
+#include "misc_string.h"
+#include "ProgressListener2.h"
+#include "ContentHandler.h"
 
 #include <gtk/gtkentry.h>
 #include <gtk/gtktogglebutton.h>
@@ -165,14 +166,11 @@ extern "C" {
 #include <libgnome/gnome-exec.h>
 #include <libgnome/gnome-i18n.h>
 #include <libgnome/gnome-config.h>
+#include <libgnome/gnome-util.h>
+#include <libgnomevfs/gnome-vfs-mime.h>
 
 #include "FilePicker.h"
 
-#include "nsIHelperAppLauncherDialog.h"
-#include "nsIWebProgressListener.h"
-#include "nsIExternalHelperAppService.h"
-#include "nsCExternalHandlerService.h"
-
 #include "nsCRT.h"
 #include "nsCOMPtr.h"
 #include "nsIFactory.h"
@@ -183,6 +181,7 @@ extern "C" {
 
 #include "nsString.h"
 #include "nsIURI.h"
+#include "nsIURL.h"
 #include "nsIMIMEInfo.h"
 #include "nsIChannel.h"
 #include "nsIFTPChannel.h"
@@ -192,7 +191,7 @@ extern "C" {
 #include "nsIDOMWindowInternal.h"
 
 class GContentHandler;
-class GDownloadProgressListener;
+class GDownloadProgressListener2;
 struct MimeAskActionDialog;
 struct HelperAppChooserDialog;
 
@@ -211,10 +210,6 @@ void helper_chooser_radio_list_toggled (GtkToggleButton *button,
 					HelperAppChooserDialog *dialog);
 void helper_chooser_radio_other_toggled (GtkToggleButton *button,
 					 HelperAppChooserDialog *dialog);
-gint progress_dialog_cancel_cb (GtkButton *button,
-				GIProgressCallbacks *aProgress);
-void progress_dialog_pause_cb (GtkButton *button,
-			       GIProgressCallbacks *aProgress);
 }
 
 /**
@@ -248,110 +243,9 @@ struct HelperAppChooserDialog
 	GtkWidget *mCheckAlwaysUse;
 };
 
-class GContentHandler : public nsIHelperAppLauncherDialog
-{
-  public:
- 	NS_DECL_ISUPPORTS
-	NS_DECL_NSIHELPERAPPLAUNCHERDIALOG
-
-	GContentHandler();
-	virtual ~GContentHandler();
-
-	NS_IMETHODIMP LaunchExternalDownloader (void);
-	NS_IMETHODIMP FindHelperApp (void);
-	NS_IMETHODIMP LaunchHelperApp (void);
-	NS_IMETHODIMP ShowHelperProgressDialog (void);
-
-	NS_IMETHODIMP GetLauncher (nsIHelperAppLauncher * *_retval);
-	NS_IMETHODIMP GetContext (nsISupports * *_retval);
-	NS_IMETHODIMP SetHelperApp(GnomeVFSMimeApplication *mHelperApp,
-				   PRBool alwaysUse);
-
-  private:
-	/* additional members */
-	NS_IMETHODIMP Init (void);
-	NS_IMETHODIMP ChooseHelperApp (void);
-	NS_IMETHODIMP ProcessMimeInfo (void);
-	NS_IMETHODIMP MIMEAskAction (void);
-	
-	nsCOMPtr<nsIHelperAppLauncher> mLauncher;
-	nsCOMPtr<nsISupports> mContext;
-
-	nsCOMPtr<nsIURI> mUri;
-	PRInt64 mTimeDownloadStarted;
-	nsCOMPtr<nsIFile> mTempFile;
-
-	MimeItem *mMimeItem;
-
-	char *mUrl;
-	char *mMimeType;
-
-	PRBool mDownloadCanceled;
-	PRBool mHelperProgress;
-
-	nsCOMPtr<nsIWebProgressListener> mListener;
-	
-};
-
-class GDownloadProgressListener : public nsIWebProgressListener,
-				  public nsSupportsWeakReference,
-				  public GIProgressCallbacks
-{
- public:
- 	NS_DECL_ISUPPORTS
-	NS_DECL_NSIWEBPROGRESSLISTENER
-	G_DECL_GIPROGRESSCALLBACKS
-
-	GDownloadProgressListener (nsIHelperAppLauncher *aLauncher,
-				   nsISupports *aContext,
-				   GContentHandler *aHandler,
-				   PRBool aUsedByHelper);
-	virtual ~GDownloadProgressListener ();
- private:
-
-	char * FormatTime (PRUint32 aTime);
-	
-	nsCOMPtr<nsIHelperAppLauncher> mLauncher;
-	nsCOMPtr<nsISupports> mContext;
-	nsCOMPtr<GContentHandler> mHandler;
-	nsCOMPtr<nsIRequest> mRequest;
-	PRBool mUsedByHelper;
-	
-	nsCOMPtr<nsIURI> mUri;
-	PRInt64 mTimeDownloadStarted;
-	nsCOMPtr<nsIFile> mFile;
-	
-	PRInt64 mStartTime;
-	PRInt64 mElapsed;
-	
-	PRInt64 mLastUpdate;
-	PRInt32 mInterval;
-
-	PRFloat64 mPriorKRate;
-	PRInt32 mRateChanges;
-	PRInt32 mRateChangeLimit;
-
-	PRBool mCheckedCanPause;
-	PRBool mCanPause;
-	PRBool mIsPaused;
-
-	GtkWidget *mProgressDialog;
-	GtkWidget *mProgressBar;
-	GtkWidget *mLocation;
-	GtkWidget *mFileName;
-	GtkWidget *mStatus;
-	GtkWidget *mTimeElapsed;
-	GtkWidget *mTimeRemaining;
-	GtkWidget *mPauseButton;
-};
-
-
 /* Implementation file */
 NS_IMPL_ISUPPORTS1(GContentHandler, nsIHelperAppLauncherDialog)
 
-NS_IMPL_ISUPPORTS3(GDownloadProgressListener, nsIWebProgressListener,
-		   GIProgressCallbacks, nsISupportsWeakReference)
-
 GContentHandler::GContentHandler()
 {
 	NS_INIT_ISUPPORTS();
@@ -368,7 +262,6 @@ GContentHandler::~GContentHandler()
 	/* destructor code */
 	g_free (mUri);
 	g_free (mMimeType);
-	mime_item_free (mMimeItem);
 }
 
 ////////////////////////////////////////////////////////////////////////////////
@@ -433,31 +326,48 @@ NS_IMETHODIMP GContentHandler::
 	nsCOMPtr<nsIDOMWindowInternal> aWindowInternal = 
 					do_QueryInterface (aWindowContext);
 	
-	nsCOMPtr<nsIPrefService> prefService = 
-				do_GetService (NS_PREFSERVICE_CONTRACTID);
-	nsCOMPtr<nsIPrefBranch> pref;
-	prefService->GetBranch ("browser.download.", getter_AddRefs (pref));
 	nsCOMPtr<nsILocalFile> saveDir;
-	rv = pref->GetComplexValue ("dir", NS_GET_IID(nsILocalFile),
-				    getter_AddRefs(saveDir));
 
-	if (NS_FAILED(rv))
+	/* first, get the this-session-only download dir from the pref */
+	gchar *dirName = eel_gconf_get_string (CONF_STATE_LAST_DOWNLOAD_DIR);
+	if (dirName && dirName[0] == '\0')
 	{
-		char *dirName = eel_gconf_get_string (CONF_DOWNLOADING_DIR);
-
-		if (!dirName)
-		{
-			dirName = g_strdup (g_get_home_dir());
-		}
-		saveDir = do_CreateInstance (NS_LOCAL_FILE_CONTRACTID);
-		saveDir->InitWithPath (dirName);
 		g_free (dirName);
+		dirName = NULL;
 	}
 
+	/* if we don't have a dir yet, get the normal pref */
+	if (!dirName) dirName = eel_gconf_get_string (CONF_DOWNLOADING_DIR);
+
+	/* if we still don't have a dir, use the home directory */
+	if (!dirName || dirName[0] == '\0')
+	{
+		if (dirName) g_free (dirName);
+		dirName = g_strdup (g_get_home_dir());
+	}
+
+	/* expand the tilde, if there is one */
+	gchar *expanded_directory = misc_string_expand_home_dir (dirName);
+	g_free (dirName);
+
+	/* make sure the directory exists, and use the home directory
+	 * otherwise */
+	if (!expanded_directory ||
+	    !g_file_test (expanded_directory, G_FILE_TEST_ISDIR))
+	{
+		if (expanded_directory) g_free (expanded_directory);
+		expanded_directory = g_strdup (g_get_home_dir());
+	}
+
+	saveDir = do_CreateInstance (NS_LOCAL_FILE_CONTRACTID);
+	saveDir->InitWithPath (expanded_directory);
+	g_free (expanded_directory);
+
 	nsCOMPtr<nsIFilePicker> aFilePicker =
 				do_CreateInstance (G_FILEPICKER_CONTRACTID);
 
-	nsString title = NS_LITERAL_STRING (N_("Save As:"));
+	nsString title = NS_LITERAL_STRING (_("Select the destination "
+					      "filename"));
 
 	aFilePicker->Init (aWindowInternal, title.get(), 
 			   nsIFilePicker::modeSave);
@@ -496,10 +406,17 @@ NS_IMETHODIMP GContentHandler::
 	{
 		nsCOMPtr<nsIFile> aDirectory;
 		rv = aFile->GetParent (getter_AddRefs(aDirectory));
-		nsCOMPtr<nsILocalFile> aLocalDirectory 
-					(do_QueryInterface(aDirectory));
-		rv = pref->SetComplexValue ("dir", NS_GET_IID(nsILocalFile),
-					    aLocalDirectory);
+		if(NS_SUCCEEDED(rv) && aDirectory)
+		{
+			nsXPIDLCString cDirectoryName;
+			rv = aDirectory->GetPath(getter_Copies(cDirectoryName));
+			if(NS_SUCCEEDED(rv))
+			{
+				eel_gconf_set_string (CONF_STATE_LAST_DOWNLOAD_DIR,
+						      cDirectoryName.get());
+			}
+		}
+
 		NS_IF_ADDREF (*_retval = aFile);
 		return NS_OK;
 	};
@@ -510,16 +427,10 @@ NS_IMETHODIMP GContentHandler::
 			ShowProgressDialog(nsIHelperAppLauncher *aLauncher,
 					   nsISupports *aContext)
 {
-	GDownloadProgressListener *aListener =
-				new GDownloadProgressListener (aLauncher,
-							       aContext,
-							       this,
-							       mHelperProgress);
-#if MOZILLA_VERSION < VERSION4(0,9,4,1)
-	NS_ADDREF (aListener);
-	mListener = NS_STATIC_CAST (nsIWebProgressListener *, aListener);
-	NS_RELEASE (aListener);
-#endif
+	DownloadAction aAction =
+		mHelperProgress ? ACTION_SAVEFORHELPER : ACTION_NONE;
+	GProgressListener2 *aListener = new GProgressListener2 ();
+	aListener->InitForDownload (aLauncher, aContext, this, aAction);
 	return NS_OK;
 }
 
@@ -529,7 +440,7 @@ NS_IMETHODIMP GContentHandler::
 
 NS_IMETHODIMP GContentHandler::LaunchExternalDownloader (void)
 {
-	downloader_save_url (mUrl);
+	downloader_external_save_url (mUrl.get());
 	return NS_OK;
 }
 
@@ -537,7 +448,12 @@ NS_IMETHODIMP GContentHandler::LaunchExternalDownloader (void)
 NS_IMETHODIMP GContentHandler::FindHelperApp (void)
 {
 	if (mMimeItem->alwaysUse)
-		return ShowHelperProgressDialog ();
+	{
+		if (mMimeItem->urlHelper)
+			return LaunchHelperApp ();
+		else
+			return ShowHelperProgressDialog ();
+	}
 	else
 		return ChooseHelperApp ();
 }
@@ -559,8 +475,19 @@ NS_IMETHODIMP GContentHandler::LaunchHelperApp (void)
 			appLauncher->DeleteTemporaryFileOnExit(mTempFile);
 		}
 
-		char *aFileName, *document(NULL);
-		mTempFile->GetPath(&aFileName);
+		nsXPIDLCString aFileName;
+		nsCString document;
+
+		mTempFile->GetPath(getter_Copies(aFileName));
+
+		PRBool freeMimeItem(PR_FALSE);
+		if (nsCRT::strcasecmp(mMimeItem->helperApp, "GNOME Default") == 0)
+		{
+			mMimeItem = mime_item_clone (mMimeItem);
+			SetHelperApp (gnome_vfs_mime_get_default_application (mMimeType),
+				      mMimeItem->alwaysUse);
+			freeMimeItem = PR_TRUE;
+		}
 
 		if (mMimeItem->urlHelper)
 		{
@@ -571,7 +498,7 @@ NS_IMETHODIMP GContentHandler::LaunchHelperApp (void)
 			document = aFileName;
 		}
 		char *full_command = g_strconcat (mMimeItem->helperApp,
-						  " ", document, NULL);
+						  " ", document.get(), NULL);
 
 		if (mMimeItem->terminalHelper == FALSE)
 		{
@@ -588,7 +515,11 @@ NS_IMETHODIMP GContentHandler::LaunchHelperApp (void)
 			gnome_execute_async (tmp_dir, 3, argv);
 		}
 		g_free (full_command);
-		g_free (aFileName);
+
+		if (freeMimeItem)
+		{
+			mime_item_free (mMimeItem);
+		}
 	}
 	else
 	{
@@ -632,7 +563,8 @@ NS_IMETHODIMP GContentHandler::SetHelperApp(GnomeVFSMimeApplication *aHelperApp,
 		mMimeItem->urlHelper = TRUE;
 	}
 
-	if (nsCRT::strcmp(aHelperApp->id,"OtHeR") == 0)
+	if (nsCRT::strcmp(aHelperApp->id,"OtHeR") == 0 ||
+	    nsCRT::strcmp(aHelperApp->id,"GNOME Default") == 0)
 	{
 		g_free (aHelperApp->command);
 		g_free (aHelperApp->id);
@@ -653,56 +585,42 @@ NS_IMETHODIMP GContentHandler::Init (void)
 	rv = mLauncher->GetMIMEInfo (getter_AddRefs(aMIMEInfo));
 	rv = aMIMEInfo->GetMIMEType (&mMimeType);
 	
-	ProcessMimeInfo ();
-
 	rv = mLauncher->GetDownloadInfo(getter_AddRefs(mUri),
 					&mTimeDownloadStarted,
 					getter_AddRefs(mTempFile));
-
-	rv = mUri->GetSpec (&mUrl);
+	rv = mUri->GetSpec (mUrl);
 #if 0
 	/* GetSource seems redundant and isn't in 0.9 This code is here while
 	   it remains unclear what GetSource is for. --phil */
 	nsCOMPtr<nsIURI> aUri;
 	rv = mLauncher->GetSource(getter_AddRefs(aUri));
-	rv = aUri->GetSpec (&mUrl);
+	rv = aUri->GetSpec (mUrl);
 #endif	
+	ProcessMimeInfo ();
+
 	return NS_OK;
 }
 
 NS_IMETHODIMP GContentHandler::ProcessMimeInfo (void)
 {
-	/* FIXME: hack for 
-           http://www.xref-tech.com/xrefactory/ftp/xref-i86pcLinux.tar.gz?Cache
-	*/
-	if (mMimeType == NULL)
+	if (mMimeType == NULL ||
+	    !nsCRT::strcmp(mMimeType, "application/octet-stream"))
 	{
-		mMimeType = g_strdup ("unknown/unknown");
+		nsresult rv;
+		nsCOMPtr<nsIURL> aURL = do_QueryInterface(mUri, &rv);
+		if (NS_SUCCEEDED(rv) && aURL)
+		{
+			nsCString uriFileName;
+			aURL->GetFileName(uriFileName);
+			mMimeType = g_strdup
+					(gnome_vfs_mime_type_from_name
+						(uriFileName.get()));
+		}
+		else
+			mMimeType = g_strdup ("application/octet-stream");
 	}
 
-	/* FIXME this it to work around a mozilla bug, when closing popups
-	   sometimes mime types handled internally by mozilla are passed
-	   to the content handler. Force the correct behavior */
-	if (!strcasecmp(mMimeType, "text/html")   ||
-	    !strcasecmp(mMimeType, "text/plain")  ||
-	    !strcasecmp(mMimeType, "application/vnd.mozilla.xul+xml")    ||
-	    !strcasecmp(mMimeType, "text/rdf")    ||
-	    !strcasecmp(mMimeType, "text/xml")    ||
-	    !strcasecmp(mMimeType, "application/xml")    ||
-	    !strcasecmp(mMimeType, "application/xhtml+xml")    ||
-	    !strcasecmp(mMimeType, "text/css")    ||
-	    !strcasecmp(mMimeType, "image/gif")   ||
-	    !strcasecmp(mMimeType, "image/jpeg")  ||
-	    !strcasecmp(mMimeType, "image/png")   ||
-	    !strcasecmp(mMimeType, "application/http-index-format"))
-	{
-		mMimeItem = new MimeItem;
-		mMimeItem->action = MIME_IGNORE;
-		g_warning ("wrong mime type detection !");
-		return NS_OK;
-	}
-
-	mMimeItem = mime_item_get (mMimeType);	
+	mMimeItem = mime_item_get (mMimeType);
 
 	if (!mMimeItem)
 	{
@@ -721,7 +639,7 @@ NS_IMETHODIMP GContentHandler::MIMEAskAction (void)
 	GtkWidget *dialogWidget;
 	gchar *msg;
 		
-	dialog->url = g_strdup (mUrl);
+	dialog->url = !mUrl.IsEmpty() ? g_strdup (mUrl.get()) : NULL;
 	dialog->mime_type = g_strdup (mMimeType);
 	dialog->mContentHandler = this;
 	dialog->gxml = glade_widget_new ("galeon.glade", "mime_ask_action_dialog", 
@@ -734,7 +652,7 @@ NS_IMETHODIMP GContentHandler::MIMEAskAction (void)
 
 	label = glade_xml_get_widget (dialog->gxml, "mime_ask_action_message");
 	msg = g_strdup_printf (_("The URL %s has MIME type %s"),
-			       mUrl, mMimeType);
+			       mUrl.get(), mMimeType);
 	gtk_label_set_text (GTK_LABEL (label), msg);
 	g_free (msg);
 	return NS_OK;
@@ -769,11 +687,16 @@ NS_IMETHODIMP GContentHandler::ChooseHelperApp (void)
 
 	dialog_set_parent (dialog->mDialogWidget, aParentWidget);
 
-	GnomeVFSMimeApplication *app;
-	app = gnome_vfs_mime_get_default_application (mMimeType);
-
 	GtkWidget *menu = gtk_menu_new ();
-	int k = 0;
+
+	if(gnome_vfs_mime_get_default_application (mMimeType))
+	{
+		GtkWidget *defaultItem = gtk_menu_item_new_with_label (
+						_("GNOME Default"));
+		gtk_menu_append (GTK_MENU(menu), defaultItem);
+		gtk_widget_show (defaultItem);
+	}
+
 	for (GList *helperApp = g_list_first(dialog->mHelperApps) ;
 	     helperApp!=NULL ;
 	     helperApp = g_list_next(helperApp))
@@ -781,19 +704,9 @@ NS_IMETHODIMP GContentHandler::ChooseHelperApp (void)
 		GtkWidget *menuItem = gtk_menu_item_new_with_label (
 			((GnomeVFSMimeApplication *)(helperApp->data))->name);
 		gtk_menu_append (GTK_MENU(menu),menuItem);
-
-		char *id = ((GnomeVFSMimeApplication *)(helperApp->data))->id;
-		if (strcmp (id, app->id) == 0)
-		{
-			gtk_menu_set_active ((GtkMenu*)menu, k);
-		}
-		k++;
-
 		gtk_widget_show (menuItem);
 	}
 
-	g_free (app);
-
 	gtk_option_menu_remove_menu (GTK_OPTION_MENU(dialog->mOptionMenu));
 	gtk_option_menu_set_menu (GTK_OPTION_MENU(dialog->mOptionMenu),
 				  menu);
@@ -845,331 +758,6 @@ nsresult NS_NewContentHandlerFactory(nsIFactory** aFactory)
 	return NS_OK;
 }
 
-//------------------------------------------------------------------------------
-
-GDownloadProgressListener::
-	GDownloadProgressListener(nsIHelperAppLauncher *aLauncher,
-				  nsISupports *aContext,
-				  GContentHandler *aHandler,
-				  PRBool aUsedByHelper)
-{
-	NS_INIT_ISUPPORTS();
-	
-	/* member initializers and constructor code */
-	nsresult rv;
-	
-	mInterval = 500000; //in microsecs == 500ms == 0.5s
-	mPriorKRate = 0;
-	mRateChanges = 0;
-	mRateChangeLimit = 2; //Only update rate every second
-	
-	mLauncher = aLauncher;
-	mContext = aContext;
-	mHandler = aHandler;
-	mUsedByHelper = aUsedByHelper;
-
-	rv = mLauncher->GetDownloadInfo(getter_AddRefs(mUri),
-					&mTimeDownloadStarted,
-					getter_AddRefs(mFile));
-
-	nsCOMPtr<nsIDOMWindow> parent = do_QueryInterface (mContext);
-	GtkWidget *aParentWidget = mozilla_find_gtk_parent (parent);
-
-	nsCOMPtr<nsIWebProgressListener> progress =
-			NS_STATIC_CAST(nsIWebProgressListener *, this);
-	nsCOMPtr<GIProgressCallbacks> callbacks =
-			do_QueryInterface (progress);
-	GladeXML *gxml = glade_widget_new ("galeon.glade", 
-					   "download_progress_dialog",
-					   &mProgressDialog, callbacks);
-
-	mProgressBar = glade_xml_get_widget (gxml, "progressbar");
-	mLocation = glade_xml_get_widget (gxml, "location_entry");
-	mFileName = glade_xml_get_widget (gxml, "filename_entry");
-	mStatus = glade_xml_get_widget (gxml, "status_entry");
-	mTimeElapsed = glade_xml_get_widget (gxml, "elapsed_entry");
-	mTimeRemaining = glade_xml_get_widget (gxml, "remaining_entry");
-	mPauseButton = glade_xml_get_widget (gxml, "pause_button");
-	gtk_object_unref (GTK_OBJECT (gxml));
-
-	mCheckedCanPause = PR_FALSE;
-	mCanPause = PR_FALSE;
-	mIsPaused = PR_FALSE;
-
-	PRInt64 now = PR_Now ();
-	mLastUpdate = now;
-#if MOZILLA_VERSION > VERSION4(0,9,4,1)
-	mStartTime = mTimeDownloadStarted;
-#else
-	mStartTime = now;
-#endif
-	mElapsed = now - mStartTime;
-
-	char *text;
-
-	rv = mUri->GetSpec (&text);
-	gtk_entry_set_text (GTK_ENTRY(mLocation),text);
-	g_free (text);
-	
-	rv = mFile->GetPath (&text);
-	gtk_entry_set_text (GTK_ENTRY(mFileName),text);
-	g_free (text);
-	
-	gtk_label_set_text (GTK_LABEL(mTimeElapsed),
-			    FormatTime(mElapsed/1000000));
-	gtk_label_set_text (GTK_LABEL(mTimeRemaining),FormatTime(0));
-
-	dialog_set_parent (mProgressDialog, aParentWidget);
-
-	aLauncher->SetWebProgressListener (this);
-
-	// Increment window count while progress is under way.
-	window_count++;
-}
-
-GDownloadProgressListener::~GDownloadProgressListener()
-{
-	/* destructor code */
-}
-
-/* void onStateChange (in nsIWebProgress aWebProgress, in nsIRequest aRequest, in long aStateFlags, in unsigned long aStatus); */
-NS_IMETHODIMP GDownloadProgressListener::
-			OnStateChange(nsIWebProgress *aWebProgress,
-				      nsIRequest *aRequest,PRInt32 aStateFlags,
-				      PRUint32 aStatus)
-{
-	if (aStateFlags & nsIWebProgressListener::STATE_STOP)
-	{
-		if (GTK_IS_WIDGET(mProgressDialog))
-			gtk_widget_destroy (mProgressDialog);
-		mLauncher->CloseProgressWindow ();
-		if (mUsedByHelper)
-			mHandler->LaunchHelperApp ();
-
-		// Decrement window count as this progess is finished.
-		window_count--;
-
-		// If there are no more windows left, exit galeon.
-		if (g_list_length (all_windows) == 0 && !galeon_server_mode
-		    && window_count ==0)
-		{
-			galeon_exit ();
-		}
-	}
-
-	return NS_OK;
-}
-
-/* void onProgressChange (in nsIWebProgress aWebProgress, in nsIRequest aRequest, in long aCurSelfProgress, in long aMaxSelfProgress, in long aCurTotalProgress, in long aMaxTotalProgress); */
-NS_IMETHODIMP GDownloadProgressListener::
-			OnProgressChange(nsIWebProgress *aWebProgress,
-					 nsIRequest *aRequest,
-					 PRInt32 aCurSelfProgress,
-					 PRInt32 aMaxSelfProgress,
-					 PRInt32 aCurTotalProgress,
-					 PRInt32 aMaxTotalProgress)
-{
-	char *text, *rateStr, *totalStr;
-
-	if (!mCheckedCanPause)
-	{
-		mCheckedCanPause = PR_TRUE;
-
-		nsresult rv;
-		nsCOMPtr<nsIFTPChannel> aChannel = do_QueryInterface (aRequest,
-								      &rv);
-		if (NS_SUCCEEDED(rv))
-		{
-			mCanPause = PR_TRUE;
-			gtk_widget_show (mPauseButton);
-		}
-		else
-		{
-			mCanPause = PR_FALSE;
-			gtk_widget_hide (mPauseButton);
-		}
-	}
-	mRequest = aRequest;
-
-	PRInt64 now = PR_Now ();
-
-	if ((now - mLastUpdate < mInterval) && 
-	     (aMaxTotalProgress != -1) &&  
-	     (aCurTotalProgress < aMaxTotalProgress))
-		return NS_OK;
-
-	mLastUpdate = now;
-	mElapsed = now - mStartTime;
-
-	gtk_label_set_text (GTK_LABEL(mTimeElapsed),
-			    FormatTime(mElapsed/1000000));
-
-	PRInt32 currentKBytes = (PRInt32)(aCurTotalProgress/1024 +.5);
-
-	PRInt32 totalKBytes = (PRInt32)(aMaxTotalProgress/1024 +.5);
-	if (aMaxTotalProgress != -1)
-	{
-		gfloat progress = (gfloat)aCurTotalProgress/
-				  (gfloat)aMaxTotalProgress;
-		gchar *strper = g_strdup_printf (_("%.2f%% - Downloading"), 
-						 progress*100);
-
-		if (progress > 0 && progress < 1)
-			gtk_window_set_title (GTK_WINDOW(mProgressDialog), 
-					      strper);
-		gtk_progress_set_percentage (GTK_PROGRESS(mProgressBar),
-					     progress);
-		totalStr = g_strdup_printf ("%d",totalKBytes);
-		g_free (strper);
-	}
-	else
-	{
-		gtk_progress_set_format_string (GTK_PROGRESS(mProgressBar),
-						"?? %%");
-		totalStr = g_strdup ("??");
-	}
-
-	PRInt64 currentRate;
-	if (mElapsed)
-		currentRate = ((PRInt64)(aCurTotalProgress))*1000000 / mElapsed;
-	else
-		currentRate = 0;
-		
-	if (currentRate)
-	{
-		PRFloat64 currentKRate = ((PRFloat64)currentRate)/1024;
-		if (currentKRate != mPriorKRate)
-		{
-			if (mRateChanges++ == mRateChangeLimit)
-			{
-				mPriorKRate = currentKRate;
-				mRateChanges = 0;
-			}
-			else
-			{
-				currentKRate = mPriorKRate;
-			}
-		}
-		else
-		{
-			mRateChanges = 0;
-		}
-		rateStr = g_strdup_printf("%.2f",currentKRate);
-	}
-	else
-	{
-		rateStr = g_strdup ("??.??");
-	}
-
-	if (mIsPaused)
-	{
-		text = g_strdup (_("Download Paused"));
-	}
-	else
-	{
-		text = g_strdup_printf (_("%dK of %sK bytes at %sK bytes/sec"),
-					currentKBytes, totalStr, rateStr);
-	}
-	gtk_label_set_text (GTK_LABEL(mStatus),text);
-	g_free (text);
-	g_free (totalStr);
-	g_free (rateStr);
-
-	if (currentRate && (aMaxTotalProgress != -1))
-	{
-		PRInt32 remaining = 
-			(PRInt32)((aMaxTotalProgress - aCurTotalProgress)
-				   /currentRate +.5);
-		gtk_label_set_text (GTK_LABEL(mTimeRemaining),
-				    FormatTime(remaining));
-	}
-	else
-	{
-		gtk_label_set_text (GTK_LABEL(mTimeRemaining), _("Unknown"));
-	}
-
-	return NS_OK;
-}
-
-/* void onLocationChange (in nsIWebProgress aWebProgress, in nsIRequest aRequest, in nsIURI location); */
-NS_IMETHODIMP GDownloadProgressListener::
-			OnLocationChange(nsIWebProgress *aWebProgress,
-					 nsIRequest *aRequest, nsIURI *location)
-{
-    return NS_OK;
-}
-
-/* void onStatusChange (in nsIWebProgress aWebProgress, in nsIRequest aRequest, in nsresult aStatus, in wstring aMessage); */
-NS_IMETHODIMP GDownloadProgressListener::
-			OnStatusChange(nsIWebProgress *aWebProgress,
-				       nsIRequest *aRequest, nsresult aStatus,
-				       const PRUnichar *aMessage)
-{
-    return NS_OK;
-}
-
-/* void onSecurityChange (in nsIWebProgress aWebProgress, in nsIRequest aRequest, in long state); */
-NS_IMETHODIMP GDownloadProgressListener::
-			OnSecurityChange(nsIWebProgress *aWebProgress,
-					 nsIRequest *aRequest, PRInt32 state)
-{
-    return NS_OK;
-}
-
-NS_IMETHODIMP GDownloadProgressListener::CancelHelperProgress (void)
-{
-	mUsedByHelper = PR_FALSE;
-	if (mIsPaused)
-		TogglePause ();
-	return mLauncher->Cancel ();
-}
-
-NS_IMETHODIMP GDownloadProgressListener::TogglePause (void)
-{
-	if (!mCheckedCanPause) return NS_ERROR_FAILURE;
-
-	nsresult rv;
-	if (mIsPaused)
-	{
-		rv = mRequest->Resume ();
-		if (NS_SUCCEEDED(rv))
-		{
-			gtk_label_set_text (GTK_LABEL(GTK_BIN(mPauseButton)
-					    ->child),
-					    _("Pause"));
-		}
-	}
-	else
-	{
-		rv = mRequest->Suspend ();
-		if (NS_SUCCEEDED(rv))
-		{
-			gtk_label_set_text (GTK_LABEL(mStatus),
-					    _("Download Paused"));
-			gtk_label_set_text (GTK_LABEL(GTK_BIN(mPauseButton)
-					    ->child),
-					    _("Resume"));
-		}
-	}
-	mIsPaused = !mIsPaused;
-	return rv;
-}
-
-char *GDownloadProgressListener::FormatTime (PRUint32 aTime)
-{
-	PRUint32 secs = (PRUint32)(aTime+.5);
-	PRUint32 hours = secs/3600;
-	secs -= hours*3600;
-	PRUint32 mins = secs/60;
-	secs -= mins*60;
-	char *result;
-	if (hours)
-		result = g_strdup_printf ("%u:%02u.%02u",hours,mins,secs);
-	else
-		result = g_strdup_printf ("%02u.%02u",mins,secs);
-	return result;
-}
-
 ////////////////////////////////////////////////////////////////////////////////
 // begin MIMEAskActionDialog callbacks.
 ////////////////////////////////////////////////////////////////////////////////
@@ -1276,8 +864,17 @@ helper_chooser_ok_clicked_cb (GtkButton *button,
 		}
 		gpointer item = gtk_menu_get_active (GTK_MENU(menu));
 		gint index = g_list_index (menuItems,item);
-		GList *helperApps = g_list_nth(dialog->mHelperApps,index);
-		helperApp = (GnomeVFSMimeApplication *)(helperApps->data);
+		if (index!=0)
+		{
+			GList *helperApps = g_list_nth(dialog->mHelperApps,index-1);
+			helperApp = (GnomeVFSMimeApplication *)(helperApps->data);
+		}
+		else
+		{
+			helperApp = new GnomeVFSMimeApplication;
+			helperApp->id = g_strdup("GNOME Default");
+			helperApp->command = g_strdup(_("GNOME Default"));
+		}
 	}
 	else
 	{
@@ -1394,20 +991,3 @@ void helper_chooser_radio_other_toggled (GtkToggleButton *button,
 	gtk_widget_set_sensitive (dialog->mCheckUseUrl, active);
 }
 
-//////////////////////////////////////////////////////////////////////////////
-// begin ProgressDialog callbacks.
-//////////////////////////////////////////////////////////////////////////////
-
-gint
-progress_dialog_cancel_cb (GtkButton *button,
-			   GIProgressCallbacks *aProgress)
-{
-	aProgress->CancelHelperProgress ();
-	return 0; /* FIXME: philipl, is this the right thing to return? */
-}
-
-void progress_dialog_pause_cb (GtkButton *button,
-			       GIProgressCallbacks *aProgress)
-{
-	aProgress->TogglePause ();
-}
