@@ -16,17 +16,35 @@
  *  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
  */
 
+/* Things to be aware of:
+ *
+ * This filepicker, like the mozilla one, does not make an attempt
+ * to verify the validity of the initial directory you pass it.
+ * It does check that the user doesn't give it a garbage path
+ * during use, but it is the caller's responsibility to give a
+ * sensible initial path.
+ *
+ * At the current moment, we instantiate the filepicker directly
+ * in our contenthandler where there is path verification code
+ * and else where through our C wrapper, which also does verification.
+ * If, at a future date, you need to instantiate filepicker without
+ * using the C wrapper, please verify the initial path. See
+ * ContentHandler for a way to do this.
+ */
+
 #include "galeon.h"
 #include "mozilla.h"
 #include "dialog.h"
-#include "misc.h"
+#include "misc_string.h"
 #include "eel-gconf-extensions.h"
+#include "gfilepicker.h"
 
 #include <gtk/gtkmain.h>
 #include <gtk/gtkcheckbutton.h>
 #include <gtk/gtktogglebutton.h>
 #include <gtk/gtkfilesel.h>
 #include <gtk/gtkhbbox.h>
+#include <gtk/gtkoptionmenu.h>
 #include <libgnome/gnome-i18n.h>
 #include <libgnomeui/gnome-messagebox.h>
 #include <libgnomeui/gnome-dialog-util.h>
@@ -45,6 +63,7 @@
 #include "nsXPIDLString.h"
 #include "nsIPrefService.h"
 #include "nsIURI.h"
+#include "nsIFileURL.h"
 #include "nsIChannel.h"
 #include "nsIFileChannel.h"
 #include "nsNetCID.h"
@@ -52,11 +71,9 @@
 #include "nsIDOMWindow.h"
 #include "nsIDOMWindowInternal.h"
 #include "nsIPromptService.h"
-#if MOZILLA_VERSION > VERSION3(0,9,4)
 #include "nsReadableUtils.h"
-#else
-#include "gStringHelper.h"
-#endif
+
+#include <libgnome/gnome-util.h>
 
 class GFilePicker;
 
@@ -87,18 +104,21 @@ class GFilePicker : public nsIFilePicker
 
 	enum { returnOKSaveContent = 256 };
 
-	GFilePicker(PRBool showContentCheck = PR_FALSE);
+	GFilePicker(PRBool showContentCheck = PR_FALSE, FileFormat *fileFormats = NULL);
 	virtual ~GFilePicker();
 	/* additional members */
 	NS_IMETHODIMP InitWithGtkWidget (GtkWidget *parentWidget, 
 					 const char *title,PRInt16 mode);
 	NS_IMETHODIMP SanityCheck (PRBool *retval);
 
+	PRInt16 mSelectedFileFormat;
+
   private:
 	nsCOMPtr<nsIDOMWindowInternal> mParentInternal;
 	nsCOMPtr<nsIDOMWindow> mParent;
 	nsString mTitle;
 	PRInt16 mMode;
+	FileFormat *mFileFormats;
 
 	nsString mFilter;
 	nsString mDefaultString;
@@ -112,38 +132,22 @@ class GFilePicker : public nsIFilePicker
 
 	GtkWidget *mParentWidget;	
 	GtkWidget *mFileSelector;
+	GtkWidget *mFormatChooser;
 };
 
 /* Implementation file */
 NS_IMPL_ISUPPORTS1(GFilePicker, nsIFilePicker)
 
-GFilePicker::GFilePicker (PRBool showContentCheck) : mSaveContent(PR_FALSE)
+GFilePicker::GFilePicker (PRBool showContentCheck, FileFormat *fileFormats) : mSaveContent(PR_FALSE)
 {
 	NS_INIT_ISUPPORTS();
 	/* member initializers and constructor code */
-	nsCOMPtr<nsIPrefService> prefService = 
-				do_GetService (NS_PREFSERVICE_CONTRACTID);
-	nsCOMPtr<nsIPrefBranch> pref;
-	prefService->GetBranch ("browser.download.", getter_AddRefs (pref));
-	nsresult rv = pref->GetComplexValue ("dir", NS_GET_IID(nsILocalFile),
-					     getter_AddRefs(mDisplayDirectory));
-
-	if (NS_FAILED(rv))
-	{
-		char *dirName = eel_gconf_get_string (CONF_DOWNLOADING_DIR);
-
-		if (!dirName)
-		{
-			dirName = g_strdup (g_get_home_dir());
-		}
-		mDisplayDirectory = do_CreateInstance (NS_LOCAL_FILE_CONTRACTID);
-		mDisplayDirectory->InitWithPath (dirName);
-		g_free (dirName);
-		rv =pref->SetComplexValue ("dir", NS_GET_IID(nsILocalFile),
-					   mDisplayDirectory);
-	}
 
 	mShowContentCheck = showContentCheck;
+	mFileFormats = fileFormats;
+	mFile = do_CreateInstance (NS_LOCAL_FILE_CONTRACTID);
+	mDisplayDirectory = do_CreateInstance (NS_LOCAL_FILE_CONTRACTID);
+	mDisplayDirectory->InitWithPath(g_get_home_dir());
 }
 
 GFilePicker::~GFilePicker()
@@ -166,8 +170,6 @@ NS_IMETHODIMP GFilePicker::Init(nsIDOMWindowInternal *parent,
 	mTitle = title;
 	mMode = mode;
 	
-	mFile = do_CreateInstance (NS_LOCAL_FILE_CONTRACTID);
-	
 	return NS_OK;
 }
 
@@ -191,6 +193,16 @@ NS_IMETHODIMP GFilePicker::AppendFilter(const PRUnichar *title,
 	return NS_OK;
 }
 
+/* attribute long filterIndex; */
+NS_IMETHODIMP GFilePicker::GetFilterIndex(PRInt32 *aFilterIndex)
+{
+	return NS_ERROR_NOT_IMPLEMENTED;
+}
+NS_IMETHODIMP GFilePicker::SetFilterIndex(PRInt32 aFilterIndex)
+{
+	return NS_OK;
+}
+
 /* attribute wstring defaultString; */
 NS_IMETHODIMP GFilePicker::GetDefaultString(PRUnichar * *aDefaultString)
 {
@@ -206,6 +218,20 @@ NS_IMETHODIMP GFilePicker::SetDefaultString(const PRUnichar * aDefaultString)
 	return NS_OK;
 }
 
+#if MOZILLA_VERSION>VERSION3(0,9,8)
+/* attribute wstring defaultExtension; */
+// Again, due to the crippled file selector, we can't really
+// do anything here.
+NS_IMETHODIMP GFilePicker::GetDefaultExtension(PRUnichar * *aDefaultExtension)
+{
+    return NS_ERROR_NOT_IMPLEMENTED;
+}
+NS_IMETHODIMP GFilePicker::SetDefaultExtension(const PRUnichar * aDefaultExtension)
+{
+    return NS_OK;
+}
+#endif
+
 /* attribute nsILocalFile displayDirectory; */
 NS_IMETHODIMP GFilePicker::GetDisplayDirectory(nsILocalFile * *aDisplayDirectory)
 {
@@ -317,6 +343,31 @@ NS_IMETHODIMP GFilePicker::Show(PRInt16 *_retval)
 		gtk_widget_show_all(bbox);
 	}
 
+	if (mFileFormats)
+	{
+		mFormatChooser = gtk_option_menu_new ();
+		GtkMenu *options = GTK_MENU (gtk_menu_new ());
+
+		FileFormat *current = mFileFormats;
+		while (current->description != NULL)
+		{
+			/* FIXME: the label should include the extensions too */
+			gchar *label = current->description;
+			GtkWidget *item = gtk_menu_item_new_with_label (label);
+			gtk_widget_show (item);
+			gtk_menu_append (options, item);
+			current++;
+		}
+		gtk_option_menu_set_menu (GTK_OPTION_MENU (mFormatChooser), GTK_WIDGET (options));
+		gtk_widget_show (mFormatChooser);
+		gtk_box_pack_start (GTK_BOX (GTK_FILE_SELECTION (mFileSelector)->action_area), mFormatChooser,
+				    FALSE, TRUE, 0);
+	}
+	else
+	{
+		mFormatChooser = NULL;
+	}
+
 	if (mMode == nsIFilePicker::modeGetFolder)
 	{
 		gtk_widget_set_sensitive (GTK_FILE_SELECTION(mFileSelector)
@@ -374,7 +425,14 @@ NS_IMETHODIMP GFilePicker::SanityCheck (PRBool *retval)
 
 	PRBool dirExists, fileExists = PR_TRUE;
 
-	mDisplayDirectory->Exists (&dirExists);
+	if (mDisplayDirectory)
+	{
+		mDisplayDirectory->Exists (&dirExists);
+	}
+	else
+	{
+		dirExists = PR_FALSE;
+	}
 
 	if (mMode == nsIFilePicker::modeOpen)
 	{
@@ -471,7 +529,9 @@ filePicker_save_as_ok_cb (GtkButton *button, GFilePicker *aFilePicker)
 	nsresult rv;
 
 	gchar *aFileName = 
-		simplify_uri (gtk_file_selection_get_filename(GTK_FILE_SELECTION(
+		misc_string_simplify_uri (
+			gtk_file_selection_get_filename (
+				GTK_FILE_SELECTION (
 						aFilePicker->mFileSelector)));
 	if (!aFileName || strlen(aFileName) == 0) return;
 
@@ -511,6 +571,26 @@ filePicker_save_as_ok_cb (GtkButton *button, GFilePicker *aFilePicker)
 	aFilePicker->mFileSelectorClosed = PR_TRUE;
 	aFilePicker->mFileSelectorOk = PR_TRUE;
 
+	if (aFilePicker->mFormatChooser)
+	{
+		gint i = 0;		
+		GtkWidget *menu = gtk_option_menu_get_menu 
+			(GTK_OPTION_MENU (aFilePicker->mFormatChooser));
+		GList *iterator = GTK_MENU_SHELL (menu)->children;
+		GtkWidget *selected = gtk_menu_get_active (GTK_MENU(menu));
+
+		while (iterator)
+		{
+			if (iterator->data == selected) 
+			{
+				aFilePicker->mSelectedFileFormat = i;
+				break;
+			}
+			iterator = iterator->next;
+			i++;
+		}
+	}
+
 	gtk_widget_destroy (GTK_WIDGET(aFilePicker->mFileSelector));
 }
 
@@ -551,6 +631,11 @@ void filePicker_save_content_cb (GtkToggleButton *button,
  * @mode: Mode to run filepicker in (modeOpen, modeSave, modeGetFolder)
  * @ret_fullpath: On a successful return, will hold the full path to selected
  *		file or directory.
+ * @file_formats: an array of FileFormat structures to fill the format chooser
+ *              optionmenu. NULL if not needed. The last item must have 
+ *              description == NULL.
+ * @ret_file_format: where to store the index of the format selected (can be
+ *              NULL)
  * returns: TRUE for success, FALSE for failure.
  */
 ////////////////////////////////////////////////////////////////////////////////
@@ -558,23 +643,38 @@ void filePicker_save_content_cb (GtkToggleButton *button,
 extern "C" gboolean
 show_file_picker (GtkWidget *parentWidget, const char* title,
 		  const char* directory, const char* file, FilePickerMode mode,
-		  char * *ret_fullpath, gboolean *ret_save_content)
+		  char * *ret_fullpath, gboolean* ret_save_content, 
+		  FileFormat *file_formats, gint *ret_file_format)
 {
 	PRBool showContentCheck;
+	gchar *expanded_directory;
+
 	if (ret_save_content == NULL)
 		showContentCheck = PR_FALSE;
 	else
 		showContentCheck = PR_TRUE;
 
-	GFilePicker *aFilePicker = new GFilePicker (showContentCheck);
+	GFilePicker *aFilePicker = new GFilePicker (showContentCheck, 
+						    file_formats);
 
 	PRUnichar *uDefaultString = mozilla_locale_to_unicode (file);
 
-	if (!directory)
-		directory = g_strdup (g_get_home_dir());
+	/* expand the tilde, if there is one */
+	expanded_directory = misc_string_expand_home_dir (directory);
+
+	/* make sure the directory exists, and use the home directory
+	 * otherwise */
+	if (!expanded_directory ||
+	    !g_file_test (expanded_directory, G_FILE_TEST_ISDIR))
+	{
+		if (expanded_directory) g_free (expanded_directory);
+		expanded_directory = g_strdup (g_get_home_dir());
+	}
+
 	nsCOMPtr<nsILocalFile> aDir = 
 				do_CreateInstance (NS_LOCAL_FILE_CONTRACTID);
-	aDir->InitWithPath (directory);
+	aDir->InitWithPath (expanded_directory);
+	g_free (expanded_directory);
 
 	aFilePicker->InitWithGtkWidget (parentWidget, title, mode);
 	aFilePicker->SetDefaultString (uDefaultString);
@@ -590,7 +690,11 @@ show_file_picker (GtkWidget *parentWidget, const char* title,
 		else
 			*ret_save_content = FALSE;
 	}
-	
+	if (ret_file_format != NULL)
+	{
+		*ret_file_format = aFilePicker->mSelectedFileFormat;
+	}
+
 	if (retval == nsIFilePicker::returnCancel)
 	{
 		delete aFilePicker;
