@@ -19,7 +19,7 @@
 
 #include "galeon.h"
 #include "mime.h"
-#include "misc.h"
+#include "misc_string.h"
 #include "window.h"
 #include "mozilla.h"
 #include "prefs.h"
@@ -69,11 +69,7 @@ extern "C" {
 #include "nsIDOMWindow.h"
 #include "nsIDOMWindowInternal.h"
 #include "nsIPromptService.h"
-#if MOZILLA_VERSION > VERSION3(0,9,4)
 #include "nsReadableUtils.h"
-#else
-#include "gStringHelper.h"
-#endif
 
 class GFilePicker;
 
@@ -226,6 +222,16 @@ NS_IMETHODIMP GFilePicker::AppendFilter(const PRUnichar *title,
 	return NS_OK;
 }
 
+/* attribute long filterIndex; */
+NS_IMETHODIMP GFilePicker::GetFilterIndex(PRInt32 *aFilterIndex)
+{
+    return NS_ERROR_NOT_IMPLEMENTED;
+}
+NS_IMETHODIMP GFilePicker::SetFilterIndex(PRInt32 aFilterIndex)
+{
+    return NS_ERROR_NOT_IMPLEMENTED;
+}
+
 /* attribute wstring defaultString; */
 NS_IMETHODIMP GFilePicker::GetDefaultString(PRUnichar * *aDefaultString)
 {
@@ -647,9 +653,11 @@ filePicker_save_as_ok_cb (GtkButton *button, GFilePicker *aFilePicker)
 	nsresult rv;
 
 	gchar *aFileName = 
-		simplify_uri (gtk_file_selection_get_filename(GTK_FILE_SELECTION(
+		misc_string_simplify_uri (
+			gtk_file_selection_get_filename (
+				GTK_FILE_SELECTION (
 						aFilePicker->mFileSelector)));
-	if (!aFileName || strlen(aFileName) == 0) return;
+	if (!aFileName || strlen (aFileName) == 0) return;
 
 	if (aFilePicker->mMode == nsIFilePicker::modeSave)
 	{
@@ -840,9 +848,9 @@ GFilePicker::FiltersToMimeTypes()
                 cMimeTypes = g_strconcat (_("All Files"),":application/octet-stream:", 
                                           _("HTML Files"),":text/html:", 
                                           _("Plain Text"),":text/plain:", 
-                                          _("Jpeg Image"),":image/jpeg:", 
-                                          _("Gif Image"),":image/gif:", 
-                                          _("Png Image"),":image/png",
+                                          _("JPEG Image"),":image/jpeg:", 
+                                          _("GIF Image"),":image/gif:", 
+                                          _("PNG Image"),":image/png",
                                           NULL);
         }
         return cMimeTypes; 
