@@ -18,9 +18,11 @@
 
 #include "galeon.h"
 #include "mozilla.h"
+#include "misc_string.h"
 #include "GaleonWrapper.h"
-#include "PrintListener.h"
 #include "GlobalHistory.h"
+#include "ProgressListener2.h"
+#include "PrintProgressListener.h"
 
 #include <gtkmozembed_internal.h>
 #include <unistd.h>
@@ -40,22 +42,24 @@
 #include "nsIPresShell.h"
 #include "nsIMarkupDocumentViewer.h"
 #include "nsIComponentManager.h"
-#include "nsIDOMDocument.h"
 #include "nsIDOMElement.h"
 #include "nsIDOMNodeList.h"
 #include "nsIScriptGlobalObject.h"
+#include "nsIScriptContext.h"
+
 #include "nsIDOMWindowInternal.h"
 #include "nsICharsetConverterManager.h"
 #include "nsICharsetConverterManager2.h"
 #include "nsIInterfaceRequestor.h"
 #include "nsIFocusController.h"
 #include "nsIWebBrowserPersist.h"
-#include "nsIStreamTransfer.h"
+#include "nsCWebBrowserPersist.h"
 #include "nsNetUtil.h"
 #include "nsIChromeEventHandler.h"
 #include "nsIClipboardCommands.h"
 #include "nsIDOMDocumentStyle.h"
 #include "nsIDocShellTreeItem.h"
+#include "nsIDocShellTreeNode.h"
 #include "nsIDocShellTreeOwner.h"
 #include "nsIHTMLContentContainer.h"
 #include "nsICSSLoader.h"
@@ -63,14 +67,18 @@
 #include "nsICSSLoaderObserver.h"
 #include "nsIStyleSet.h"
 #include "nsIDocumentObserver.h"
-#if MOZILLA_VERSION > VERSION3(0,9,4)
-#include "ProgressListener.h"
 #include "nsCWebBrowser.h"
 #include "nsReadableUtils.h"
-#else
-#include "PersistProgress.h"
-#include "gStringHelper.h"
-#endif
+#include "nsIDOMNSHTMLDocument.h"
+#include "nsIDOMHTMLDocument.h"
+#include "nsIDOMHTMLCollection.h"
+#include "nsIDOMHTMLElement.h"
+#include "nsIDOMHTMLImageElement.h"
+#include "nsIDOMHTMLFormElement.h"
+#include "nsIDOMHTMLAnchorElement.h"
+#include "caps/nsIPrincipal.h"
+#include "nsIDeviceContext.h"
+#include "nsIPresContext.h"
 
 GaleonWrapper::GaleonWrapper ()
 {
@@ -87,7 +95,7 @@ nsresult GaleonWrapper::Init (GaleonEmbed *galeon_embed)
 	return_val_if_not_embed (galeon_embed, NS_ERROR_FAILURE);
 
 	embed = galeon_embed;
-	mGtkMozEmbed = GTK_MOZ_EMBED(embed->mozEmbed);
+	mGtkMozEmbed = GTK_MOZ_EMBED(embed->mozembed);
 
 	gtk_moz_embed_get_nsIWebBrowser (mGtkMozEmbed,
 					 getter_AddRefs(mWebBrowser));
@@ -114,23 +122,7 @@ nsresult GaleonWrapper::Init (GaleonEmbed *galeon_embed)
 
 	nsCOMPtr<nsIGlobalHistory> inst =  
 		do_GetService(NS_GLOBALHISTORY_CONTRACTID, &result);
-	dsHistory->SetGlobalHistory(inst);
-
-	nsCOMPtr<nsPIDOMWindow> piWin;
-	GetPIDOMWindow(getter_AddRefs(piWin));
-	if (!piWin) return NS_ERROR_FAILURE;
-  
-        nsCOMPtr<nsIChromeEventHandler> chromeHandler;
-        piWin->GetChromeEventHandler(getter_AddRefs(chromeHandler));
-  
-	mEventReceiver = do_QueryInterface(chromeHandler);
-
-        mEventListener = new GaleonEventListener();
-        mEventListener->Init(this);
-
-	return AddListener ();
-
-	return NS_OK;
+	return dsHistory->SetGlobalHistory(inst);
 }
 
 nsresult GaleonWrapper::GetDocShell (nsIDocShell **aDocShell)
@@ -161,54 +153,6 @@ nsresult GaleonWrapper::GetDocShell (nsIDocShell **aDocShell)
         return NS_OK;
 }
 
-nsresult GaleonWrapper::GetPIDOMWindow(nsPIDOMWindow **aPIWin)
-{
-  *aPIWin = nsnull;
-
-  nsCOMPtr<nsIDOMWindow> domWindow;
-  mWebBrowser->GetContentDOMWindow(getter_AddRefs(domWindow));
-  if (!domWindow)
-    return NS_ERROR_FAILURE;
-
-  nsCOMPtr<nsPIDOMWindow> domWindowPrivate = do_QueryInterface(domWindow);
-  nsCOMPtr<nsIDOMWindowInternal> rootWindow;
-  domWindowPrivate->GetPrivateRoot(getter_AddRefs(rootWindow));
-  
-  nsCOMPtr<nsIChromeEventHandler> chromeHandler;
-  nsCOMPtr<nsPIDOMWindow> piWin(do_QueryInterface(rootWindow));
-
-  *aPIWin = piWin.get();
-
-  if (*aPIWin) {
-    NS_ADDREF(*aPIWin);
-    return NS_OK;
-  }
-
-  return NS_ERROR_FAILURE;
-}
-
-/* This is how gtkmozembed is focusing childs,
-   I'm not using nsIWebBrowserFocus because gtkmozembed
-   doesnt, so I guess something could be buggy there */
-
-nsresult GaleonWrapper::Activate ()
-{
-	nsCOMPtr<nsPIDOMWindow> piWin;
-	GetPIDOMWindow(getter_AddRefs(piWin));
-	if (!piWin) return NS_ERROR_FAILURE;
-
-	return piWin->Activate();
-}
-
-nsresult GaleonWrapper::Deactivate ()
-{
-	nsCOMPtr<nsPIDOMWindow> piWin;
-	GetPIDOMWindow(getter_AddRefs(piWin));
-	if (!piWin) return NS_ERROR_FAILURE;
-
-	return piWin->Deactivate();
-}
-
 nsresult GaleonWrapper::SetOffline(PRBool aOffline)
 {
 	nsresult rv;
@@ -219,7 +163,7 @@ nsresult GaleonWrapper::SetOffline(PRBool aOffline)
 	return io->SetOffline(aOffline);
 }
 
-nsresult GaleonWrapper::Print (nsIPrintOptions *options, PRBool preview)
+nsresult GaleonWrapper::Print (nsIPrintSettings *options, PRBool preview)
 {
 	nsresult result;
 
@@ -227,12 +171,17 @@ nsresult GaleonWrapper::Print (nsIPrintOptions *options, PRBool preview)
 	if (NS_FAILED(result) || !print) return NS_ERROR_FAILURE;
 
 	nsCOMPtr<nsIDOMWindow> DOMWindow;
-	result = GetMainDOMWindow (getter_AddRefs(DOMWindow));
+	result = mWebBrowser->GetContentDOMWindow (getter_AddRefs(DOMWindow));
 	if (NS_FAILED(result) || !DOMWindow) return NS_ERROR_FAILURE;
 
 	if (!preview)
 	{
-		result = print->Print (DOMWindow, options, NULL);
+		GPrintListener *listener = new GPrintListener();
+#if MOZILLA_VERSION > VERSION3(0,9,8)
+		result = print->Print (options, listener);
+#else
+		result = print->Print (DOMWindow, options, listener);
+#endif
 	}
 	else
 	{
@@ -247,18 +196,36 @@ nsresult GaleonWrapper::Print (nsIPrintOptions *options, PRBool preview)
 		close (res);
 		mozilla_delete_temp_file_on_exit (filename);
 
-		nsCString fileString(filename);
-		options->SetToFileName (ToNewUnicode(fileString));
+		PRUnichar *uc_filename = mozilla_locale_to_unicode( filename );
+		options->SetToFileName ( uc_filename );
+		g_free( uc_filename );
 		options->SetPrintToFile (TRUE);
 
-		PrintListener *listener = 
-			new PrintListener(filename);
-		result = print->Print (DOMWindow, options, listener); 
+		GPrintListener *listener =
+			new GPrintListener(filename);
+#if MOZILLA_VERSION > VERSION3(0,9,8)
+		result = print->Print (options, listener);
+#else
+		result = print->Print (DOMWindow, options, listener);
+#endif
+		g_free (filename);
 	}
 
 	return result;
 }
 
+nsresult GaleonWrapper::GetPrintSettings (nsIPrintSettings **options)
+{
+	nsresult result;
+	nsCOMPtr<nsIWebBrowserPrint> print(do_GetInterface(mWebBrowser, &result));
+	if (NS_FAILED(result) || !print) return NS_ERROR_FAILURE;
+#if MOZILLA_VERSION > VERSION3(0,9,8)
+	return print->GetGlobalPrintSettings(options);
+#else
+	return print->GetPrintSettings(options);
+#endif
+}
+
 nsresult GaleonWrapper::GetSHistory (nsISHistory **aSHistory)
 {
 	nsresult result;
@@ -283,46 +250,12 @@ nsresult GaleonWrapper::GetSHistory (nsISHistory **aSHistory)
 
 nsresult GaleonWrapper::Destroy ()
 {
-  	RemoveListener ();
       	mWebBrowser = nsnull;
 	mChromeNav = nsnull;
 	
 	return NS_OK;
 }
 
-nsresult GaleonWrapper::AddListener ()
-{
-	nsresult rv = NS_OK;
-
-        if (!mEventReceiver || !mEventListener) return NS_ERROR_FAILURE;
-
-        nsCOMPtr<nsIDOMEventListener> eventListener =
-                NS_STATIC_CAST(nsIDOMEventListener *,
-                               NS_STATIC_CAST(nsIDOMDragListener *, mEventListener));
-
-        rv = mEventReceiver->AddEventListenerByIID(eventListener,
-                                                 NS_GET_IID(nsIDOMDragListener));
-        if (NS_FAILED(rv)) return NS_ERROR_FAILURE;
-	
-	return NS_OK;
-}
-
-nsresult GaleonWrapper::RemoveListener ()
-{
-	nsresult rv = NS_OK;
-
-	if (!mEventReceiver || !mEventListener) return NS_ERROR_FAILURE;
-        
-        nsCOMPtr<nsIDOMEventListener> eventListener =
-                NS_STATIC_CAST(nsIDOMEventListener *,
-                               NS_STATIC_CAST(nsIDOMDragListener *, mEventListener));
-
-        rv = mEventReceiver->RemoveEventListenerByIID(eventListener,
-                                                      NS_GET_IID(nsIDOMDragListener));
-
-	return rv;
-}
-
 nsresult GaleonWrapper::GoToHistoryIndex (PRInt16 index)
 {
 	nsresult result;
@@ -338,7 +271,7 @@ nsresult GaleonWrapper::GoToHistoryIndex (PRInt16 index)
 	return  ContentNav->GotoIndex (index);
 }
 
-nsresult GaleonWrapper::SetZoom (float aZoom)
+nsresult GaleonWrapper::SetZoom (float aZoom, PRBool reflow)
 {
 	nsresult result;
 
@@ -346,15 +279,56 @@ nsresult GaleonWrapper::SetZoom (float aZoom)
 	result = GetDocShell (getter_AddRefs(DocShell));
 	if (NS_FAILED(result) || !DocShell) return NS_ERROR_FAILURE;
 
-	nsCOMPtr<nsIContentViewer> contentViewer;	
-	result = DocShell->GetContentViewer (getter_AddRefs(contentViewer));
-	if (!NS_SUCCEEDED (result) || !contentViewer) return NS_ERROR_FAILURE;
+	if (reflow)
+	{
+		nsCOMPtr<nsIContentViewer> contentViewer;	
+		result = DocShell->GetContentViewer (getter_AddRefs(contentViewer));
+		if (!NS_SUCCEEDED (result) || !contentViewer) return NS_ERROR_FAILURE;
 
-	nsCOMPtr<nsIMarkupDocumentViewer> mdv = do_QueryInterface(contentViewer,
+		nsCOMPtr<nsIMarkupDocumentViewer> mdv = do_QueryInterface(contentViewer,
 								  &result);
-	if (NS_FAILED(result) || !mdv) return NS_ERROR_FAILURE;
+		if (NS_FAILED(result) || !mdv) return NS_ERROR_FAILURE;
 
-	return mdv->SetTextZoom (aZoom);
+		return mdv->SetTextZoom (aZoom);
+	}
+	else
+	{
+		SetZoomOnDocshell (aZoom, DocShell);
+
+		nsCOMPtr<nsIDocShellTreeNode> docShellNode(do_QueryInterface(DocShell));
+		if (docShellNode)
+		{
+			PRInt32 i;
+			PRInt32 n;
+			docShellNode->GetChildCount(&n);
+			for (i=0; i < n; i++) 
+			{
+				nsCOMPtr<nsIDocShellTreeItem> child;
+				docShellNode->GetChildAt(i, getter_AddRefs(child));
+				nsCOMPtr<nsIDocShell> childAsShell(do_QueryInterface(child));
+				if (childAsShell) 
+				{
+					return SetZoomOnDocshell (aZoom, childAsShell);
+				}
+			}
+		}
+	}
+
+	return NS_OK;
+}
+
+nsresult GaleonWrapper::SetZoomOnDocshell (float aZoom, nsIDocShell *DocShell)
+{
+	nsresult result;
+
+	nsCOMPtr<nsIPresContext> PresContext;
+	result = DocShell->GetPresContext (getter_AddRefs(PresContext));
+	if (NS_FAILED(result)) return NS_ERROR_FAILURE;
+					
+	nsCOMPtr<nsIDeviceContext> DeviceContext;
+	result = PresContext->GetDeviceContext (getter_AddRefs(DeviceContext));
+
+	return DeviceContext->SetTextZoom (aZoom);
 }
 
 nsresult GaleonWrapper::GetZoom (float *aZoom)
@@ -376,36 +350,43 @@ nsresult GaleonWrapper::GetZoom (float *aZoom)
 	return mdv->GetTextZoom (aZoom);
 }
 
-nsresult GaleonWrapper::GetDOMWindow (nsIDOMWindow **aDOMWindow)
+nsresult GaleonWrapper::GetFocusedDOMWindow (nsIDOMWindow **aDOMWindow)
 {
 	nsresult rv;
-
+	
 	nsCOMPtr<nsIWebBrowserFocus> focus = do_GetInterface(mWebBrowser, &rv);
 	if (NS_FAILED(rv) || !focus) return NS_ERROR_FAILURE;
 
 	rv = focus->GetFocusedWindow (aDOMWindow);
 	if (NS_FAILED(rv))
-		rv = GetMainDOMWindow (aDOMWindow);
+		rv = mWebBrowser->GetContentDOMWindow (aDOMWindow);
 	return rv;
 }
 
-nsresult GaleonWrapper::GetMainDOMWindow (nsIDOMWindow **aDOMWindow)
-{
-	return mWebBrowser->GetContentDOMWindow (aDOMWindow);
-}
-
 nsresult GaleonWrapper::GetDOMDocument (nsIDOMDocument **aDOMDocument)
 {
 	nsresult result;
 
-	nsCOMPtr<nsIDOMWindow> DOMWindow;
-	result = GetDOMWindow (getter_AddRefs(DOMWindow));
-	if (NS_FAILED(result) || !DOMWindow)
+	/* Use the current target document */
+	if (mTargetDocument)
 	{
-		return GetMainDOMDocument (aDOMDocument);
+		*aDOMDocument = mTargetDocument.get();
+
+		NS_IF_ADDREF(*aDOMDocument);
+
+		return NS_OK;
 	}
 
-	return DOMWindow->GetDocument (aDOMDocument);
+	/* Use the focused document */
+	nsCOMPtr<nsIDOMWindow> DOMWindow;
+	result = GetFocusedDOMWindow (getter_AddRefs(DOMWindow));
+	if (NS_SUCCEEDED(result) && DOMWindow)
+	{
+		return DOMWindow->GetDocument (aDOMDocument);
+	}
+
+	/* Use the main document */
+	return GetMainDOMDocument (aDOMDocument);
 }
 
 nsresult GaleonWrapper::GetMainDOMDocument (nsIDOMDocument **aDOMDocument)
@@ -423,7 +404,8 @@ nsresult GaleonWrapper::GetMainDOMDocument (nsIDOMDocument **aDOMDocument)
 	return contentViewer->GetDOMDocument (aDOMDocument);
 }
 
-nsresult GaleonWrapper::SaveURI (const char *url, char *filename, int action)
+nsresult GaleonWrapper::SaveURI (const char *url, char *filename, DownloadAction action,
+				 gpointer info)
 {
         nsresult rv;
 
@@ -433,7 +415,6 @@ nsresult GaleonWrapper::SaveURI (const char *url, char *filename, int action)
 	rv = NS_NewURI(getter_AddRefs(linkURI), s);
 	if (NS_FAILED(rv) || !linkURI) return NS_ERROR_FAILURE;
 
-#if MOZILLA_VERSION > VERSION3(0,9,4)
 	nsCOMPtr<nsIWebBrowserPersist> persist = 
 			do_CreateInstance(NS_WEBBROWSERPERSIST_CONTRACTID, &rv);
 	if (NS_FAILED(rv) || !persist) return NS_ERROR_FAILURE;
@@ -443,49 +424,39 @@ nsresult GaleonWrapper::SaveURI (const char *url, char *filename, int action)
 	if (NS_FAILED(rv) || !file) return NS_ERROR_FAILURE;
 
 	nsCOMPtr<nsIDOMWindow> parent;
-	rv = GetDOMWindow (getter_AddRefs(parent));
+	rv = mWebBrowser->GetContentDOMWindow (getter_AddRefs(parent));
 
-	mProgress =
-		new GPersistProgressListener (persist, parent,
-					       linkURI, file, action);
-	return persist->SaveURI(linkURI, nsnull, file);
-#else
-	nsCOMPtr<nsIWebBrowserPersist> persist = do_GetInterface(mWebBrowser, &rv);
-	if (NS_FAILED(rv) || !persist) return NS_ERROR_FAILURE;
+	GProgressListener2 *aProgress = new GProgressListener2 ();
+	aProgress->InitForPersist (persist, parent,
+					       linkURI, file,
+					       action, info);
+	mProgress = aProgress;
 
-	nsCOMPtr<nsIWebBrowserPersistProgress> progress;
-
-	if (action)
+	switch (action)
 	{
-		progress = new PersistProgress(filename, action);
-		persist->SetProgressListener (progress);
+	case ACTION_VIEWSOURCE:
+		persist->SetPersistFlags (nsIWebBrowserPersist::PERSIST_FLAGS_FROM_CACHE);
+		break;
+	case ACTION_SET_PIXMAP:
+		break;
+	case ACTION_FAVICON_EDITOR:
+	case ACTION_FAVICON:
+		persist->SetPersistFlags (nsIWebBrowserPersist::PERSIST_FLAGS_BYPASS_CACHE);
+		break;
+	default:
+		size_t len = strlen (filename);
+		if((filename[len-1] == 'z' && filename[len-2] == 'g') ||
+		   (filename[len-1] == 'Z' && filename[len-2] == 'G'))
+			persist->SetPersistFlags (nsIWebBrowserPersist::PERSIST_FLAGS_NO_CONVERSION);
+		else
+			persist->SetPersistFlags (nsIWebBrowserPersist::PERSIST_FLAGS_NONE);
+		break;
 	}
-	return persist->SaveURI(linkURI, nsnull, filename);
-#endif
-}
-
-nsresult GaleonWrapper::SaveLink (const char *url)
-{
-	nsresult rv;
-
-	nsCOMPtr<nsIDOMWindow> DOMWindow;
-	rv = mWebBrowser->GetContentDOMWindow (getter_AddRefs(DOMWindow));
-	if (NS_FAILED(rv) || !DOMWindow) return NS_ERROR_FAILURE;
-
-	nsCOMPtr<nsIDOMWindowInternal> DOMWInternal =
-						do_QueryInterface (DOMWindow);
-	if (!DOMWInternal) return NS_ERROR_FAILURE;
-
-	nsCOMPtr<nsIStreamTransfer> xfer = 
-				do_GetService (NS_ISTREAMTRANSFER_CONTRACTID);
-	if (!xfer) return NS_ERROR_FAILURE;
-
-	return xfer->SelectFileAndTransferLocationSpec 
-		(url, DOMWInternal, "", "", PR_TRUE, nsnull, nsnull);
+	return persist->SaveURI (linkURI, nsnull, file);
 }
 
 nsresult GaleonWrapper::SaveDocument (char *filename, const char *datapath,
-				      int action, PRBool mainDoc)
+				      DownloadAction action, PRBool mainDoc)
 {
 	nsresult rv;
 
@@ -496,7 +467,6 @@ nsresult GaleonWrapper::SaveDocument (char *filename, const char *datapath,
 		rv = GetDOMDocument (getter_AddRefs(aDOMDocument));
 	if (NS_FAILED(rv) || !aDOMDocument) return NS_ERROR_FAILURE;
 
-#if MOZILLA_VERSION > VERSION3(0,9,4)
 	nsCOMPtr<nsIWebBrowserPersist> persist = 
 			do_CreateInstance(NS_WEBBROWSERPERSIST_CONTRACTID, &rv);
 	if (NS_FAILED(rv) || !persist) return NS_ERROR_FAILURE;
@@ -523,27 +493,14 @@ nsresult GaleonWrapper::SaveDocument (char *filename, const char *datapath,
 		if (NS_FAILED(rv) || !aURI) return NS_ERROR_FAILURE;
 
 		nsCOMPtr<nsIDOMWindow> parent;
-		GetDOMWindow (getter_AddRefs(parent));
+		mWebBrowser->GetContentDOMWindow (getter_AddRefs(parent));
 
-		mProgress =
-			new GPersistProgressListener (persist, parent,
-						       aURI, file, action);
+		GProgressListener2 *aProgress = new GProgressListener2 ();
+		aProgress->InitForPersist (persist, parent,
+					       aURI, file, action, NULL);
+		mProgress = aProgress;
 	}
-	return persist->SaveDocument (aDOMDocument, file, path);
-#else
-	nsCOMPtr<nsIWebBrowserPersist> persist =
-			do_GetInterface(mWebBrowser, &rv);
-	if (NS_FAILED(rv) || !persist) return NS_ERROR_FAILURE;
-
-	nsCOMPtr<nsIWebBrowserPersistProgress> progress;
-
-	if (action)
-	{
-		progress = new PersistProgress(filename, action);
-		persist->SetProgressListener (progress);
-	}
-	return persist->SaveDocument (aDOMDocument, filename, datapath);
-#endif
+	return persist->SaveDocument (aDOMDocument, file, path, nsnull, 0, 0);
 }
 
 nsresult GaleonWrapper::GetSHInfo (PRInt32 *count, PRInt32 *index)
@@ -579,7 +536,7 @@ nsresult GaleonWrapper::GetSHTitleAtIndex (PRInt32 index, PRUnichar **title)
 	return NS_OK;
 }
 
-nsresult GaleonWrapper::GetSHUrlAtIndex (PRInt32 index, char **url)
+nsresult GaleonWrapper::GetSHUrlAtIndex (PRInt32 index, nsCString &url)
 {
 	nsresult result;
 
@@ -590,14 +547,14 @@ nsresult GaleonWrapper::GetSHUrlAtIndex (PRInt32 index, char **url)
 	nsCOMPtr<nsIHistoryEntry> he;
 	result = SessionHistory->GetEntryAtIndex (index, PR_FALSE,
 						  getter_AddRefs (he));
-	if (!NS_SUCCEEDED(result) || (!he)) return NS_ERROR_FAILURE;
+	if (NS_FAILED(result) || (!he)) return NS_ERROR_FAILURE;
 
 	nsCOMPtr<nsIURI> uri;
 	result = he->GetURI (getter_AddRefs(uri));
-	if (!NS_SUCCEEDED(result) || (!uri)) return NS_ERROR_FAILURE;
+	if (NS_FAILED(result) || (!uri)) return NS_ERROR_FAILURE;
 
 	result = uri->GetSpec(url);
-	if (!NS_SUCCEEDED(result) || (!*url)) return NS_ERROR_FAILURE;
+	if (NS_FAILED(result) || url.IsEmpty()) return NS_ERROR_FAILURE;
 
 	return NS_OK;
 }
@@ -647,7 +604,7 @@ nsresult GaleonWrapper::ReloadDocument ()
 	return NS_OK;
 }
 
-nsresult GaleonWrapper::GetMainDocumentUrl (char **url)
+nsresult GaleonWrapper::GetMainDocumentUrl (nsCString &url)
 {
 	nsresult result;
 
@@ -665,7 +622,27 @@ nsresult GaleonWrapper::GetMainDocumentUrl (char **url)
 	return uri->GetSpec (url);
 }
 
-nsresult GaleonWrapper::GetDocumentUrl (char **url)
+nsresult GaleonWrapper::GetDocumentUrl (nsCString &url)
+{
+        nsresult result;
+
+        nsCOMPtr<nsIDOMDocument> aDOMDocument;
+
+        result = GetDOMDocument (getter_AddRefs(aDOMDocument));
+        if (NS_FAILED(result) || !aDOMDocument) return NS_ERROR_FAILURE;
+
+        nsCOMPtr<nsIDocument> doc = do_QueryInterface(aDOMDocument);
+        if(!doc) return NS_ERROR_FAILURE;
+
+        nsCOMPtr<nsIURI> uri;
+        doc->GetDocumentURL(getter_AddRefs(uri));
+
+        uri->GetSpec (url);
+
+        return NS_OK;
+}
+
+nsresult GaleonWrapper::GetDocumentTitle (char **title)
 {
 	nsresult result;
 
@@ -677,10 +654,10 @@ nsresult GaleonWrapper::GetDocumentUrl (char **url)
 	nsCOMPtr<nsIDocument> doc = do_QueryInterface(aDOMDocument);
 	if(!doc) return NS_ERROR_FAILURE;
 
-	nsCOMPtr<nsIURI> uri;
-	doc->GetDocumentURL(getter_AddRefs(uri));
+	const nsString* t;
+	t = doc->GetDocumentTitle();
 
-	uri->GetSpec (url);
+	*title = mozilla_unicode_to_locale (t->get());
 
 	return NS_OK;
 }
@@ -761,13 +738,13 @@ nsresult GaleonWrapper::SetSitePermission (gboolean block, PRInt32 type)
 	nsCOMPtr<nsIURI> uri;
 	doc->GetDocumentURL(getter_AddRefs(uri));
 
-	char *url;
-	uri->GetSpec (&url);
+	nsCString url;
+	uri->GetSpec (url);
 	
 	nsCOMPtr<nsIPermissionManager> permissionManager =
 			do_CreateInstance (NS_PERMISSIONMANAGER_CONTRACTID);
 	
-	result = permissionManager->Add (url,block ? PR_TRUE : PR_FALSE, type);
+	result = permissionManager->Add (url.get(), block ? PR_TRUE : PR_FALSE, type);
 
 	return result;
 }
@@ -831,11 +808,10 @@ nsresult GaleonWrapper::Paste(void)
 	return clipboard->Paste ();
 }
 
-nsresult GaleonWrapper::GetFaviconURL (char **favicon_url)
+nsresult GaleonWrapper::GetFaviconURL (nsCString &favicon_url)
 {
 	nsresult result;
 	PRUint32 links_count;
-	char *relstr;
 
 	nsCOMPtr<nsIDOMDocument> aDOMDocument;
 	result = GetMainDOMDocument (getter_AddRefs(aDOMDocument));
@@ -865,41 +841,33 @@ nsresult GaleonWrapper::GetFaviconURL (char **favicon_url)
 		nsAutoString attr;
 		attr.AssignWithConversion("rel");
 		nsAutoString value;
-		linkElement->GetAttribute (attr, value);
-		relstr = ToNewCString (value);
-		g_strup (relstr);
+		result = linkElement->GetAttribute (attr, value);
+		if (NS_FAILED(result)) return NS_ERROR_FAILURE;
 
-		if (strcmp(relstr, "SHORTCUT ICON") == 0)
+		if (value.EqualsWithConversion ("SHORTCUT ICON", PR_TRUE) ||
+		    value.EqualsWithConversion ("ICON", PR_TRUE))
 		{
-			char *link = NULL;
-			
-			nsMemory::Free (relstr);
-
 			nsAutoString attr;
 			attr.AssignWithConversion("href");
 			nsAutoString value;
-			linkElement->GetAttribute (attr, value);
-			if (value.IsEmpty()) continue;
-
-			link = ToNewCString (value);
-
-			if (!link)
-			{
-				*favicon_url = NULL;
-				return NS_OK;
-			}
+			result = linkElement->GetAttribute (attr, value);
+			if (NS_FAILED (result) || value.IsEmpty())
+				return NS_ERROR_FAILURE;
 			
+			nsCString link;
+			link.AssignWithConversion(value);
+
 			nsCOMPtr<nsIDocument> doc = 
 				do_QueryInterface (aDOMDocument);
 			if(!doc) return NS_ERROR_FAILURE;
 			
 			nsCOMPtr<nsIURI> uri;
-			doc->GetDocumentURL(getter_AddRefs(uri));
+			result = doc->GetDocumentURL(getter_AddRefs(uri));
+			if (NS_FAILED (result)) return NS_ERROR_FAILURE;
 
 			result = uri->Resolve (link, favicon_url);
 			if (NS_FAILED (result)) return NS_ERROR_FAILURE;
 			
-			nsMemory::Free (link);
 			return NS_OK;
 		}
 	}
@@ -970,12 +938,16 @@ nsresult GaleonWrapper::LoadOverrideStyleSheet(char *css,
 	result = ps->GetStyleSet(getter_AddRefs(styleSet));
 	if (NS_FAILED(result) || !styleSet) return NS_ERROR_FAILURE;
 
-	styleSet->AppendOverrideStyleSheet(styleSheet);
+	/* Override - Agent */
+	styleSet->AppendUserStyleSheet(styleSheet);
 	styleSheet->SetOwningDocument(doc);
 
-	/* hack to notify mozilla that stylesheets have changed */
+	/* notify mozilla that stylesheets have changed */
 	styleSheet->SetEnabled(PR_FALSE);
 	styleSheet->SetEnabled(PR_TRUE);
+	/* This is Right[tm], but not working for some people (?!)
+	doc->SetStyleSheetDisabledState(styleSheet, PR_FALSE);
+	*/
 
 	*return_sheet = (nsIStyleSheet*)styleSheet;
 	
@@ -1013,3 +985,593 @@ nsresult GaleonWrapper::RemoveOverrideStyleSheet(nsIStyleSheet *remove)
 
 	return NS_OK;
 }
+
+nsresult GaleonWrapper::GetLinkInterfaceItems (GList **list)
+{
+	nsresult result;
+	PRUint32 links_count;
+	char *relstr;
+
+	/* we accept these rel=.. elements, specified by the w3c */
+	const gchar *rel_types[] = {
+		"START", "NEXT", "PREV", "PREVIOUS", "CONTENTS", "TOC", "INDEX",
+		"GLOSSARY", "COPYRIGHT", "CHAPTER",  "SECTION",
+		"SUBSECTION", "APPENDIX", "HELP", "TOP", "SEARCH", "MADE",
+		"BOOKMARK", "HOME",
+		NULL /* terminator, must be last */
+	};
+
+	nsCOMPtr<nsIDOMDocument> aDOMDocument;
+	result = GetMainDOMDocument (getter_AddRefs(aDOMDocument));
+	if (NS_FAILED(result) || !aDOMDocument) return NS_ERROR_FAILURE;
+
+	/* get list of link elements*/
+	nsString strname;
+	strname.AssignWithConversion("LINK");
+
+	nsCOMPtr<nsIDOMNodeList> aLinks;
+	result = aDOMDocument->GetElementsByTagName (strname, 
+						     getter_AddRefs (aLinks));
+	if (NS_FAILED (result)) return NS_ERROR_FAILURE;
+
+	result = aLinks->GetLength (&links_count);
+	if (NS_FAILED (result)) return NS_ERROR_FAILURE;
+
+	for (PRUint32 i = 0; i < links_count; i++)
+	{
+		/* get to the link element */
+		nsCOMPtr<nsIDOMNode> aLink;
+		result = aLinks->Item (i, getter_AddRefs (aLink));
+		if (NS_FAILED (result)) return NS_ERROR_FAILURE;
+
+		nsCOMPtr<nsIDOMElement> linkElement;
+		linkElement = do_QueryInterface (aLink);
+		if (!linkElement) return NS_ERROR_FAILURE;
+
+		/* get rel=.. element */
+		nsAutoString attr;
+		attr.AssignWithConversion("rel");
+		nsAutoString value;
+		linkElement->GetAttribute (attr, value);
+
+		if (value.IsEmpty())
+		{
+			attr.AssignWithConversion("rev");
+			linkElement->GetAttribute (attr, value);
+			if (value.IsEmpty()) continue;
+		}
+
+		relstr = ToNewCString (value);
+		g_strup (relstr);
+
+		/* check for elements we want */
+		for (gint j = 0; (rel_types[j] != NULL); j++)
+		{
+			if (strcmp (relstr, rel_types[j]) == 0)
+			{
+				/* found one! */
+				LinkInterfaceItem *lti =
+					g_new0 (LinkInterfaceItem, 1);
+
+				/* fill in struct */
+				lti->type = (LinkInterfaceItemType) j;
+
+				/* get href=.. element */
+				nsAutoString attr;
+				attr.AssignWithConversion("href");
+				nsAutoString value;
+				linkElement->GetAttribute (attr, value);
+
+				if (value.IsEmpty())
+				{
+					g_free (lti);
+					continue;
+				}
+
+				nsCString link;
+				link.AssignWithConversion(value);
+
+				/* resolve uri */
+				nsCOMPtr<nsIDocument> doc = 
+					do_QueryInterface (aDOMDocument);
+				if(!doc) return NS_ERROR_FAILURE;
+			
+				nsCOMPtr<nsIURI> uri;
+				doc->GetDocumentURL(getter_AddRefs(uri));
+
+				nsCString href;
+				result = uri->Resolve (link, href);
+				if (NS_FAILED (result)) return NS_ERROR_FAILURE;
+				lti->href = g_strdup (href.get());
+		
+				/* append to list of items */
+				*list = g_list_append (*list, lti);
+		
+				/* get optional title=... element */
+				attr.AssignWithConversion("title");
+				linkElement->GetAttribute (attr, value);
+				if (value.IsEmpty()) continue;
+
+				char *title = ToNewCString (value);
+				lti->title = misc_string_strip_newline (title);
+				nsMemory::Free (title);
+			}
+		}
+
+		nsMemory::Free (relstr);
+	}
+
+	return NS_OK;
+}
+
+nsresult GaleonWrapper::GetRealURL (nsCString &ret)
+{
+	nsresult result;
+
+	nsCOMPtr<nsIDocShell> DocShell;
+	result = GetDocShell (getter_AddRefs(DocShell));
+	if (NS_FAILED(result) || !DocShell) return NS_ERROR_FAILURE;
+
+	nsCOMPtr<nsIWebNavigation> ContentNav = do_QueryInterface (DocShell,
+								   &result);
+	if (!ContentNav) return NS_ERROR_FAILURE;
+
+	nsCOMPtr<nsIURI> uri;
+	result = ContentNav->GetCurrentURI (getter_AddRefs(uri));
+	if (!NS_SUCCEEDED(result) || (!uri)) return NS_ERROR_FAILURE;
+
+	result = uri->GetSpec(ret);
+	if (!NS_SUCCEEDED(result) || ret.IsEmpty()) return NS_ERROR_FAILURE;
+
+	return NS_OK;
+}
+
+nsresult GaleonWrapper::SelectAll (void)
+{
+	nsCOMPtr<nsIClipboardCommands> clipboard (do_GetInterface(mWebBrowser));
+	return clipboard->SelectAll ();
+}
+
+nsresult GaleonWrapper::ScrollUp (void)
+{
+	nsresult result;
+
+	nsCOMPtr<nsIDOMWindow> DOMWindow;
+	result = GetFocusedDOMWindow (getter_AddRefs(DOMWindow));
+	if (NS_FAILED(result) || !DOMWindow) return NS_ERROR_FAILURE;
+
+	DOMWindow->ScrollByLines(-1);
+
+	return NS_OK;
+}
+
+nsresult GaleonWrapper::ScrollDown (void)
+{
+	nsresult result;
+
+	nsCOMPtr<nsIDOMWindow> DOMWindow;
+	result = GetFocusedDOMWindow (getter_AddRefs(DOMWindow));
+	if (NS_FAILED(result) || !DOMWindow) return NS_ERROR_FAILURE;
+
+	DOMWindow->ScrollByLines(1);
+	
+	return NS_OK;
+}
+
+nsresult GaleonWrapper::ScrollLeft (void)
+{
+	nsresult result;
+
+	nsCOMPtr<nsIDOMWindow> DOMWindow;
+	result = GetFocusedDOMWindow (getter_AddRefs(DOMWindow));
+	if (NS_FAILED(result) || !DOMWindow) return NS_ERROR_FAILURE;
+
+	DOMWindow->ScrollBy(-16, 0);
+	
+	return NS_OK;
+}
+
+nsresult GaleonWrapper::ScrollRight (void)
+{
+	nsresult result;
+
+	nsCOMPtr<nsIDOMWindow> DOMWindow;
+	result = GetFocusedDOMWindow (getter_AddRefs(DOMWindow));
+	if (NS_FAILED(result) || !DOMWindow) return NS_ERROR_FAILURE;
+
+	DOMWindow->ScrollBy(16, 0);
+	
+	return NS_OK;
+}
+
+nsresult GaleonWrapper::GetLastModified (gchar **ret)
+{
+	nsresult result;
+
+	nsCOMPtr<nsIDOMDocument> aDOMDocument;
+
+	result = GetDOMDocument (getter_AddRefs(aDOMDocument));
+	if (NS_FAILED(result) || !aDOMDocument) return NS_ERROR_FAILURE;
+
+	nsCOMPtr<nsIDOMNSHTMLDocument> doc = do_QueryInterface(aDOMDocument);
+	if(!doc) return NS_ERROR_FAILURE;
+
+	nsAutoString value;
+	doc->GetLastModified(value);
+	char *tmp = ToNewCString (value);
+	*ret = g_strdup (tmp);
+	nsMemory::Free (tmp);
+
+	return NS_OK;
+}
+
+nsresult GaleonWrapper::GetImages (GList **ret)
+{
+	nsresult result;
+	GHashTable *hash = g_hash_table_new (g_str_hash, g_str_equal);
+
+	nsCOMPtr<nsIDOMDocument> aDOMDocument;
+
+	result = GetDOMDocument (getter_AddRefs(aDOMDocument));
+	if (NS_FAILED(result) || !aDOMDocument) return NS_ERROR_FAILURE;
+
+	nsCOMPtr<nsIDOMHTMLDocument> doc = do_QueryInterface(aDOMDocument);
+	if(!doc) return NS_ERROR_FAILURE;
+
+	nsCOMPtr<nsIDOMHTMLCollection> col;
+	doc->GetImages(getter_AddRefs(col));
+
+	PRUint32 count, i;
+	col->GetLength(&count);
+	for (i = 0; i < count; i++)
+	{
+		nsCOMPtr<nsIDOMNode> node;
+		col->Item(i, getter_AddRefs(node));
+		if (!node) continue;
+
+		nsCOMPtr<nsIDOMHTMLElement> element;
+		element = do_QueryInterface(node);
+		if (!element) continue;
+		
+		nsCOMPtr<nsIDOMHTMLImageElement> img;
+		img = do_QueryInterface(element);
+		if (!img) continue;
+
+		ImageListItem *item = g_new0 (ImageListItem, 1);
+		
+		nsAutoString tmp;
+		result = img->GetSrc (tmp);
+		if (NS_SUCCEEDED(result))
+		{
+			char *c;
+			c = ToNewCString(tmp);
+			if (g_hash_table_lookup (hash, c))
+			{
+				nsMemory::Free (c);
+				g_free (item);
+				continue;
+			}
+			item->url = g_strdup (c);
+			nsMemory::Free (c);
+			g_hash_table_insert (hash, item->url,
+					     GINT_TO_POINTER (TRUE));
+		}
+		result = img->GetAlt (tmp);
+		if (NS_SUCCEEDED(result))
+		{
+			char *c;
+			c = ToNewCString(tmp);
+			item->alt = misc_string_strip_newline (c);
+			nsMemory::Free (c);
+		}
+		result = element->GetTitle (tmp);
+		if (NS_SUCCEEDED(result))
+		{
+			char *c;
+			c = ToNewCString(tmp);
+			item->title = misc_string_strip_newline (c);
+			nsMemory::Free (c);
+		}
+		result = img->GetWidth (&(item->width));
+		result = img->GetHeight (&(item->height));
+
+		*ret = g_list_append (*ret, item);
+	}
+
+	g_hash_table_destroy (hash);
+
+	return NS_OK;
+}
+
+nsresult GaleonWrapper::GetForms (GList **ret)
+{
+	nsresult result;
+
+	nsCOMPtr<nsIDOMDocument> aDOMDocument;
+
+	result = GetDOMDocument (getter_AddRefs(aDOMDocument));
+	if (NS_FAILED(result) || !aDOMDocument) return NS_ERROR_FAILURE;
+
+	nsCOMPtr<nsIDOMHTMLDocument> doc = do_QueryInterface(aDOMDocument);
+	if(!doc) return NS_ERROR_FAILURE;
+
+	nsCOMPtr<nsIDOMHTMLCollection> col;
+	doc->GetForms(getter_AddRefs(col));
+
+	PRUint32 count, i;
+	col->GetLength(&count);
+	for (i = 0; i < count; i++)
+	{
+		nsCOMPtr<nsIDOMNode> node;
+		col->Item(i, getter_AddRefs(node));
+		if (!node) continue;
+
+		nsCOMPtr<nsIDOMHTMLElement> element;
+		element = do_QueryInterface(node);
+		if (!element) continue;
+		
+		nsCOMPtr<nsIDOMHTMLFormElement> form;
+		form = do_QueryInterface(element);
+		if (!form) continue;
+
+		FormListItem *item = g_new0 (FormListItem, 1);
+		
+		nsAutoString tmp;
+		result = form->GetAction (tmp);
+		if (NS_SUCCEEDED(result))
+		{
+			nsCString c, s;
+			c.AssignWithConversion(tmp);
+
+			nsCOMPtr<nsIDocument> doc = 
+				do_QueryInterface (aDOMDocument);
+			if(!doc) return NS_ERROR_FAILURE;
+			
+			nsCOMPtr<nsIURI> uri;
+			doc->GetDocumentURL(getter_AddRefs(uri));
+
+			result = uri->Resolve (c, s);
+
+			item->action = s.Length() ? g_strdup (s.get()) : g_strdup (c.get());
+		}
+		result = form->GetMethod (tmp);
+		if (NS_SUCCEEDED(result))
+		{
+			char *c;
+			c = ToNewCString(tmp);
+			item->method = g_strdup (c);
+			nsMemory::Free (c);
+		}
+		result = form->GetName (tmp);
+		if (NS_SUCCEEDED(result))
+		{
+			char *c;
+			c = ToNewCString(tmp);
+			item->name = g_strdup (c);
+			nsMemory::Free (c);
+		}
+
+		*ret = g_list_append (*ret, item);
+	}
+
+	return NS_OK;
+}
+
+nsresult GaleonWrapper::GetLinks (GList **ret)
+{
+	nsresult result;
+
+	nsCOMPtr<nsIDOMDocument> aDOMDocument;
+	result = GetMainDOMDocument (getter_AddRefs(aDOMDocument));
+	if (NS_FAILED(result) || !aDOMDocument) return NS_ERROR_FAILURE;
+
+	/* first, get a list of <link> elements */
+	PRUint32 links_count;
+
+	nsString strname;
+	strname.AssignWithConversion("LINK");
+
+	nsCOMPtr<nsIDOMNodeList> aLinks;
+	result = aDOMDocument->GetElementsByTagName (strname, 
+						     getter_AddRefs (aLinks));
+	if (NS_FAILED (result)) return NS_ERROR_FAILURE;
+
+	result = aLinks->GetLength (&links_count);
+	if (NS_FAILED (result)) return NS_ERROR_FAILURE;
+
+	for (PRUint32 i = 0; i < links_count; i++)
+	{
+		nsCOMPtr<nsIDOMNode> aLink;
+		result = aLinks->Item (i, getter_AddRefs (aLink));
+		if (NS_FAILED (result)) continue;
+
+		nsCOMPtr<nsIDOMElement> linkElement;
+		linkElement = do_QueryInterface (aLink);
+		if (!linkElement) continue;
+
+		nsAutoString attr;
+		attr.AssignWithConversion("href");
+		nsAutoString value;
+		linkElement->GetAttribute (attr, value);
+		if (value.IsEmpty()) continue;
+
+		nsCString link;
+		link.AssignWithConversion(value);
+
+		if (link.IsEmpty()) continue;
+			
+		nsCOMPtr<nsIDocument> doc = 
+			do_QueryInterface (aDOMDocument);
+		if(!doc) continue;
+		
+		nsCOMPtr<nsIURI> uri;
+		doc->GetDocumentURL(getter_AddRefs(uri));
+
+		nsCString tmp;
+		result = uri->Resolve (link, tmp);
+		
+		LinkListItem *i = g_new0 (LinkListItem, 1);
+
+		if (!tmp.IsEmpty())
+		{
+			i->url = g_strdup (tmp.get());
+		}
+		else
+		{
+			i->url = g_strdup (link.get());
+		}
+
+		attr.AssignWithConversion("title");
+		linkElement->GetAttribute (attr, value);
+		if (!value.IsEmpty())
+		{
+			char *s = ToNewCString (value);
+			i->title = misc_string_strip_newline (s);
+			nsMemory::Free (s);
+		}
+
+		attr.AssignWithConversion("rel");
+		linkElement->GetAttribute (attr, value);
+		if (!value.IsEmpty())
+		{
+			char *s = ToNewCString (value);
+			i->rel = g_strdup (s);
+			g_strdown (i->rel);
+			nsMemory::Free (s);
+		}
+		if (!i->rel || strlen (i->rel) == 0)
+		{
+			attr.AssignWithConversion("rev");
+			linkElement->GetAttribute (attr, value);
+			if (!value.IsEmpty())
+			{
+				char *s = ToNewCString (value);
+				i->rel = g_strdup (s);
+				g_strdown (i->rel);
+				nsMemory::Free (s);
+			}
+		}
+		
+		*ret = g_list_append (*ret, i);
+	}
+
+	/* next, get a list of anchors */
+	nsCOMPtr<nsIDOMHTMLDocument> doc = do_QueryInterface(aDOMDocument);
+	if(!doc) return NS_ERROR_FAILURE;
+
+	nsCOMPtr<nsIDOMHTMLCollection> col;
+	doc->GetLinks(getter_AddRefs(col));
+
+	PRUint32 count, i;
+	col->GetLength(&count);
+	for (i = 0; i < count; i++)
+	{
+		nsCOMPtr<nsIDOMNode> node;
+		col->Item(i, getter_AddRefs(node));
+		if (!node) continue;
+
+		nsCOMPtr<nsIDOMHTMLElement> element;
+		element = do_QueryInterface(node);
+		if (!element) continue;
+		
+		nsCOMPtr<nsIDOMHTMLAnchorElement> lnk;
+		lnk = do_QueryInterface(element);
+		if (!lnk) continue;
+
+		LinkListItem *i = g_new0 (LinkListItem, 1);
+
+		nsAutoString tmp;
+		char *c;
+		result = lnk->GetHref (tmp);
+		if (NS_SUCCEEDED(result))
+		{
+			c = ToNewCString(tmp);
+			i->url = g_strdup (c);
+			nsMemory::Free (c);
+		}
+
+		result = lnk->GetRel (tmp);
+		if (NS_SUCCEEDED(result))
+		{
+			c = ToNewCString(tmp);
+			i->rel = g_strdup (c);
+			g_strdown (i->rel);
+			nsMemory::Free (c);
+		}
+
+		if (!i->rel || strlen (i->rel) == 0)
+		{
+			result = lnk->GetRev (tmp);
+			if (NS_SUCCEEDED(result))
+			{
+				c = ToNewCString(tmp);
+				i->rel = g_strdup (c);
+				g_strdown (i->rel);
+				nsMemory::Free (c);
+			}
+		}
+
+		i->title = mozilla_get_link_text (node);
+		if (i->title == NULL)
+		{
+			result = element->GetTitle (tmp);
+			if (NS_SUCCEEDED(result))
+			{
+				c = ToNewCString(tmp);
+				i->title = misc_string_strip_newline (c);
+				nsMemory::Free (c);
+			}
+		}
+
+
+		*ret = g_list_append (*ret, i);
+	}
+
+	return NS_OK;
+}
+
+nsresult GaleonWrapper::EvaluateJS (char *script)
+{
+	nsresult rv;
+
+	nsCOMPtr<nsIDOMWindow> DOMWindow;
+	rv = mWebBrowser->GetContentDOMWindow(getter_AddRefs(DOMWindow));
+
+	nsCOMPtr<nsIScriptGlobalObject> globalObject;
+	globalObject = do_QueryInterface (DOMWindow);
+	if (!globalObject) return NS_ERROR_FAILURE;
+
+	nsCOMPtr<nsIScriptContext> context;
+	rv = globalObject->GetContext(getter_AddRefs(context));
+	if (NS_FAILED(rv) || !context) {
+		return NS_ERROR_FAILURE;
+	}
+
+	context->SetProcessingScriptTag(PR_TRUE);
+
+	PRBool isUndefined;
+	nsString ret;
+	nsAutoString aScript;
+	aScript.AssignWithConversion (script);
+	context->EvaluateString(aScript, nsnull, nsnull, nsnull,
+				0, nsnull, 
+				ret, &isUndefined);  
+
+	context->SetProcessingScriptTag(PR_FALSE);
+
+	return NS_OK;
+}
+
+nsresult GaleonWrapper::PushTargetDocument (nsIDOMDocument *domDoc)
+{
+	mTargetDocument = domDoc;
+
+	return NS_OK;
+}
+
+nsresult GaleonWrapper::PopTargetDocument ()
+{
+	mTargetDocument = nsnull;
+
+	return NS_OK;
+}
