@@ -16,36 +16,10 @@
  *  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
  */
 
-#include "nsIFactory.h"
-#include "nsXPComFactory.h"
-
 #include "ManProtocolHandler.h"
 
 /* Implementation file */
 NS_IMPL_ISUPPORTS1 (GManProtocolHandler, nsIProtocolHandler)
 
-#ifndef XPCOM_SHARED_LIB
-NS_DEF_FACTORY (GManProtocolHandler, GManProtocolHandler);
-
-/**
- * NS_NewManProtocolHandlerFactory:
- */ 
-nsresult NS_NewManHandlerFactory(nsIFactory** aFactory)
-{
-	NS_ENSURE_ARG_POINTER(aFactory);
-	*aFactory = nsnull;
-
-	nsGManProtocolHandlerFactory *result = new nsGManProtocolHandlerFactory;
-	if (result == NULL)
-	{
-		return NS_ERROR_OUT_OF_MEMORY;
-	}
-    
-	NS_ADDREF(result);
-	*aFactory = result;
-
-	return NS_OK;
-}
-#endif
 ////////////////////////////////////////////////////////////////////////////////
 
