@@ -17,11 +17,11 @@
  */
 
 #include "galeon.h"
-#include "misc.h"
 #include "mozilla.h"
 #include "dialog.h"
 #include "glade.h"
 #include "PromptService.h"
+#include "themes.h"
 
 #include "nsCOMPtr.h"
 #include "nsIFactory.h"
@@ -36,6 +36,7 @@
 #include <gtk/gtktogglebutton.h>
 #include <gtk/gtklist.h>
 #include <libgnomeui/gnome-stock.h>
+#include <libgnomeui/gnome-dialog.h>
 #include <libgnome/gnome-i18n.h>
 #include <libgnome/gnome-triggers.h>
 
@@ -92,17 +93,22 @@ NS_IMETHODIMP CPromptService::Alert (nsIDOMWindow *parent,
 {    
 	GtkWidget *label;
 	GtkWidget *dialog;
+	GtkWidget *pixmap;
 	GladeXML *gxml;
+	PixmapData *icon =
+		themes_get_pixmap ("gnome-warning.png", FALSE);
 
 	/* build the dialog */
 	gxml = glade_widget_new ("prompts.glade", "alert_dialog", 
 				 &dialog, NULL);
 	label = glade_xml_get_widget (gxml, "label");
+	pixmap = glade_xml_get_widget (gxml, "pixmap");
 	gtk_object_unref (GTK_OBJECT (gxml));
 
 	/* set dynamic attributes */
 	set_title (dialog, dialogTitle);
 	set_label_text (label, text);
+	gtk_pixmap_set (GTK_PIXMAP (pixmap), icon->pixmap, icon->mask);
 
 	GtkWidget *parentWidget = mozilla_find_gtk_parent (parent);
 	dialog_set_parent (dialog, parentWidget);
@@ -129,20 +135,25 @@ NS_IMETHODIMP CPromptService::AlertCheck (nsIDOMWindow *parent,
 					  PRBool *checkValue)
 {
 	GtkWidget *label;
+	GtkWidget *pixmap;
 	GtkWidget *check_button;
 	GtkWidget *dialog;
 	GladeXML *gxml;
+	PixmapData *icon =
+		themes_get_pixmap ("gnome-warning.png", FALSE);
 
 	/* build the dialog */
 	gxml = glade_widget_new ("prompts.glade", "alert_check_dialog", 
 				 &dialog, NULL);
 	label = glade_xml_get_widget (gxml, "label");
+	pixmap = glade_xml_get_widget (gxml, "pixmap");
 	check_button = glade_xml_get_widget (gxml, "check_button");
 	gtk_object_unref (GTK_OBJECT (gxml));
 
 	/* set dynamic attributes */
 	set_title (dialog, dialogTitle);
 	set_label_text (label, text);
+	gtk_pixmap_set (GTK_PIXMAP (pixmap), icon->pixmap, icon->mask);
 	set_check_button_text (check_button, checkMsg);
 	set_check_button (check_button, checkValue);
 	set_check_button_size_to_label (check_button, label);
@@ -171,19 +182,24 @@ NS_IMETHODIMP CPromptService::Confirm (nsIDOMWindow *parent,
 				       PRBool *_retval)
 {
 	GtkWidget *label;
+	GtkWidget *pixmap;
 	GtkWidget *dialog;
 	GladeXML *gxml;
+	PixmapData *icon =
+		themes_get_pixmap ("gnome-question.png", FALSE);
 	gint ret;
 
 	/* build the dialog */
 	gxml = glade_widget_new ("prompts.glade", "confirm_dialog", 
 				 &dialog, NULL);
 	label = glade_xml_get_widget (gxml, "label");
+	pixmap = glade_xml_get_widget (gxml, "pixmap");
 	gtk_object_unref (GTK_OBJECT (gxml));
 
 	/* set dynamic attributes */
 	set_title (dialog, dialogTitle);
 	set_label_text (label, text);
+	gtk_pixmap_set (GTK_PIXMAP (pixmap), icon->pixmap, icon->mask);
 
 	GtkWidget *parentWidget = mozilla_find_gtk_parent (parent);
 	dialog_set_parent (dialog, parentWidget);
@@ -246,8 +262,11 @@ NS_IMETHODIMP CPromptService::ConfirmEx (nsIDOMWindow *parent,
 {
 	GtkWidget *dialog;
 	GtkWidget *label;
+	GtkWidget *pixmap;
 	GtkWidget *check_button;
 	GladeXML *gxml;
+	PixmapData *icon =
+		themes_get_pixmap ("gnome-question.png", FALSE);
 	gint ret;
 	gint i;
 
@@ -255,15 +274,25 @@ NS_IMETHODIMP CPromptService::ConfirmEx (nsIDOMWindow *parent,
 	gxml = glade_widget_new ("prompts.glade", "confirmex_dialog", 
 				 &dialog, NULL);
 	label = glade_xml_get_widget (gxml, "label");
+	pixmap = glade_xml_get_widget (gxml, "pixmap");
 	check_button = glade_xml_get_widget (gxml, "check_button");
 	gtk_object_unref (GTK_OBJECT (gxml));
 	
 	/* set dynamic attributes */
 	set_title (dialog, dialogTitle);
 	set_label_text (label, text);
-	set_check_button_text (check_button, checkMsg);
-	set_check_button (check_button, checkValue);
-	set_check_button_size_to_label (check_button, label);
+	gtk_pixmap_set (GTK_PIXMAP (pixmap), icon->pixmap, icon->mask);
+
+	if (checkMsg)
+	{
+		set_check_button_text (check_button, checkMsg);
+		set_check_button (check_button, checkValue);
+		set_check_button_size_to_label (check_button, label);
+	}
+	else
+	{
+		gtk_widget_hide (GTK_WIDGET (check_button));
+	}
 
 	/* create buttons */
 	for (i = 0; i < 4; i++)
@@ -324,6 +353,8 @@ NS_IMETHODIMP CPromptService::ConfirmEx (nsIDOMWindow *parent,
 		buttonFlags >>= 8;
 	}
 
+	gnome_dialog_set_default (GNOME_DIALOG(dialog), 0);
+
 	GtkWidget *parentWidget = mozilla_find_gtk_parent (parent);
 	dialog_set_parent (dialog, parentWidget);
 
@@ -356,8 +387,11 @@ NS_IMETHODIMP CPromptService::Prompt (nsIDOMWindow *parent,
 	GtkWidget *dialog;
 	GtkWidget *entry;
 	GtkWidget *label;
+	GtkWidget *pixmap;
 	GtkWidget *check_button;
 	GladeXML *gxml;
+	PixmapData *icon =
+		themes_get_pixmap ("gnome-question.png", FALSE);
 	gint ret;
 
 	/* build and show the dialog */
@@ -365,12 +399,14 @@ NS_IMETHODIMP CPromptService::Prompt (nsIDOMWindow *parent,
 				 &dialog, NULL);
 	entry = glade_xml_get_widget (gxml, "entry");
 	label = glade_xml_get_widget (gxml, "label");
+	pixmap = glade_xml_get_widget (gxml, "pixmap");
 	check_button = glade_xml_get_widget (gxml, "check_button");
 	gtk_object_unref (GTK_OBJECT (gxml));
 
 	/* set dynamic attributes */
 	set_title (dialog, dialogTitle);
 	set_label_text (label, text);
+	gtk_pixmap_set (GTK_PIXMAP (pixmap), icon->pixmap, icon->mask);
 	set_check_button_text (check_button, checkMsg);
 	set_editable (entry, value);
 	set_check_button (check_button, checkValue);
@@ -418,9 +454,12 @@ NS_IMETHODIMP CPromptService::PromptUsernameAndPassword
 	GtkWidget *dialog;
 	GtkWidget *check_button;
 	GtkWidget *label;
+	GtkWidget *pixmap;
 	GtkWidget *username_entry;
 	GtkWidget *password_entry;
 	GladeXML *gxml;
+	PixmapData *icon =
+		themes_get_pixmap ("gnome-question.png", FALSE);
 	gint ret;
 
 	/* build and show the dialog */
@@ -428,6 +467,7 @@ NS_IMETHODIMP CPromptService::PromptUsernameAndPassword
 				 &dialog, NULL);
 	check_button = glade_xml_get_widget (gxml, "check_button");
 	label = glade_xml_get_widget (gxml, "label");
+	pixmap = glade_xml_get_widget (gxml, "pixmap");
 	username_entry = glade_xml_get_widget (gxml, "username_entry");
 	password_entry = glade_xml_get_widget (gxml, "password_entry");
 	gtk_object_unref (GTK_OBJECT (gxml));
@@ -440,6 +480,7 @@ NS_IMETHODIMP CPromptService::PromptUsernameAndPassword
 	set_check_button (check_button, checkValue);
 	set_label_text (label, text);
 	set_check_button_size_to_label (check_button, label);
+	gtk_pixmap_set (GTK_PIXMAP (pixmap), icon->pixmap, icon->mask);
 
 	/* make an enter keypress activate the "ok" button */
 	gnome_dialog_editable_enters (GNOME_DIALOG (dialog),
@@ -481,9 +522,12 @@ NS_IMETHODIMP CPromptService::PromptPassword (nsIDOMWindow *parent,
 {
 	GtkWidget *dialog;
 	GtkWidget *label;
+	GtkWidget *pixmap;
 	GtkWidget *check_button;
 	GtkWidget *password_entry;
 	GladeXML *gxml;
+	PixmapData *icon =
+		themes_get_pixmap ("gnome-question.png", FALSE);
 	gint ret;
 
 	/* build and show the dialog */
@@ -491,6 +535,7 @@ NS_IMETHODIMP CPromptService::PromptPassword (nsIDOMWindow *parent,
 				 &dialog, NULL);
 	check_button = glade_xml_get_widget (gxml, "check_button");
 	label = glade_xml_get_widget (gxml, "label");
+	pixmap = glade_xml_get_widget (gxml, "pixmap");
 	password_entry = glade_xml_get_widget (gxml, "password_entry");
 	gtk_object_unref (GTK_OBJECT (gxml));
 
@@ -501,6 +546,7 @@ NS_IMETHODIMP CPromptService::PromptPassword (nsIDOMWindow *parent,
 	set_check_button (check_button, checkValue);
 	set_label_text (label, text);
 	set_check_button_size_to_label (check_button, label);
+	gtk_pixmap_set (GTK_PIXMAP (pixmap), icon->pixmap, icon->mask);
 
 	/* make an enter keypress activate the "ok" button */
 	gnome_dialog_editable_enters (GNOME_DIALOG (dialog),
@@ -539,19 +585,24 @@ NS_IMETHODIMP CPromptService::Select (nsIDOMWindow *parent,
 	GtkWidget *dialog;
 	GtkWidget *list;
 	GtkWidget *label;
+	GtkWidget *pixmap;
 	GladeXML *gxml;
+	PixmapData *icon =
+		themes_get_pixmap ("gnome-question.png", FALSE);
 	gint ret;
 
 	/* build and show the dialog */
 	gxml = glade_widget_new ("prompts.glade", "select_dialog", 
 				 &dialog, NULL);
 	list   = glade_xml_get_widget (gxml, "list");
+	pixmap = glade_xml_get_widget (gxml, "pixmap");
 	label   = glade_xml_get_widget (gxml, "label");
 	gtk_object_unref (GTK_OBJECT (gxml));
 
 	/* set dynamic attributes */
 	set_title (dialog, dialogTitle);
 	set_label_text (label, text);
+	gtk_pixmap_set (GTK_PIXMAP (pixmap), icon->pixmap, icon->mask);
 
 	GList *gList = NULL;
 	for (PRUint32 i = 0 ; i < count ; i++)
