@@ -109,6 +109,16 @@ mozilla_user_agent_notifier(GConfClient *client,
 			    GConfEntry *entry,
 			    gpointer user_data);
 
+void mozilla_default_charset_notifier(GConfClient *client,
+				      guint cnxn_id,
+				      GConfEntry *entry,
+				      gpointer user_data);
+void
+mozilla_socks_version_notifier (GConfClient *client,
+				guint cnxn_id,
+				GConfEntry *entry,
+				gpointer user_data);
+
 static char * 
 get_bracketed_string (char *str);
 
@@ -126,6 +136,12 @@ notifiers [] =
 {
 	{ CONF_FILTERING_JAVA_ENABLED, generic_mozilla_bool_notifier,
 	  "security.enable_java"},
+	{ CONF_SECURITY_SSL2, generic_mozilla_bool_notifier,
+	  "security.enable_ssl2"},
+	{ CONF_SECURITY_SSL3, generic_mozilla_bool_notifier,
+	  "security.enable_ssl3"},
+	{ CONF_SECURITY_TLS, generic_mozilla_bool_notifier,
+	  "security.enable_tls"},
 	{ CONF_FILTERING_JAVASCRIPT_ENABLED, generic_mozilla_bool_notifier,
 	  "javascript.enabled"},
 	{ CONF_RENDERING_USE_OWN_COLORS, mozilla_own_colors_notifier,
@@ -156,12 +172,16 @@ notifiers [] =
 	  "network.proxy.ftp"},
 	{ CONF_NETWORK_SSL_PROXY, generic_mozilla_string_notifier,
 	  "network.proxy.ssl"},
+	{ CONF_NETWORK_SOCKS_PROXY, generic_mozilla_string_notifier,
+	  "network.proxy.socks"},
 	{ CONF_NETWORK_HTTP_PROXY_PORT, generic_mozilla_int_notifier,
 	  "network.proxy.http_port"},
 	{ CONF_NETWORK_FTP_PROXY_PORT, generic_mozilla_int_notifier,
 	  "network.proxy.ftp_port"},
 	{ CONF_NETWORK_SSL_PROXY_PORT, generic_mozilla_int_notifier,
 	  "network.proxy.ssl_port"},
+	{ CONF_NETWORK_SOCKS_PROXY_PORT, generic_mozilla_int_notifier,
+	  "network.proxy.socks_port"},
 	{ CONF_NETWORK_NO_PROXIES_FOR, generic_mozilla_string_notifier,
 	  "network.proxy.no_proxies_on"},
 	{ CONF_NETWORK_PROXY_KA, generic_mozilla_bool_notifier,
@@ -198,6 +218,14 @@ notifiers [] =
 	"mousewheel.withaltkey.numlines"},
 	{ CONF_MOUSE_WHEEL_SHIFT_STEP, generic_mozilla_int_notifier,
 	"mousewheel.withshiftkey.numlines"},
+	{ CONF_MOUSE_WHEEL_STEP_SIZE, generic_mozilla_int_notifier,
+	"mousewheel.withnokey.numlines"},
+	{ CONF_MOUSE_WHEEL_CTRL_STEP, generic_mozilla_int_notifier,
+	"mousewheel.withcontrolkey.numlines"},
+	{ CONF_MOUSE_WHEEL_ALT_STEP, generic_mozilla_int_notifier,
+	"mousewheel.withaltkey.numlines"},
+	{ CONF_MOUSE_WHEEL_SHIFT_STEP, generic_mozilla_int_notifier,
+	"mousewheel.withshiftkey.numlines"},
 	{ CONF_PERSISTENT_COOKIE_WARN, generic_mozilla_bool_notifier,
 	"network.cookie.warnAboutCookies"},
 	{ CONF_PERSISTENT_COOKIES_BEHAVIOR, generic_mozilla_int_notifier,
@@ -210,8 +238,8 @@ notifiers [] =
 	NULL},
 	{ CONF_DOWNLOADING_DIR, generic_mozilla_string_notifier,
 	"browser.download.dir"},
-	{ CONF_LANGUAGE_DEFAULT_CHARSET, generic_mozilla_string_notifier,
-	"intl.charset.default"},
+	{ CONF_LANGUAGE_DEFAULT_CHARSET, mozilla_default_charset_notifier,
+	NULL },
 	{ CONF_RENDERING_LANGUAGE, mozilla_language_notifier,
 	NULL},
 	{ CONF_LANGUAGE_AUTODETECT_CHARSET, mozilla_autodetect_charset_notifier,
@@ -222,6 +250,8 @@ notifiers [] =
 	NULL},
 	{ CONF_GHELP_USE_GALEON, mozilla_ghelp_notifier,
 	NULL},
+	{ CONF_NETWORK_SOCKS_PROXY_VERSION, mozilla_socks_version_notifier,
+	  NULL},
 	{NULL, NULL, NULL}
 };
 
@@ -342,6 +372,29 @@ generic_mozilla_bool_notifier(GConfClient *client,
 	}
 }
 
+void mozilla_default_charset_notifier(GConfClient *client,
+				      guint cnxn_id,
+				      GConfEntry *entry,
+				      gpointer user_data)
+{
+	const gchar *charset_title;
+	gchar *cur_charset;
+
+	if (sorted_charset_titles == NULL)
+	{
+		mozilla_get_charsets (&charsets, &sorted_charset_titles);
+	}
+
+        charset_title = gconf_value_get_string (entry->value);
+	if (charset_title)
+	{
+		cur_charset = g_hash_table_lookup (charsets, 
+						   charset_title);
+
+		mozilla_preference_set ("intl.charset.default", cur_charset);
+	}
+}
+
 
 void
 mozilla_own_colors_notifier(GConfClient *client,
@@ -407,28 +460,9 @@ mozilla_allow_popups_notifier(GConfClient *client,
 			      GConfEntry *entry,
 			      gpointer user_data)
 {
-#ifndef ENABLE_EXPERIMENTAL_FILTERS
-#if MOZILLA_VERSION >= VERSION3(0,9,4)
 	gboolean new_val = eel_gconf_get_boolean(CONF_FILTERING_ALLOW_POPUPS);
 	mozilla_preference_set_boolean ("dom.disable_open_during_load", 
 					!new_val);
-
-#else
-	switch (eel_gconf_get_boolean(CONF_FILTERING_ALLOW_POPUPS))
-	{
-	case TRUE:
-		mozilla_preference_set ("capability.policy.default.Window.open",
-					"allAccess");
-		mozilla_preference_set ("browser.target_new_blocked", "false");
-		break;
-	case FALSE: 
-		mozilla_preference_set ("capability.policy.default.Window.open",
-					"noAccess");
-		mozilla_preference_set ("browser.target_new_blocked", "true");
-		break;
-	}
-#endif //MOZILLA_VERSION
-#endif //ENABLE_EXPERIMENTAL_FILTERS
 }
 
 void
@@ -437,12 +471,12 @@ mozilla_language_notifier(GConfClient *client,
 			  GConfEntry *entry,
 			  gpointer user_data)
 {
-	gchar *languages = NULL;
-	GSList *language_list = NULL;
-	GSList *cur_lang_list = NULL;
+	gchar *languages;
+	GSList *language_list ,*cur_lang_list;
 
-	language_list = 
-		eel_gconf_get_string_list(CONF_RENDERING_LANGUAGE);
+	language_list = eel_gconf_get_string_list (CONF_RENDERING_LANGUAGE);
+
+	languages = NULL;
 	cur_lang_list = language_list;
 	while (cur_lang_list != NULL) {
 		char *lang, *tmp;
@@ -462,13 +496,14 @@ mozilla_language_notifier(GConfClient *client,
 		cur_lang_list = cur_lang_list->next;
 	}
 
-	if (languages)
+	if (languages == NULL)
 	{
-		mozilla_preference_set ("intl.accept_languages", languages);
-		g_free (languages);
+		languages = g_strdup ("");
 	}
-	g_slist_free(language_list);
+	mozilla_preference_set ("intl.accept_languages", languages);
+	g_free (languages);
 
+	g_slist_free(language_list);
 }
 
 /**
@@ -539,7 +574,7 @@ mozilla_ftp_notifier(GConfClient *client,
 {
 	/* Register/Unregister the FTP Handler as appropriate */
 	if ((eel_gconf_get_integer(CONF_DOWNLOADING_EXTERNAL_FTP) == 1) &&
-	    !preferences_galeon_used_by_gnome_for_protocol ("ftp"))
+	    !prefs_galeon_used_by_gnome_for_protocol ("ftp"))
 	{
 		mozilla_register_FtpProtocolHandler ();
 	}
@@ -557,10 +592,10 @@ mozilla_ghelp_notifier(GConfClient *client,
 {
 	/* Register/Unregister the gHelp Handlers as appropriate */
 	if (eel_gconf_get_boolean(CONF_GHELP_USE_GALEON) == TRUE ||
-	    preferences_galeon_used_by_gnome_for_protocol ("man") ||
-	    preferences_galeon_used_by_gnome_for_protocol ("info") ||
-	    preferences_galeon_used_by_gnome_for_protocol ("ghelp") ||
-	    preferences_galeon_used_by_gnome_for_protocol ("gnome-help"))
+	    prefs_galeon_used_by_gnome_for_protocol ("man") ||
+	    prefs_galeon_used_by_gnome_for_protocol ("info") ||
+	    prefs_galeon_used_by_gnome_for_protocol ("ghelp") ||
+	    prefs_galeon_used_by_gnome_for_protocol ("gnome-help"))
 	{
 		mozilla_register_gHelpProtocolHandlers ();
 	}
@@ -587,3 +622,15 @@ mozilla_user_agent_notifier (GConfClient *client,
 			break;
 	}
 }
+
+void
+mozilla_socks_version_notifier (GConfClient *client,
+				guint cnxn_id,
+				GConfEntry *entry,
+				gpointer user_data)
+{
+	int version;
+	version = gconf_value_get_int(entry->value) + 4;
+	mozilla_preference_set_int ("network.proxy.socks_version", 
+				    version);
+}
