@@ -33,10 +33,6 @@
 
 void mozilla_prefs_set_default_font (void);
 static int
-mozilla_prefs_set_font (gchar *name, gchar *key, int lang_enc);
-static void
-mozilla_prefs_set_font_size (gchar *key, int size, int lang_enc);
-static int
 mozilla_prefs_get_font_size (gchar *key, int lang_enc);
 static gchar*
 mozilla_prefs_get_font (gchar *name_key, gchar *size_key, int lang_enc);
@@ -59,60 +55,31 @@ mozilla_prefs_set (void)
 
 	/* set default search engine */
 	mozilla_preference_set ("keyword.URL",_("http://www.google.com/search?q="));
+	mozilla_preference_set_boolean ("keyword.enabled", TRUE);
 	mozilla_preference_set_boolean ("security.checkloaduri", FALSE); 
 
 	/* while we have no UI */
- 	mozilla_preference_set_boolean ("wallet.captureForms", FALSE);
+	mozilla_preference_set_boolean ("wallet.captureForms", FALSE);
+
+	/* deactivate mailcap and mime.types support */
+	mozilla_preference_set ("helpers.global_mime_types_file", "");
+	mozilla_preference_set ("helpers.global_mailcap_file", "");
+	mozilla_preference_set ("helpers.private_mime_types_file", "");
+	mozilla_preference_set ("helpers.private_mailcap_file", "");
+	
+	/* dont allow xpi installs from galeon, there are crashes */
+	mozilla_preference_set_boolean ("xpinstall.enabled", FALSE);
+
+	/* disable sucky XUL ftp view, have nice ns4-like html page instead */
+	mozilla_preference_set_boolean ("network.dir.generate_html", TRUE);
+	
+	/* set the right accept encoding flags */
+	mozilla_preference_set ("network.http.accept-encoding" ,
+				"gzip, deflate, compress;q=0.9");
 
 	mozilla_save_prefs ();
 }
 
-/**
- * mozilla_prefs_set_fonts: set fonts preferences
- */
-void 
-mozilla_prefs_set_fonts (LanguageFontInfo *info, int lang_enc)
-{
-	int variable_size, fixed_size;
-
-	g_return_if_fail (info != NULL);
-
-	/* set serif font */
-	variable_size = mozilla_prefs_set_font 
-		(info->serif, "font.name.serif", 
-		lang_enc);
-
-	/* set sans serif font */
-	mozilla_prefs_set_font 
-		(info->sansserif, "font.name.sans-serif", 
-		 lang_enc);
-
-	/* set cursive font */
-	mozilla_prefs_set_font 
-		(info->cursive, "font.name.cursive", 
-		 lang_enc);
-
-	/* set fantasy font */
-	mozilla_prefs_set_font 
-		(info->fantasy, "font.name.fantasy", 
-		 lang_enc);
-
-	/* set monospace font */
-	fixed_size = mozilla_prefs_set_font 
-		(info->monospace, "font.name.monospace", 
-		 lang_enc);
-
-	/* set font size */
-	mozilla_prefs_set_font_size ("font.size.variable", 
-				     variable_size, lang_enc);
-	mozilla_prefs_set_font_size ("font.size.fixed",
-				    fixed_size, lang_enc);
-	mozilla_prefs_set_font_size ("font.min-size.variable", 
-				    info->min_size, lang_enc);
-	mozilla_prefs_set_font_size ("font.min-size.fixed", 
-				    info->min_size, lang_enc);
-}
-
 /**
  * mozilla_prefs_get_fonts: get fonts preferences
  */
@@ -147,8 +114,8 @@ mozilla_prefs_get_fonts (LanguageFontInfo *info, int lang_enc)
 		 lang_enc);
 
 	/* get font size */
-	info->min_size = mozilla_prefs_get_font_size 
-		("font.min-size.variable", lang_enc);
+        info->min_size = mozilla_prefs_get_font_size 
+                ("font.min-size.variable", lang_enc);
 }
 
 static int
@@ -193,7 +160,7 @@ mozilla_prefs_get_font (gchar *name_key, gchar *size_key, int lang_enc)
 	return result;
 }
 
-static int
+int
 mozilla_prefs_set_font (gchar *name, gchar *key, int lang_enc)
 {
 	gchar *font_name;
@@ -222,7 +189,7 @@ mozilla_prefs_set_font (gchar *name, gchar *key, int lang_enc)
 	return size;
 }
 
-static void
+void
 mozilla_prefs_set_font_size (gchar *key, int size, int lang_enc)
 {
 	gchar key_buf[128];
@@ -254,7 +221,7 @@ mozilla_prefs_set_user_agent ()
 		}
 		
 		default_user_agent = g_strdup_printf
-			("Mozilla/5.0 Galeon/" VERSION " (%s) Gecko/%d", 
+			("Mozilla/5.0 Galeon/" VERSION " (X11; %s; U;) Gecko/%d", 
 			 system,
 			 NS_BUILD_ID/100);
 
