@@ -17,10 +17,10 @@
  */
 
 #include "galeon.h"
-#include "misc.h"
 #include "embed.h"
 #include "glade.h"
 #include "bookmarks.h"
+#include "bookmarks_io.h"
 #include "prefs.h"
 #include "mozilla.h"
 
@@ -30,6 +30,7 @@
 #include <gtk/gtktable.h>
 #include <libgnome/gnome-config.h>
 #include <libgnome/gnome-i18n.h>
+#include <libgnomevfs/gnome-vfs.h>
 
 /* local function prototypes */
 void new_user_druid_finish_cb (GtkWidget *widget, gpointer dummy);
@@ -39,8 +40,14 @@ gboolean new_user_druid_delete_event_cb (GtkWidget *widget, GdkEventAny *event,
 void new_user_druid_create_smart_bookmarks_check_button_toggled_cb 
 (GtkWidget *toggle);
 static void add_smart (BookmarkItem *parent, gchar *name, gchar *nick,
-		       gchar *url, gchar *pixmap_file, gchar *notes);
+		       gchar *url, gchar *smarturl, gchar *pixmap_file,
+		       gchar *notes);
 static void add_bookmarklets (BookmarkItem *parent);
+static gchar *get_first_directory (const gchar *start_dir);
+
+/* Should not use these two here... */
+BookmarkTranslator *bookmarks_io_netscape_init (void);
+BookmarkTranslator *bookmarks_io_mozilla_init (void);
 
 /**
  * new_user_druid_finish_cb: called when the user gets to the end of
@@ -68,7 +75,7 @@ new_user_druid_finish_cb (GtkWidget *widget, gpointer dummy)
 	gboolean debian_packages_checked;
 	gboolean bugzilla_checked;
 	gboolean stock_quotes_checked;
-	gboolean imdb_movies_checked;
+	gboolean linuxpackages_checked;
 	gboolean google_images_checked;
 	gboolean bookmarklets_checked;
 	gboolean http_checked;
@@ -113,8 +120,8 @@ new_user_druid_finish_cb (GtkWidget *widget, gpointer dummy)
 		  &bugzilla_checked, },
 		{ "new_user_druid_stock_quotes_check_button",
 		  &stock_quotes_checked, },
-		{ "new_user_druid_imdb_movies_check_button",
-		  &imdb_movies_checked, },
+		{ "new_user_druid_linuxpackages_check_button",
+		  &linuxpackages_checked, },
 		{ "new_user_druid_google_images_check_button",
 		  &google_images_checked, },
 		{ "new_user_druid_bookmarklets_check_button",
@@ -222,19 +229,48 @@ new_user_druid_finish_cb (GtkWidget *widget, gpointer dummy)
 			 * right language (normally detection is done at
 			 * the google homepage I think -- MattA) */
 			add_smart (toolbar, "Google", "google",
+				   "http://www.google.com",
 				   _("http://www.google.com/search?q=%s"),
 				   SHARE_DIR "/google.png", NULL);
 		}
 		if (google_groups_checked)
 		{
 			add_smart (toolbar, "Usenet", "news",
+				   "http://groups.google.com",
 				   "http://groups.google.com/groups?q=%s"
 				   "&meta=site%3Dgroups", 
 				   SHARE_DIR "/google_groups.png", NULL);
 		}
+		if (google_images_checked)
+		{
+			add_smart (toolbar, "Images", "img",
+				   "http://images.google.com",
+				   "http://images.google.com/images?q=%s",
+				   SHARE_DIR "/google_images.png", NULL);
+		}
+		if (dictionary_checked)
+		{
+			/* URL marked as translatable so that non-English
+			 * dictionaries can be used -- don't forget to
+			 * make the equivalent change in the glade file */
+			add_smart (toolbar, _("Dictionary"), "dict",
+				   /* xgettext: no-c-format */
+				   _("http://www.m-w.com"),
+				   /* xgettext: no-c-format */
+				   _("http://www.m-w.com/cgi-bin/"
+				     "dictionary?va=%s"), NULL, NULL);
+		}
+		if (yahoo_checked)
+		{
+			add_smart (toolbar, "Yahoo!", "yahoo",
+				   "http://www.yahoo.com",
+				   "http://search.yahoo.com/bin/search?p=%s",
+				   NULL, NULL);
+		}
 		if (dogpile_checked)
 		{
 			add_smart (toolbar, "Dogpile", "dogpile",
+				   "http://dogpile.com",
 				   "http://search.dogpile.com/texis/search"
 				   "?q=%s&geo=no&fs=web&av=custom&"
 				   "engines=goto&engines=looksmart&"
@@ -246,25 +282,27 @@ new_user_druid_finish_cb (GtkWidget *widget, gpointer dummy)
 				   "engines=altavista&engines=yahoo",
 				   NULL, NULL);
 		}
-		if (yahoo_checked)
+		if (linuxpackages_checked)
 		{
-			add_smart (toolbar, "Yahoo!", "yahoo",
-				   "http://search.yahoo.com/bin/search?p=%s",
-				   NULL, NULL);
+			add_smart (toolbar, "LinuxPackages", "linuxpackages",
+				   "http://www.linuxpackages.net",
+				   "http://www.linuxpackages.net/"
+				   "search_view.php?by=name&what_cat=&"
+				   "subcat=&ver=8&name=%s",
+				   SHARE_DIR "/slack.png", NULL);
 		}
-		if (dictionary_checked)
+		if (stock_quotes_checked)
 		{
-			/* URL marked as translatable so that non-English
-			 * dictionaries can be used -- don't forget to
-			 * make the equivalent change in the glade file */
-			add_smart (toolbar, _("Dictionary"), "dict",
-				   /* xgettext: no-c-format */
-				   _("http://www.m-w.com/cgi-bin/"
-				     "dictionary?va=%s"), NULL, NULL);
+			add_smart (toolbar, "Stocks", "st",
+				   "http://finance.yahoo.com",
+				   "http://finance.yahoo.com/q?s=%s"
+				   "&d=c&k=c4&t=1d",
+				   SHARE_DIR "/stocks.png", NULL);
 		}
 		if (bugzilla_checked)
 		{
 			add_smart (toolbar, "Bug #", "bug",
+				   "http://bugzilla.gnome.org",
 				   "http://bugzilla.gnome.org/"
 				   "show_bug.cgi?id=%s", 
 				   SHARE_DIR "/bug.png", NULL);
@@ -272,12 +310,14 @@ new_user_druid_finish_cb (GtkWidget *widget, gpointer dummy)
 		if (freshmeat_checked)
 		{
 			add_smart (toolbar, "Freshmeat", "fm",
+				   "http://freshmeat.net",
 				   "http://freshmeat.net/search?q=%s",
 				   SHARE_DIR "/freshmeat.png", NULL);
 		}
 		if (rpmfind_checked)
 		{
 			add_smart (toolbar, "RPMFind", "rpm",
+				   "http://rpmfind.net",
 				   "http://rpmfind.net/linux/rpm2html/"
 				   "search.php?query=%s",
 				   SHARE_DIR "/rpm.png", NULL);
@@ -285,31 +325,13 @@ new_user_druid_finish_cb (GtkWidget *widget, gpointer dummy)
 		if (debian_packages_checked)
 		{
 			add_smart (toolbar, "Debian", "deb",
+				   "http://packages.debian.org",
 				   "http://packages.debian.org/cgi-bin/"
 				   "search_packages.pl?keywords=%s&"
 				   "searchon=names&subword=1&version=all&"
 				   "release=all", 
 				   SHARE_DIR "/debian.png", NULL);
 		}
-		if (google_images_checked)
-		{
-			add_smart (toolbar, "Images", "img",
-				   "http://images.google.com/images?q=%s",
-				   SHARE_DIR "/google_images.png", NULL);
-		}
-		if (stock_quotes_checked)
-		{
-			add_smart (toolbar, "Stocks", "st",
-				   "http://finance.yahoo.com/q?s=%s"
-				   "&d=c&k=c4&t=1d",
-				   SHARE_DIR "/stocks.png", NULL);
-		}
-		if (imdb_movies_checked)
-		{
-			add_smart (toolbar, "IMDb", "imdb",
-				   "http://www.imdb.com/Find?%s", 
-				   SHARE_DIR "/imdb_movies.png", NULL);
-		}
 		if (bookmarklets_checked)
 		{
 			add_bookmarklets (toolbar);
@@ -319,13 +341,50 @@ new_user_druid_finish_cb (GtkWidget *widget, gpointer dummy)
 	/* import bookmarks */
 	if (import_netscape_bookmarks_checked)
 	{
-  	        /* FIXME: dont use this callback directly! */
-	        bookmarks_import_netscape_activate_cb (NULL, NULL);
+		gchar *prefs_dir = g_get_home_dir ();
+		if (prefs_dir) {
+			gchar *fname = g_strconcat (prefs_dir, 
+						    NETSCAPE_BOOKMARKS, NULL);
+			BookmarkTranslator *t = bookmarks_io_netscape_init ();
+			BookmarkItem *loaded = t->load_from_file (fname, NULL);
+			bookmarks_merge_trees (bookmarks_root, loaded);
+			g_free (fname);
+		}
 	}
 	if (import_mozilla_bookmarks_checked)
 	{
-  	        /* FIXME: dont use this callback directly! */
-	        bookmarks_import_mozilla_activate_cb (NULL, NULL);
+		gchar *prefs_dir = NULL;
+		gchar *default_dir;
+
+		default_dir =
+			g_strconcat (g_get_home_dir(), MOZILLA_PREFS_DIR, NULL);
+
+		/* descend first dir (_usually_ .mozilla/default) */
+		prefs_dir = get_first_directory (default_dir);
+		g_free (default_dir);
+
+		if (prefs_dir)
+		{
+			default_dir = prefs_dir;
+			
+			/* descend second dir... something like
+			 * (.mozilla/default/n8u3dbaou.slt) */
+			prefs_dir = get_first_directory (default_dir);
+			g_free (default_dir);
+
+			if (prefs_dir)
+			{
+				gchar *fname = g_strconcat (prefs_dir, 
+					"/bookmarks.html", NULL);
+				BookmarkTranslator *t =
+					bookmarks_io_mozilla_init ();
+				BookmarkItem *loaded =
+					t->load_from_file (fname, NULL);
+				bookmarks_merge_trees (bookmarks_root, loaded);
+				g_free (prefs_dir);
+				g_free (fname);
+			}
+		}
 	}
 	
 	/* set URL handlers as requested */
@@ -382,7 +441,10 @@ new_user_druid_finish_cb (GtkWidget *widget, gpointer dummy)
 	/* create an initial window if needed */
 	if (g_list_length (all_windows) == 0)
 	{
-		embed_create_default (NULL, TRUE);
+		embed_create_in_window (NULL, NULL, NULL,
+					EMBED_CREATE_LOAD_DEFAULT_URL |
+					EMBED_CREATE_FORCE_JUMP |
+					EMBED_CREATE_RAISE_WINDOW);
 	}
 }
 
@@ -402,7 +464,10 @@ new_user_druid_cancel_cb (GtkWidget *widget, gpointer dummy)
 	/* create an initial window if needed */
 	if (g_list_length (all_windows) == 0)
 	{
-		embed_create_default (NULL, TRUE);
+		embed_create_in_window (NULL, NULL, NULL, 
+					EMBED_CREATE_LOAD_DEFAULT_URL |
+					EMBED_CREATE_FORCE_JUMP |
+					EMBED_CREATE_RAISE_WINDOW);
 	}
 }
 
@@ -417,7 +482,10 @@ new_user_druid_delete_event_cb (GtkWidget *widget, GdkEventAny *event,
 	/* create an initial window if needed */
 	if (g_list_length (all_windows) == 0)
 	{
-		embed_create_default (NULL, TRUE);
+		embed_create_in_window (NULL, NULL, NULL,
+					EMBED_CREATE_LOAD_DEFAULT_URL |
+					EMBED_CREATE_FORCE_JUMP |
+					EMBED_CREATE_RAISE_WINDOW);
 	}
 	
 	/* still destroy the druid window */
@@ -452,7 +520,7 @@ new_user_druid_create_smart_bookmarks_check_button_toggled_cb
  */
 static void
 add_smart (BookmarkItem *parent, gchar *name, gchar *nick, 
-	   gchar *url, gchar *pixmap_file, gchar * notes)
+	   gchar *url, gchar *smarturl, gchar *pixmap_file, gchar * notes)
 {
 	BookmarkItem *site;
 	gchar* utfstr;
@@ -461,6 +529,7 @@ add_smart (BookmarkItem *parent, gchar *name, gchar *nick,
 	/* create the bookmarks */
 	site = bookmarks_new_bookmark (BM_SITE, TRUE, utfstr, url, nick, notes, 
 				       pixmap_file);
+	site->smarturl = g_strdup (smarturl);
 	g_free(utfstr);
 	
 	/* add it to the toolbar */
@@ -484,6 +553,7 @@ add_bookmarklets (BookmarkItem *parent)
 	} bookmarklets[] = {
 		{ N_("More Bookmarklets"), "http://www.bookmarklets.com/" }, 
 		{ "separator", NULL },
+		{ N_("Old versions of this page"), "javascript:location.href=\"http://web.archive.org/web/*/\"+location.href" },
 		{ N_("More Info About..."), "javascript:if(frames.length>0){F=' (Open frame in new window first.)'}else{F=''}Q=document.getSelection();if(!Q){void(Q=prompt('No text selected on page.'+F+'\\n\\nKeywords...?',''))};if(Q)location.href='http://bookmarklets.com/moreinfo.phtml?q='+escape(Q)" },
 		{ "separator", NULL },
 		{ N_("Zoom on First Image..."), "javascript:if(document.images.length<1){alert('No image!')}else{D0iB=document.images[0];sZ2T=prompt('Resize by..','1.5');if(sZ2T){with(document){write('<img src='+D0iB.src+' width='+sZ2T*D0iB.width+' height='+sZ2T*D0iB.height+'>');close()}}}void(null)" },
@@ -503,6 +573,7 @@ add_bookmarklets (BookmarkItem *parent)
 		{ "separator", NULL },
 		{ N_("Read Cookie for Site"), "javascript:if(document.cookie.length<1){alert('No cookie for this site.')}else{alert('Cookie for this site: '+document.cookie)}" },
 		{ N_("AutoFill Anonymous"), "javascript:function ROIoiW(){var i=0,j,A='anonymous',D,E,F=document.forms;while(i<F.length){E=F[i].elements;for(j=0;j<E.length;j++){D=E[j];if(D.type=='text'){D.value=(D.name.toUpperCase().indexOf('MAIL')!=-1)?A+'@example.com':A}}i++}}ROIoiW();void(null)" },
+		{ N_("Translate to English"), "javascript:void(location.href='http://translate.google.com/translate?hl=en&u='+location.href)"},
 		{ NULL, NULL}
 	};
 	gchar* utfstr;
@@ -538,3 +609,35 @@ add_bookmarklets (BookmarkItem *parent)
 	}
 }
 
+static gchar *
+get_first_directory (const gchar *start_dir) 
+{
+	GnomeVFSResult rc;
+	GList *list = NULL;
+	GList *node;
+	GnomeVFSFileInfo *info;
+	gchar *prefs_dir = NULL;
+
+	rc = gnome_vfs_directory_list_load(&list,
+					   start_dir,
+					   (GNOME_VFS_FILE_INFO_GET_MIME_TYPE
+					    | GNOME_VFS_FILE_INFO_FORCE_FAST_MIME_TYPE
+					    | GNOME_VFS_FILE_INFO_FOLLOW_LINKS),
+					   NULL);
+
+	for (node = list; node; node = node->next) {
+		info = node->data;
+
+		if (info->type == GNOME_VFS_FILE_TYPE_DIRECTORY && 
+				strcmp (info->name, ".") &&
+				strcmp (info->name, ".."))
+		{
+			prefs_dir = g_strdup_printf("%s/%s/",start_dir, info->name);
+			break;
+		}
+	}
+
+	gnome_vfs_file_info_list_free(list);
+
+	return prefs_dir;
+}
