@@ -2,20 +2,30 @@
 #define CONF_GENERAL_STARTPAGE_TYPE "/apps/galeon/Browsing/General/startpage_type"
 #define CONF_GENERAL_ALWAYS_SAVE_SESSION "/apps/galeon/Browsing/General/always_save_session"
 #define CONF_GENERAL_NEWPAGE_TYPE "/apps/galeon/Browsing/General/newpage_type"
+#define CONF_GENERAL_HOST_FAVICONS "/apps/galeon/Browsing/General/host_favicons"
 #define CONF_BOOKMARKS_TITLE "/apps/galeon/Browsing/Bookmarks/bookmark_title"
 #define CONF_BOOKMARKS_XITEMS "/apps/galeon/Browsing/Bookmarks/bm_extra_items_location"
 #define CONF_BOOKMARKS_ENTRY_WIDTH "/apps/galeon/Browsing/Bookmarks/smart_bm_entry_width"
-#define CONF_BOOKMARKS_GOTO_BUTTON "/apps/galeon/Browsing/Bookmarks/smart_bm_goto_button"
 #define CONF_BOOKMARKS_HISTORY "/apps/galeon/Browsing/Bookmarks/smart_bm_history"
 #define CONF_BOOKMARKS_CLEAR "/apps/galeon/Browsing/Bookmarks/smart_bm_clear"
+#define CONF_BOOKMARKS_TOOLTIPS "/apps/galeon/Browsing/Bookmarks/bm_tooltips"
+#define CONF_BOOKMARKS_SMARTBOOKMARK_HIDE_ARROWS "/apps/galeon/Browsing/Bookmarks/smartbookmarks_hide_arrows"
+#define CONF_BOOKMARKS_REMOVE_CONFIRMATION_MIN "/apps/galeon/Browsing/Bookmarks/remove_confirmation_min"
 #define CONF_FIND_MATCH_CASE "/apps/galeon/Browsing/Find/match_case"
 #define CONF_FIND_MATCH_WORD "/apps/galeon/Browsing/Find/match_word"
 #define CONF_FIND_SEARCH_IN_FRAMES "/apps/galeon/Browsing/Find/search_in_frames"
 #define CONF_FIND_AUTOWRAP "/apps/galeon/Browsing/Find/autowrap"
+#define CONF_FIND_WORD "/apps/galeon/Browsing/Find/word"
 #define CONF_FILTERING_ALLOW_POPUPS "/apps/galeon/Advanced/Filtering/allow_popups"
-#define CONF_BOOKMARKS_FAVICONS_ENABLED "/apps/galeon/Browsing/Bookmarks/favicons_enabled"
+#define CONF_FILTERING_DEFAULT_STATUSBAR "/apps/galeon/Advanced/Filtering/default_allow_statusbar"
+#define CONF_FILTERING_DEFAULT_POPUPS "/apps/galeon/Advanced/Filtering/default_allow_popups"
+#define CONF_FILTERING_DEFAULT_PROMPTS "/apps/galeon/Advanced/Filtering/default_allow_prompts"
+#define CONF_FILTERING_DEFAULT_RESIZE "/apps/galeon/Advanced/Filtering/default_allow_resize"
 #define CONF_HISTORY_AUTOCOMP_ENABLE "/apps/galeon/Browsing/History/completion_enabled"
-#define CONF_COMPLETION_SORT "/apps/galeon/Browsing/History/completion_sorting_type"
+#define CONF_COMPLETION_SHOW_LIST_AUTO "/apps/galeon/Browsing/History/completion_show_list_auto"
+#define CONF_COMPLETION_MAX_SHORT "/apps/galeon/Browsing/History/completion_max_short"
+#define CONF_COMPLETION_SHOW_TITLES "/apps/galeon/Browsing/History/completion_show_titles"
+#define CONF_COMPLETION_HUGE_WINDOW "/apps/galeon/Browsing/History/completion_huge"
 #define CONF_TOOLBAR_URL_LOCATION "/apps/galeon/UI/Toolbar/url_location"
 #define CONF_STATE_WINWIDTH "/apps/galeon/State/winwidth"
 #define CONF_STATE_WINHEIGHT "/apps/galeon/State/winheight"
@@ -27,7 +37,8 @@
 #define CONF_RENDERING_VISITED_LINKS "/apps/galeon/Rendering/FontsColors/visited_link_color"
 #define CONF_RENDERING_UNVISITED_LINKS "/apps/galeon/Rendering/FontsColors/unvisited_link_color"
 #define CONF_RENDERING_UNDERLINE_LINKS "/apps/galeon/Rendering/FontsColors/underline_links"
-#define CONF_RENDERING_USER_STYLESHEET "/apps/galeon/Rendering/user_stylesheet"
+#define CONF_RENDERING_STYLESHEETS "/apps/galeon/Rendering/stylesheets"
+#define CONF_RENDERING_DEFAULT_STYLESHEETS "/apps/galeon/Rendering/default_stylesheets"
 #define CONF_TABS_TABBED "/apps/galeon/UI/Tabs/tabbed"
 #define CONF_TABS_TABBED_POPUPS "/apps/galeon/UI/Tabs/tabbed_popups"
 #define CONF_TABS_TABBED_AUTOJUMP "/apps/galeon/UI/Tabs/tabbed_autojump"
@@ -40,10 +51,11 @@
 #define CONF_TABS_TABBED_SEPARATOR "/apps/galeon/UI/Tabs/tabbed_separator"
 #define CONF_TABS_TABBED_VOWEL "/apps/galeon/UI/Tabs/tabbed_vowel"
 #define CONF_TABS_TABBED_SHORTEN_STYLE "/apps/galeon/UI/Tabs/tabbed_shorten_style"
-#define  CONF_TABS_TABBED_SHORTEN_CHARS "/apps/galeon/UI/Tabs/tabbed_shorten"
-#define  CONF_TABS_TABBED_SHORTEN_POINTS "/apps/galeon/UI/Tabs/tabbed_shorten_points"
+#define CONF_TABS_TABBED_SHORTEN_CHARS "/apps/galeon/UI/Tabs/tabbed_shorten"
+#define CONF_TABS_TABBED_SHORTEN_POINTS "/apps/galeon/UI/Tabs/tabbed_shorten_points"
 #define CONF_TABS_TABBED_LOADING_COLOR "/apps/galeon/UI/Tabs/tabbed_loading_color"
 #define CONF_TABS_TABBED_NEW_COLOR "/apps/galeon/UI/Tabs/tabbed_new_color"
+#define CONF_TABS_TABBED_FAVICONS "/apps/galeon/UI/Tabs/favicons_in_tabs"
 #define CONF_WINDOWS_FULLSCREEN_STACKING "/apps/galeon/UI/Windows/fullscreen_stacking"
 #define CONF_WINDOWS_FS_SHOW_MENUBAR "/apps/galeon/UI/Windows/show_menubar_in_fullscreen"
 #define CONF_WINDOWS_FS_SHOW_TOOLBARS "/apps/galeon/UI/Windows/show_toolbars_in_fullscreen"
@@ -53,13 +65,12 @@
 #define CONF_WINDOWS_SHOW_TOOLBARS "/apps/galeon/UI/Windows/show_toolbars"
 #define CONF_WINDOWS_SHOW_BOOKMARKS "/apps/galeon/UI/Windows/show_bookmarks"
 #define CONF_WINDOWS_SHOW_STATUSBAR "/apps/galeon/UI/Windows/show_statusbar"
-#define CONF_WINDOWS_SHOW_HISTORY_DOCK "/apps/galeon/UI/Windows/show_history_dock"
-#define CONF_WINDOWS_SHOW_BOOKMARKS_DOCK "/apps/galeon/UI/Windows/show_bookmarks_dock"
+#define CONF_WINDOWS_SHOW_DOCK "/apps/galeon/UI/Windows/show_dock"
 #define CONF_FILTERING_IMAGE_LOADING_TYPE "/apps/galeon/Advanced/Filtering/image_loading_type"
 #define CONF_FILTERING_ANIMATE_TYPE "/apps/galeon/Advanced/Filtering/animate_type"
 #define CONF_FILTERING_STATUSBAR_REWRITE "/apps/galeon/Advanced/Filtering/statusbar_rewrite"
+#define CONF_FILTERING_ZONES "/apps/galeon/Advanced/Filtering/zones"
 #define CONF_LANGUAGE_DEFAULT_CHARSET "/apps/galeon/Rendering/Language/default_charset"
-#define CONF_LANGUAGE_DEFAULT_CHARSET_TITLE "/apps/galeon/Rendering/Language/default_charset_title"
 #define CONF_RENDERING_DEFAULT_FONT "/apps/galeon/Rendering/FontsColors/default_font"
 #define CONF_TOOLBAR_SPINNER_DIR "/apps/galeon/UI/Toolbar/spinner_dir"
 #define CONF_TOOLBAR_THEME_DIR "/apps/galeon/UI/Toolbar/theme_dir"
@@ -72,18 +83,21 @@
 #define CONF_DOWNLOADING_DIR "/apps/galeon/Handlers/Downloading/download_dir"
 #define CONF_DOWNLOADING_ASK_DIR "/apps/galeon/Handlers/Downloading/ask_for_download_dir"
 #define CONF_DOWNLOADING_DOWNLOADER "/apps/galeon/Handlers/Downloading/downloader"
-#define CONF_DOWNLOADING_AUTO_DOWNLOAD "/apps/galeon/Handlers/Downloading/auto_download"
 #define CONF_DOWNLOADING_EXTERNAL_COMMAND "/apps/galeon/Handlers/Downloading/external_download_command"
 #define CONF_DOWNLOADING_EXTERNAL_TERMINAL "/apps/galeon/Handlers/Downloading/external_download_terminal"
+#define CONF_DOWNLOADING_KEEP_OPEN "/apps/galeon/Handlers/Downloading/keep_open"
 #define CONF_FILTERING_JAVA_ENABLED "/apps/galeon/Advanced/Filtering/java_enabled"
 #define CONF_FILTERING_JAVASCRIPT_ENABLED "/apps/galeon/Advanced/Filtering/javascript_enabled"
 #define CONF_NETWORK_PROXY_MODE "/apps/galeon/Advanced/Network/proxy_mode"
 #define CONF_NETWORK_HTTP_PROXY "/apps/galeon/Advanced/Network/http_proxy"
 #define CONF_NETWORK_SSL_PROXY "/apps/galeon/Advanced/Network/ssl_proxy"
 #define CONF_NETWORK_FTP_PROXY "/apps/galeon/Advanced/Network/ftp_proxy"
+#define CONF_NETWORK_SOCKS_PROXY "/apps/galeon/Advanced/Network/socks_proxy"
 #define CONF_NETWORK_HTTP_PROXY_PORT "/apps/galeon/Advanced/Network/http_proxy_port"
 #define CONF_NETWORK_SSL_PROXY_PORT "/apps/galeon/Advanced/Network/ssl_proxy_port"
 #define CONF_NETWORK_FTP_PROXY_PORT "/apps/galeon/Advanced/Network/ftp_proxy_port"
+#define CONF_NETWORK_SOCKS_PROXY_PORT "/apps/galeon/Advanced/Network/socks_proxy_port"
+#define CONF_NETWORK_SOCKS_PROXY_VERSION "/apps/galeon/Advanced/Network/socks_proxy_version"
 #define CONF_NETWORK_NO_PROXIES_FOR "/apps/galeon/Advanced/Network/no_proxies_for"
 #define CONF_NETWORK_PROXY_KA "/apps/galeon/Advanced/Network/proxy_keepalive"
 #define CONF_NETWORK_PROXY_AUTO_URL "/apps/galeon/Advanced/Network/proxy_auto_url"
@@ -91,6 +105,9 @@
 #define CONF_NETWORK_MEMORY_CACHE "/apps/galeon/Advanced/Network/mem_cache_size"
 #define CONF_NETWORK_CACHE_COMPARE "/apps/galeon/Advanced/Network/cache_compare"
 #define CONF_NETWORK_USER_AGENT "/apps/galeon/Advanced/Network/user_agent"
+#define CONF_SECURITY_SSL2 "/apps/galeon/Advanced/Security/ssl2"
+#define CONF_SECURITY_SSL3 "/apps/galeon/Advanced/Security/ssl3"
+#define CONF_SECURITY_TLS "/apps/galeon/Advanced/Security/tls"
 #define CONF_RENDERING_LANGUAGE "/apps/galeon/Rendering/Language/lang"
 #define CONF_BOOKMARKS_AB_ENABLED "/apps/galeon/Browsing/Bookmarks/autobookmarks_enabled"
 #define CONF_BOOKMARKS_AB_COUNT "/apps/galeon/Browsing/Bookmarks/autobookmarks_count"
@@ -99,12 +116,14 @@
 #define CONF_PERSISTENT_COOKIE_WARN "/apps/galeon/Advanced/Persistent/cookie_warn"
 #define CONF_PERSISTENT_PASSWORDS_SAVE "/apps/galeon/Advanced/Persistent/passwords_save"
 #define CONF_NETWORK_HTTP_VERSION "/apps/galeon/Advanced/Network/http_version"
-#define CONF_CRASH_RECOVERY "/apps/galeon/Advanced/Crash/crash_recovery"
 #define CONF_CRASH_CRASHED "/apps/galeon/Advanced/Crash/crashed"
 #define CONF_STATE_SESSION_SAVED "/apps/galeon/State/session_saved"
+#define CONF_STATE_RECOVERY_METHOD "/apps/galeon/State/recovery_method"
+#define CONF_STATE_WINDOW_CLOSE_CONFIRMATION "/apps/galeon/State/window_close_confirmation"
+#define CONF_STATE_SESSION_OPEN_CONFIRMATION "/apps/galeon/State/session_open_confirmation"
 #define CONF_MOUSE_HISTORY_POPUP "/apps/galeon/UI/Mouse/history_popup"
-#define CONF_MOUSE_RMB_ACTION "/apps/galeon/UI/Mouse/right_button_action"
 #define CONF_MOUSE_MMB_ACTION "/apps/galeon/UI/Mouse/middle_button_action"
+#define CONF_MOUSE_RMB_ACTION "/apps/galeon/UI/Mouse/right_button_action"
 #define CONF_MOUSE_WHEEL_NOKEY_ACTION "/apps/galeon/UI/Mouse/wheel_nokey_action"
 #define CONF_MOUSE_WHEEL_SHIFT_ACTION "/apps/galeon/UI/Mouse/wheel_shift_action"
 #define CONF_MOUSE_WHEEL_CTRL_ACTION "/apps/galeon/UI/Mouse/wheel_ctrl_action"
@@ -117,7 +136,7 @@
 #define CONF_MOUSE_WHEEL_CTRL_STEP "/apps/galeon/UI/Mouse/wheel_ctrl_step"
 #define CONF_MOUSE_WHEEL_ALT_STEP "/apps/galeon/UI/Mouse/wheel_alt_step"
 #define CONF_MOUSE_WHEEL_SHIFT_STEP "/apps/galeon/UI/Mouse/wheel_shift_step"
-#define CONF_HISTORY_AUTOZOOM "/apps/galeon/Browsing/History/autozoom"
+#define CONF_MOUSE_WHEEL_STEP_SIZE "/apps/galeon/UI/Mouse/wheel_step_size"
 #define CONF_HISTORY_EXPIRE "/apps/galeon/Browsing/History/expire"
 #define CONF_HISTORY_SORT_ORDER "/apps/galeon/Browsing/History/sort_order"
 #define CONF_HISTORY_SORT_COLUMN "/apps/galeon/Browsing/History/sort_column"
@@ -127,25 +146,45 @@
 #define CONF_PRINT_BOTTOM_MARGIN "/apps/galeon/Print/bottom_margin"
 #define CONF_PRINT_TOP_MARGIN "/apps/galeon/Print/top_margin"
 #define CONF_PRINT_LEFT_MARGIN "/apps/galeon/Print/left_margin"
-#define CONF_PRINT_TIGHT_MARGIN "/apps/galeon/Print/right_margin"
+#define CONF_PRINT_RIGHT_MARGIN "/apps/galeon/Print/right_margin"
+
+/* header/footer stuff */
+#define CONF_PRINT_PAGE_TITLE "/apps/galeon/Print/page_title_toggle"
+#define CONF_PRINT_PAGE_URL "/apps/galeon/Print/page_url_toggle"
+#define CONF_PRINT_DATE "/apps/galeon/Print/date_toggle"
+#define CONF_PRINT_PAGE_NUMBERS "/apps/galeon/Print/page_numbers_toggle"
+
 #define CONF_PRINT_PRINTER "/apps/galeon/Print/printer"
 #define CONF_PRINT_FILE "/apps/galeon/Print/file"
 #define CONF_PRINT_PRINTON "/apps/galeon/Print/printon"
 #define CONF_PRINT_PAPER "/apps/galeon/Print/paper"
+#define CONF_PRINT_ALL_PAGES "/apps/galeon/Print/all_pages"
+#define CONF_PRINT_FIRST "/apps/galeon/Print/first"
+#define CONF_PRINT_COLOR "/apps/galeon/Print/color"
+#define CONF_PRINT_ORIENTATION "/apps/galeon/Print/orientation"
+#define CONF_PRINT_FRAME_TYPE "/apps/galeon/Print/frame_type"
 #define CONF_BOOKMARKS_SHORTTEXT "/apps/galeon/Browsing/Bookmarks/shorttext"
 #define CONF_STATE_HISTORY_DOCK_WIDTH "/apps/galeon/State/history_dock_width"
 #define CONF_STATE_BOOKMARKS_DOCK_WIDTH "/apps/galeon/State/bookmarks_dock_width"
 #define CONF_STATE_BOOKMARKS_DOCK_ENABLE "/apps/galeon/State/bookmarks_dock_enable"
 #define CONF_STATE_HISTORY_DOCK_ENABLE "/apps/galeon/State/history_dock_enable"
+#define CONF_STATE_BM_DOCK_CONTROLS "/apps/galeon/State/bookmarks_dock_controls_show"
+#define CONF_STATE_BM_DLG_CONTROLS "/apps/galeon/State/bookmarks_dialog_controls_show"
+#define CONF_STATE_JS_CONSOLE_SHOW_MENUBAR "/apps/galeon/State/js_console_show_menubar"
+#define CONF_STATE_JS_CONSOLE_SHOW_TOOLBAR "/apps/galeon/State/js_console_show_toolbar"
+#define CONF_STATE_JS_CONSOLE_SHOW_ENTRYBAR "/apps/galeon/State/js_console_show_entrybar"
+#define CONF_STATE_JS_CONSOLE_SHOW "/apps/galeon/State/js_console_show"
+#define CONF_STATE_LAST_DOWNLOAD_DIR "/apps/galeon/State/last_download_dir"
 #define CONF_MOZ_MISMATCH "/apps/galeon/moz_mismatch_remember"
 #define CONF_MOZ_MISMATCH_ACTION "/apps/galeon/moz_mismatch_action"
-#define CONF_CRASH_BEHAVIOUR "/apps/galeon/Advanced/Crash/crash_behaviour"
 #define CONF_STATE_PATH "/apps/galeon/State"
 #define CONF_LANGUAGE_AUTODETECT_CHARSET "/apps/galeon/Rendering/Language/autodetect_charset"
 #define CONF_TOOLBAR_BOOKMARKS_EDITOR_STYLE \
 	"/apps/galeon/UI/Toolbar/bookmarks_editor_style"
 #define CONF_TOOLBAR_BOOKMARKS_EDITOR_DOCKED_STYLE \
 	"/apps/galeon/UI/Toolbar/bookmarks_editor_docked_style"
+#define CONF_TOOLBAR_JS_CONSOLE_STYLE \
+	"/apps/galeon/UI/Toolbar/js_console_style"
 #define CONF_TOOLBAR_STYLE "/apps/galeon/UI/Toolbar/toolbar_style"
 #define CONF_TOOLBAR_ICON_LIST "/apps/galeon/UI/Toolbar/icon_list"
 #define CONF_STATE_SAVE_DIR           "/apps/galeon/State/savedir"
@@ -161,4 +200,11 @@
 	g_strconcat ("/apps/galeon/State/SmartBookmarks/History/", Path, NULL)
 #define CONF_STATE_SMARTBOOKMARK_ENTRY_SIZE(Path) \
 	g_strconcat ("/apps/galeon/State/SmartBookmarks/EntrySize/", Path, NULL)
-
+#define CONF_STATE_PAGE_INFO_IMAGES_COLUMN "/apps/galeon/State/PageInfo/images_column"
+#define CONF_STATE_PAGE_INFO_FORMS_COLUMN "/apps/galeon/State/PageInfo/forms_column"
+#define CONF_STATE_PAGE_INFO_CSS_COLUMN "/apps/galeon/State/PageInfo/css_column"
+#define CONF_STATE_PAGE_INFO_LINKS_COLUMN "/apps/galeon/State/PageInfo/links_column"
+#define CONF_STATE_PAGE_INFO_IMAGES_ORDER "/apps/galeon/State/PageInfo/images_order"
+#define CONF_STATE_PAGE_INFO_FORMS_ORDER "/apps/galeon/State/PageInfo/forms_order"
+#define CONF_STATE_PAGE_INFO_CSS_ORDER "/apps/galeon/State/PageInfo/css_order"
+#define CONF_STATE_PAGE_INFO_LINKS_ORDER "/apps/galeon/State/PageInfo/links_order"
