@@ -19,6 +19,10 @@
 #ifndef __print_h
 #define __print_h
 
+
+#include "galeon.h"
+#include <glib.h>
+
 typedef struct
 {
 	gboolean print_to_file;
@@ -32,10 +36,30 @@ typedef struct
 	gint pages;
 	gint from_page;
 	gint to_page;
+	gint frame_type;
+	gint orientation;
 	gboolean reversed;
+	gboolean print_color;
+
+	 /* 
+	 * &T - title
+	 * &U - Document URL
+	 * &D - Date/Time
+	 * &P - Page Number
+	 * &PT - Page Number with total Number of Pages (example: 1 of 34)
+	 *
+	 * So, if headerLeftStr = "&T" the title and the document URL
+	 * will be printed out on the top left-hand side of each page.
+	 */
+	gchar *header_left_string;
+	gchar *header_center_string;
+	gchar *header_right_string;
+	gchar *footer_left_string;
+	gchar *footer_center_string;
+	gchar *footer_right_string;
 }
 PrintInfo;
 
-void print_show_dialog (GaleonWindow *window);
+void print_show_dialog (GaleonEmbed *embed);
 
 #endif /* __print_h */
