@@ -21,10 +21,9 @@ extern "C" {
 #endif
 
 /* themes.c */
-PixmapData *get_theme_pixmap (const gchar *filename, gboolean prelight);
-void reset_theme_pixmap_cache (void);
-gint get_theme_pixmap_height ();
-gchar *get_theme_dir ();
+PixmapData *themes_get_pixmap (const gchar *filename, gboolean prelight);
+gint themes_get_height (void);
+void themes_reset_cache (void);
 
 #ifdef __cplusplus
 }
