@@ -24,13 +24,14 @@
 #include "bookmarks.h"
 #include "toolbar.h"
 #include "embed.h"
-#include "misc.h"
 #include "mozilla.h"
 #include "themes.h"
+#include "stylesheets.h"
+#include "favicon.h"
+#include "link_interfaces.h"
 
 #include <libgnome/gnome-util.h>
-
-extern gboolean fullscreen_active;
+#include <libgnomeui/gtkpixmapmenuitem.h>
 
 void main_toolbar_notifier(GConfClient *client,
 			   guint cnxn_id,
@@ -53,6 +54,8 @@ tabs_appeareance_notifier(GConfClient *client,
 			  guint cnxn_id,
 			  GConfEntry *entry,
 			  GaleonWindow *window);
+void tabs_favicons_notifier(GConfClient *client, guint cnxn_id,
+			    GConfEntry *entry, GaleonWindow *window);
 void
 statusbar_appeareance_notifier(GConfClient *client,
 			       guint cnxn_id,
@@ -131,10 +134,11 @@ void stylesheet_notifier (GConfClient *client,
 			  GConfEntry *entry,
 			  GaleonWindow *window);
 
-void update_ui_hack_notifier(GConfClient *client,
-			     guint cnxn_id,
-			     GConfEntry *entry,
-			     GaleonWindow *window);
+void go_menu_notifier (GConfClient *client,
+		       guint cnxn_id,
+		       GConfEntry *entry,
+		       GaleonWindow *window);
+
 /*
  * window_menubar_install_notifier: installs the notifiers for one control 
  * in the settings menu
@@ -145,7 +149,6 @@ window_menubar_install_notifier(GaleonWindow *window,
 				GConfClientNotifyFunc notifier_cb,
 				gpointer user_data)
 {	
-	GConfClient* client = eel_gconf_client_get_global();
 	GConfValue* value;
 
 	galeon_notification_add(pref_name, 
@@ -154,11 +157,11 @@ window_menubar_install_notifier(GaleonWindow *window,
 				&window->notifiers_id);
 
 	/* set the control according to the current state */
-	value = gconf_client_get (client, pref_name, NULL);
+	value = eel_gconf_get_value (pref_name);
 	if (value)
 	{
-		gconf_client_set (client, pref_name, value, NULL);
-		gconf_value_free (value);
+		eel_gconf_set_value (pref_name, value);
+		eel_gconf_value_free (value);
 	}
 }
 
@@ -217,10 +220,11 @@ window_notifiers [] =
 	{CONF_TOOLBAR_SPINNER_DIR, main_toolbar_notifier},
 	{CONF_TOOLBAR_THEME_DIR, theme_changed_notifier},
 	{CONF_BOOKMARKS_ENTRY_WIDTH, bookmarks_toolbars_notifier},
-	{CONF_BOOKMARKS_GOTO_BUTTON, bookmarks_toolbars_notifier},
 	{CONF_BOOKMARKS_HISTORY, bookmarks_toolbars_notifier},
 	{CONF_BOOKMARKS_CLEAR, bookmarks_toolbars_notifier},
 	{CONF_BOOKMARKS_XITEMS, bookmarks_menus_notifier},
+	{CONF_BOOKMARKS_TOOLTIPS, bookmarks_menus_notifier},
+	{CONF_BOOKMARKS_TOOLTIPS, bookmarks_toolbars_notifier},
 	{CONF_TABS_TABBED_POSITION, tabs_position_notifier},
 	{CONF_TABS_TABBED_ALWAYS_SHOW, tabs_appeareance_notifier},
 	{CONF_TABS_TABBED_SHORTEN_STYLE, tabs_appeareance_notifier},
@@ -229,6 +233,7 @@ window_notifiers [] =
 	{CONF_TABS_TABBED_CLOSEBUTTON, tabs_appeareance_notifier},
 	{CONF_TABS_TABBED_NEW_COLOR, tabs_appeareance_notifier},
 	{CONF_TABS_TABBED_LOADING_COLOR, tabs_appeareance_notifier},
+	{CONF_TABS_TABBED_FAVICONS, tabs_favicons_notifier},
 	{CONF_BOOKMARKS_AB_COUNT, autobookmarks_notifier},
 	{CONF_BOOKMARKS_AB_SHORTEN, autobookmarks_notifier},
 	{CONF_BOOKMARKS_AB_ENABLED, autobookmarks_notifier},
@@ -237,8 +242,7 @@ window_notifiers [] =
 	{CONF_WINDOWS_FS_SHOW_TOOLBARS, fullscreen_toolbar_notifier},
 	{CONF_WINDOWS_FS_SHOW_STATUSBAR, fullscreen_statusbar_notifier},
 	{CONF_WINDOWS_TITLE, window_title_notifier},
-	{CONF_RENDERING_USER_STYLESHEET, stylesheet_notifier},
-	{"/apps/galeon/updateuihack", update_ui_hack_notifier},
+	{CONF_RENDERING_STYLESHEETS, stylesheet_notifier},
 	{NULL, NULL}
 };
 
@@ -258,14 +262,6 @@ window_init_notifiers(GaleonWindow *window)
 	}
 }
 
-void update_ui_hack_notifier(GConfClient *client,
-			     guint cnxn_id,
-			     GConfEntry *entry,
-			     GaleonWindow *window)
-{
-	window_update_ui_hack (window);
-}
-
 void java_bool_notifier(GConfClient *client,
 			guint cnxn_id,
 			GConfEntry *entry,
@@ -348,7 +344,10 @@ void main_toolbar_notifier(GConfClient *client,
 			   GConfEntry *entry,
 			   GaleonWindow *window)
 {
-	window->to_update |= MAIN_TOOLBAR;
+	window_save_layout (window);
+	toolbar_recreate (window);
+	window_restore_layout (window);
+	link_interfaces_set_sensitivity (window->active_embed);
 }
 
 void bookmarks_toolbars_notifier(GConfClient *client,
@@ -356,7 +355,8 @@ void bookmarks_toolbars_notifier(GConfClient *client,
 				 GConfEntry *entry,
 				 GaleonWindow *window)
 {
-	window->to_update |= BOOKMARKS_TOOLBAR;
+	bookmarks_tb_dirty = TRUE;
+	bookmarks_updated ();
 }
 
 void bookmarks_menus_notifier(GConfClient *client,
@@ -385,7 +385,8 @@ tabs_appeareance_notifier (GConfClient *client,
 			   GConfEntry *entry,
 			   GaleonWindow *window)
 {
-	window->to_update |= TABS_APPEAREANCE;
+	g_print ("tabs");
+	window_update_tabs_appearance (window);
 }
 
 void
@@ -401,7 +402,8 @@ statusbar_appeareance_notifier(GConfClient *client,
 		GaleonWindow *window = (GaleonWindow *) w->data;
 
 		window_statusbar_set_security_icon (window,
-			window->active_embed->secure);
+			window->active_embed->secure,
+			window->active_embed->security_tooltip);
 	}
 }
 
@@ -422,7 +424,7 @@ fullscreen_menubar_notifier (GConfClient *client,
 {
 	gboolean show;
 
-	if (!fullscreen_active) return;
+	if (!window->fullscreen_active) return;
 
 	show = eel_gconf_get_boolean (CONF_WINDOWS_FS_SHOW_MENUBAR);
 	gtk_check_menu_item_set_active 
@@ -437,7 +439,7 @@ fullscreen_toolbar_notifier (GConfClient *client,
 {
 	gboolean show;
 
-	if (!fullscreen_active) return;
+	if (!window->fullscreen_active) return;
 
 	show = eel_gconf_get_boolean (CONF_WINDOWS_FS_SHOW_TOOLBARS);
 	gtk_check_menu_item_set_active 
@@ -452,7 +454,7 @@ fullscreen_bookmarks_notifier (GConfClient *client,
 {
 	gboolean show;
 
-	if (!fullscreen_active) return;
+	if (!window->fullscreen_active) return;
 
 	show = eel_gconf_get_boolean (CONF_WINDOWS_FS_SHOW_BOOKMARKS);
 	gtk_check_menu_item_set_active 
@@ -467,7 +469,7 @@ fullscreen_statusbar_notifier (GConfClient *client,
 {
 	gboolean show;
 
-	if (!fullscreen_active) return;
+	if (!window->fullscreen_active) return;
 
 	show = eel_gconf_get_boolean (CONF_WINDOWS_FS_SHOW_STATUSBAR);
 	gtk_check_menu_item_set_active 
@@ -484,8 +486,7 @@ window_title_notifier (GConfClient *client,
 
 	for (w = all_windows; w != NULL; w = g_list_next (w))
 	{
-		GaleonWindow *window = (GaleonWindow *) (w->data);
-		embed_update_page_title (window->active_embed);
+		window_update_title ((GaleonWindow *) (w->data));
 	}
 }
 
@@ -494,13 +495,15 @@ void theme_changed_notifier (GConfClient *client,
 			     GConfEntry *entry,
 			     GaleonWindow *window)
 {
-	reset_theme_pixmap_cache ();
+	themes_reset_cache ();
 	bookmarks_load_icons ();
 	bookmarks_toolbars_notifier (client, cnxn_id, entry, window);
 	bookmarks_menus_notifier (client, cnxn_id, entry, window);
 	tabs_appeareance_notifier (client, cnxn_id, entry, window);
+	tabs_favicons_notifier (client, cnxn_id, entry, window);
 	statusbar_appeareance_notifier (client, cnxn_id, entry, window);
 	main_toolbar_notifier (client, cnxn_id, entry, window);
+	go_menu_notifier (client, cnxn_id, entry, window);
 }
 
 void proxy_notifier(GConfClient *client,
@@ -533,31 +536,52 @@ void stylesheet_notifier (GConfClient *client,
 			  GConfEntry *entry,
 			  GaleonWindow *window)
 {
-	gchar *usersheet = eel_gconf_get_string
-				(CONF_RENDERING_USER_STYLESHEET);
-	gboolean exists = TRUE;
 	GList *l;
 
-	if (!usersheet || !g_file_exists (usersheet))
-		exists = FALSE;
-
 	for (l = window->embed_list; l; l = g_list_next (l))
 	{
 		GaleonEmbed *embed = (GaleonEmbed *) l->data;
 
-		if (embed->usersheet)
+		if (embed->is_active)
 		{
-			mozilla_remove_user_sheet (embed);
-			embed->usercss_applied = FALSE;
-		}
-		
-		if (exists)
-		{
-			embed->usersheet =
-				mozilla_set_user_sheet (embed, usersheet);
-			embed->usercss_applied = TRUE;
+			stylesheets_menu_build (embed);
 		}
 	}
+}
 
-	if (usersheet) g_free (usersheet);
+void go_menu_notifier (GConfClient *client,
+		       guint cnxn_id,
+		       GConfEntry *entry,
+		       GaleonWindow *window)
+{
+ 	GList *l;
+
+	for (l = gtk_container_children (GTK_CONTAINER (window->go_menu)); l;
+	     l = g_list_next (l))
+	{
+		GtkWidget *p = 
+			gtk_object_get_data (GTK_OBJECT (l->data), "pixmap");
+		HistoryItem *h =
+			gtk_object_get_user_data (GTK_OBJECT (l->data));
+
+		if (p && h)
+		{
+			const PixmapData *d = favicon_get_pixmap (h->url);
+			gtk_pixmap_set (GTK_PIXMAP (p), d->pixmap, d->mask);
+		}
+	}
+}
+
+void
+tabs_favicons_notifier (GConfClient *client, guint cnxn_id, GConfEntry *entry,
+			GaleonWindow *window)
+{
+	GList *l;
+
+	for (l = window->embed_list; l; l = g_list_next (l))
+	{
+		GaleonEmbed *embed = (GaleonEmbed *) l->data;
+
+		favicon_update_drag_handle (embed, FALSE);
+	}	
 }
