@@ -17,4 +17,4 @@
  */
 
 
-gboolean galeon_x_clipboard_present();
+gboolean galeon_x_clipboard_present(void);
